@@ -20,7 +20,6 @@ package org.apache.hadoop.io;
 
 import java.io.IOException;
 import java.nio.charset.StandardCharsets;
-import java.nio.charset.UnsupportedCharsetException;
 import java.util.ArrayList;
 
 import org.apache.commons.codec.binary.Base64;
@@ -75,14 +74,10 @@ public class DefaultStringifier<T> implements Stringifier<T> {
 
   @Override
   public T fromString(String str) throws IOException {
-    try {
-      byte[] bytes = Base64.decodeBase64(str.getBytes("UTF-8"));
-      inBuf.reset(bytes, bytes.length);
-      T restored = deserializer.deserialize(null);
-      return restored;
-    } catch (UnsupportedCharsetException ex) {
-      throw new IOException(ex.toString());
-    }
+    byte[] bytes = Base64.decodeBase64(str.getBytes(StandardCharsets.UTF_8));
+    inBuf.reset(bytes, bytes.length);
+    T restored = deserializer.deserialize(null);
+    return restored;
   }
 
   @Override
