@@ -35,8 +35,10 @@ import org.apache.hadoop.classification.InterfaceStability;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.CommonConfigurationKeys;
 import org.apache.hadoop.fs.HardLink;
+import org.apache.hadoop.fs.PathIOException;
 import org.apache.hadoop.io.IOUtils;
 import org.apache.hadoop.io.SecureIOUtils.AlreadyExistsException;
+import org.apache.hadoop.util.CleanerUtil;
 import org.apache.hadoop.util.NativeCodeLoader;
 import org.apache.hadoop.util.Shell;
 import org.apache.hadoop.util.PerformanceAdvisory;
@@ -45,7 +47,7 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import sun.misc.Unsafe;
 
-import com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.classification.VisibleForTesting;
 
 /**
  * JNI wrappers for various native IO-related calls not available in Java.
@@ -98,6 +100,51 @@ public class NativeIO {
        write.  */
     public static int SYNC_FILE_RANGE_WAIT_AFTER = 4;
 
+    /**
+     * Keeps the support state of PMDK.
+     */
+    public enum SupportState {
+      UNSUPPORTED(-1),
+      PMDK_LIB_NOT_FOUND(1),
+      SUPPORTED(0);
+
+      private byte stateCode;
+      SupportState(int stateCode) {
+        this.stateCode = (byte) stateCode;
+      }
+
+      public int getStateCode() {
+        return stateCode;
+      }
+
+      public String getMessage() {
+        String msg;
+        switch (stateCode) {
+        // -1 represents UNSUPPORTED.
+        case -1:
+          msg = "The native code was built without PMDK support.";
+          break;
+        // 1 represents PMDK_LIB_NOT_FOUND.
+        case 1:
+          msg = "The native code was built with PMDK support, but PMDK libs " +
+              "were NOT found in execution environment or failed to be loaded.";
+          break;
+        // 0 represents SUPPORTED.
+        case 0:
+          msg = "The native code was built with PMDK support, and PMDK libs " +
+              "were loaded successfully.";
+          break;
+        default:
+          msg = "The state code: " + stateCode + " is unrecognized!";
+        }
+        return msg;
+      }
+    }
+
+    // Denotes the state of supporting PMDK. The actual value is set via JNI.
+    private static SupportState pmdkSupportState =
+        SupportState.UNSUPPORTED;
+
     private static final Logger LOG = LoggerFactory.getLogger(NativeIO.class);
 
     // Set to true via JNI if possible
@@ -122,6 +169,109 @@ public class NativeIO {
       POSIX.cacheManipulator = cacheManipulator;
     }
 
+    // This method is invoked by JNI.
+    public static void setPmdkSupportState(int stateCode) {
+      for (SupportState state : SupportState.values()) {
+        if (state.getStateCode() == stateCode) {
+          pmdkSupportState = state;
+          return;
+        }
+      }
+      LOG.error("The state code: " + stateCode + " is unrecognized!");
+    }
+
+    public static String getPmdkSupportStateMessage() {
+      if (getPmdkLibPath() != null) {
+        return pmdkSupportState.getMessage() +
+            " The pmdk lib path: " + getPmdkLibPath();
+      }
+      return pmdkSupportState.getMessage();
+    }
+
+    public static boolean isPmdkAvailable() {
+      LOG.info(pmdkSupportState.getMessage());
+      return pmdkSupportState == SupportState.SUPPORTED;
+    }
+
+    /**
+     * Denote memory region for a file mapped.
+     */
+    public static class PmemMappedRegion {
+      private long address;
+      private long length;
+      private boolean isPmem;
+
+      public PmemMappedRegion(long address, long length, boolean isPmem) {
+        this.address = address;
+        this.length = length;
+        this.isPmem = isPmem;
+      }
+
+      public boolean isPmem() {
+        return this.isPmem;
+      }
+
+      public long getAddress() {
+        return this.address;
+      }
+
+      public long getLength() {
+        return this.length;
+      }
+    }
+
+    /**
+     * JNI wrapper of persist memory operations.
+     */
+    public static class Pmem {
+      // Check whether the address is a Pmem address or DIMM address
+      public static boolean isPmem(long address, long length) {
+        return NativeIO.POSIX.isPmemCheck(address, length);
+      }
+
+      // Map a file in persistent memory, if the given file exists,
+      // directly map it. If not, create the named file on persistent memory
+      // and then map it.
+      public static PmemMappedRegion mapBlock(
+          String path, long length, boolean isFileExist) {
+        return NativeIO.POSIX.pmemMapFile(path, length, isFileExist);
+      }
+
+      // Unmap a pmem file
+      public static boolean unmapBlock(long address, long length) {
+        return NativeIO.POSIX.pmemUnMap(address, length);
+      }
+
+      // Copy data from disk file(src) to pmem file(dest), without flush
+      public static void memCopy(byte[] src, long dest, boolean isPmem,
+          long length) {
+        NativeIO.POSIX.pmemCopy(src, dest, isPmem, length);
+      }
+
+      // Flush the memory content to persistent storage
+      public static void memSync(PmemMappedRegion region) {
+        if (region.isPmem()) {
+          NativeIO.POSIX.pmemDrain();
+        } else {
+          NativeIO.POSIX.pmemSync(region.getAddress(), region.getLength());
+        }
+      }
+
+      public static String getPmdkLibPath() {
+        return POSIX.getPmdkLibPath();
+      }
+    }
+
+    private static native String getPmdkLibPath();
+    private static native boolean isPmemCheck(long address, long length);
+    private static native PmemMappedRegion pmemMapFile(String path,
+        long length, boolean isFileExist);
+    private static native boolean pmemUnMap(long address, long length);
+    private static native void pmemCopy(byte[] src, long dest, boolean isPmem,
+        long length);
+    private static native void pmemDrain();
+    private static native void pmemSync(long address, long length);
+
     /**
      * Used to manipulate the operating system cache.
      */
@@ -141,8 +291,8 @@ public class NativeIO {
       }
 
       public void posixFadviseIfPossible(String identifier,
-        FileDescriptor fd, long offset, long len, int flags)
-            throws NativeIOException {
+          FileDescriptor fd, long offset, long len, int flags)
+          throws NativeIOException {
         NativeIO.POSIX.posixFadviseIfPossible(identifier, fd, offset,
             len, flags);
       }
@@ -205,7 +355,7 @@ public class NativeIO {
     }
 
     /**
-     * Return true if the JNI-based native IO extensions are available.
+     * @return Return true if the JNI-based native IO extensions are available.
      */
     public static boolean isAvailable() {
       return NativeCodeLoader.isNativeCodeLoaded() && nativeLoaded;
@@ -217,10 +367,19 @@ public class NativeIO {
       }
     }
 
-    /** Wrapper around open(2) */
+    /**
+     * Wrapper around open(2) .
+     * @param path input path.
+     * @param flags input flags.
+     * @param mode input mode.
+     * @return FileDescriptor.
+     * @throws IOException raised on errors performing I/O.
+     */
     public static native FileDescriptor open(String path, int flags, int mode) throws IOException;
     /** Wrapper around fstat(2) */
     private static native Stat fstat(FileDescriptor fd) throws IOException;
+    /** Wrapper around stat(2). */
+    private static native Stat stat(String path) throws IOException;
 
     /** Native chmod implementation. On UNIX, it is a wrapper around chmod(2) */
     private static native void chmodImpl(String path, int mode) throws IOException;
@@ -276,6 +435,10 @@ public class NativeIO {
      * for this syscall for more information. On systems where this
      * call is not available, does nothing.
      *
+     * @param fd input fd.
+     * @param offset input offset.
+     * @param nbytes input nbytes.
+     * @param flags input flag.
      * @throws NativeIOException if there is an error with the syscall
      */
     public static void syncFileRangeIfPossible(
@@ -312,7 +475,7 @@ public class NativeIO {
       }
       mlock_native(buffer, len);
     }
-    
+
     /**
      * Unmaps the block from memory. See munmap(2).
      *
@@ -326,10 +489,14 @@ public class NativeIO {
      * @param buffer    The buffer to unmap.
      */
     public static void munmap(MappedByteBuffer buffer) {
-      if (buffer instanceof sun.nio.ch.DirectBuffer) {
-        sun.misc.Cleaner cleaner =
-            ((sun.nio.ch.DirectBuffer)buffer).cleaner();
-        cleaner.clean();
+      if (CleanerUtil.UNMAP_SUPPORTED) {
+        try {
+          CleanerUtil.getCleaner().freeBuffer(buffer);
+        } catch (IOException e) {
+          LOG.info("Failed to unmap the buffer", e);
+        }
+      } else {
+        LOG.trace(CleanerUtil.UNMAP_NOT_SUPPORTED_REASON);
       }
     }
 
@@ -428,6 +595,37 @@ public class NativeIO {
       return stat;
     }
 
+    /**
+     * Return the file stat for a file path.
+     *
+     * @param path  file path
+     * @return  the file stat
+     * @throws IOException  thrown if there is an IO error while obtaining the
+     * file stat
+     */
+    public static Stat getStat(String path) throws IOException {
+      if (path == null) {
+        String errMessage = "Path is null";
+        LOG.warn(errMessage);
+        throw new IOException(errMessage);
+      }
+      Stat stat = null;
+      try {
+        if (!Shell.WINDOWS) {
+          stat = stat(path);
+          stat.owner = getName(IdCache.USER, stat.ownerId);
+          stat.group = getName(IdCache.GROUP, stat.groupId);
+        } else {
+          stat = stat(path);
+        }
+      } catch (NativeIOException nioe) {
+        LOG.warn("NativeIO.getStat error ({}): {} -- file path: {}",
+            nioe.getErrorCode(), nioe.getMessage(), path);
+        throw new PathIOException(path, nioe);
+      }
+      return stat;
+    }
+
     private static String getName(IdCache domain, int id) throws IOException {
       Map<Integer, CachedName> idNameCache = (domain == IdCache.USER)
         ? USER_ID_NAME_CACHE : GROUP_ID_NAME_CACHE;
@@ -525,7 +723,14 @@ public class NativeIO {
     private static native void createDirectoryWithMode0(String path, int mode)
         throws NativeIOException;
 
-    /** Wrapper around CreateFile() on Windows */
+    /**
+     * @return Wrapper around CreateFile() on Windows.
+     * @param path input path.
+     * @param desiredAccess input desiredAccess.
+     * @param shareMode input shareMode.
+     * @param creationDisposition input creationDisposition.
+     * @throws IOException raised on errors performing I/O.
+     */
     public static native FileDescriptor createFile(String path,
         long desiredAccess, long shareMode, long creationDisposition)
         throws IOException;
@@ -562,7 +767,13 @@ public class NativeIO {
         long desiredAccess, long shareMode, long creationDisposition, int mode)
         throws NativeIOException;
 
-    /** Wrapper around SetFilePointer() on Windows */
+    /**
+     * @return Wrapper around SetFilePointer() on Windows.
+     * @param fd input fd.
+     * @param distanceToMove input distanceToMove.
+     * @param moveMethod input moveMethod.
+     * @throws IOException raised on errors performing I/O.
+     */
     public static native long setFilePointer(FileDescriptor fd,
         long distanceToMove, long moveMethod) throws IOException;
 
@@ -570,7 +781,7 @@ public class NativeIO {
     private static native String getOwner(FileDescriptor fd) throws IOException;
 
     /** Supported list of Windows access right flags */
-    public static enum AccessRight {
+    public enum AccessRight {
       ACCESS_READ (0x0001),      // FILE_READ_DATA
       ACCESS_WRITE (0x0002),     // FILE_WRITE_DATA
       ACCESS_EXECUTE (0x0020);   // FILE_EXECUTE
@@ -653,7 +864,7 @@ public class NativeIO {
   }
 
   /**
-   * Return true if the JNI-based native IO extensions are available.
+   * @return Return true if the JNI-based native IO extensions are available.
    */
   public static boolean isAvailable() {
     return NativeCodeLoader.isNativeCodeLoaded() && nativeLoaded;
@@ -709,8 +920,9 @@ public class NativeIO {
    * user account name, of the format DOMAIN\UserName. This method
    * will remove the domain part of the full logon name.
    *
-   * @param Fthe full principal name containing the domain
+   * @param name the full principal name containing the domain
    * @return name with domain removed
+   * @throws IOException raised on errors performing I/O.
    */
   private static String stripDomain(String name) {
     int i = name.indexOf('\\');
@@ -746,6 +958,11 @@ public class NativeIO {
    * file opened at a given offset, i.e. other process can delete
    * the file the FileDescriptor is reading. Only Windows implementation
    * uses the native interface.
+   *
+   * @param f input f.
+   * @param seekOffset input seekOffset.
+   * @return FileDescriptor.
+   * @throws IOException raised on errors performing I/O.
    */
   public static FileDescriptor getShareDeleteFileDescriptor(
       File f, long seekOffset) throws IOException {
@@ -756,7 +973,7 @@ public class NativeIO {
       }
       return rf.getFD();
     } else {
-      // Use Windows native interface to create a FileDescriptor that
+      // Use Windows native interface to create a FileInputStream that
       // shares delete permission on the file opened, and set it to the
       // given offset.
       //
@@ -774,7 +991,7 @@ public class NativeIO {
   }
 
   /**
-   * Create the specified File for write access, ensuring that it does not exist.
+   * @return Create the specified File for write access, ensuring that it does not exist.
    * @param f the file that we want to create
    * @param permissions we want to have on the file (if security is enabled)
    *
@@ -858,7 +1075,7 @@ public class NativeIO {
    *
    * @param src source file
    * @param dst hardlink location
-   * @throws IOException
+   * @throws IOException raised on errors performing I/O.
    */
   @Deprecated
   public static void link(File src, File dst) throws IOException {
@@ -916,7 +1133,7 @@ public class NativeIO {
    *
    * @param src                  The source path
    * @param dst                  The destination path
-   * @throws IOException
+   * @throws IOException raised on errors performing I/O.
    */
   public static void copyFileUnbuffered(File src, File dst) throws IOException {
     if (nativeLoaded && Shell.WINDOWS) {
