@@ -47,8 +47,10 @@ public final class CallerContext {
   // field names
   public static final String CLIENT_IP_STR = "clientIp";
   public static final String CLIENT_PORT_STR = "clientPort";
+  public static final String CLIENT_ID_STR = "clientId";
+  public static final String CLIENT_CALL_ID_STR = "clientCallId";
   public static final String REAL_USER_STR = "realUser";
-
+  public static final String PROXY_USER_PORT = "proxyUserPort";
   /** The caller context.
    *
    * It will be truncated if it exceeds the maximum allowed length in
@@ -139,12 +141,8 @@ public final class CallerContext {
     }
 
     public Builder(String context, Configuration conf) {
-      if (isValid(context)) {
-        sb.append(context);
-      }
-      fieldSeparator = conf.get(HADOOP_CALLER_CONTEXT_SEPARATOR_KEY,
-          HADOOP_CALLER_CONTEXT_SEPARATOR_DEFAULT);
-      checkFieldSeparator(fieldSeparator);
+      this(context, conf.get(HADOOP_CALLER_CONTEXT_SEPARATOR_KEY,
+              HADOOP_CALLER_CONTEXT_SEPARATOR_DEFAULT));
     }
 
     public Builder(String context, String separator) {
