@@ -21,8 +21,7 @@ import java.nio.ByteBuffer;
 import java.util.UUID;
 
 import org.apache.hadoop.classification.InterfaceAudience;
-
-import org.apache.hadoop.thirdparty.com.google.common.base.Preconditions;
+import org.apache.hadoop.util.Preconditions;
 
 /**
  * A class defining a set of static helper methods to provide conversion between
