@@ -19,6 +19,7 @@ package org.apache.hadoop.util;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.charset.StandardCharsets;
 import java.util.List;
 
 import org.apache.hadoop.classification.InterfaceAudience;
@@ -27,10 +28,8 @@ import org.apache.zookeeper.ZooDefs;
 import org.apache.zookeeper.data.ACL;
 import org.apache.zookeeper.data.Id;
 
-import com.google.common.base.Charsets;
-import com.google.common.base.Splitter;
-import com.google.common.collect.Lists;
-import com.google.common.io.Files;
+import org.apache.hadoop.thirdparty.com.google.common.base.Splitter;
+import org.apache.hadoop.thirdparty.com.google.common.io.Files;
 
 /**
  * Utilities for working with ZooKeeper.
@@ -89,8 +88,9 @@ public class ZKUtil {
    * Parse comma separated list of ACL entries to secure generated nodes, e.g.
    * <code>sasl:hdfs/host1@MY.DOMAIN:cdrwa,sasl:hdfs/host2@MY.DOMAIN:cdrwa</code>
    *
+   * @param aclString aclString.
    * @return ACL list
-   * @throws {@link BadAclFormatException} if an ACL is invalid
+   * @throws BadAclFormatException if an ACL is invalid
    */
   public static List<ACL> parseACLs(String aclString) throws
       BadAclFormatException {
@@ -128,7 +128,7 @@ public class ZKUtil {
    * 
    * @param authString the comma-separated auth mechanisms
    * @return a list of parsed authentications
-   * @throws {@link BadAuthFormatException} if the auth format is invalid
+   * @throws BadAuthFormatException if the auth format is invalid
    */
   public static List<ZKAuthInfo> parseAuth(String authString) throws
       BadAuthFormatException{
@@ -148,7 +148,7 @@ public class ZKUtil {
             "Auth '" + comp + "' not of expected form scheme:auth");
       }
       ret.add(new ZKAuthInfo(parts[0],
-          parts[1].getBytes(Charsets.UTF_8)));
+          parts[1].getBytes(StandardCharsets.UTF_8)));
     }
     return ret;
   }
@@ -172,7 +172,7 @@ public class ZKUtil {
       return valInConf;
     }
     String path = valInConf.substring(1).trim();
-    return Files.asCharSource(new File(path), Charsets.UTF_8).read().trim();
+    return Files.asCharSource(new File(path), StandardCharsets.UTF_8).read().trim();
   }
 
   /**
