@@ -25,6 +25,7 @@ import java.security.SecureRandom;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
+import java.util.concurrent.TimeUnit;
 
 import javax.crypto.Cipher;
 import javax.crypto.spec.IvParameterSpec;
@@ -58,7 +59,7 @@ public class TestKeyProviderCryptoExtension {
   private static KeyVersion encryptionKey;
 
   @Rule
-  public Timeout testTimeout = new Timeout(180000);
+  public Timeout testTimeout = new Timeout(180000, TimeUnit.MILLISECONDS);
 
   @BeforeClass
   public static void setup() throws Exception {
