@@ -26,6 +26,7 @@ import org.apache.hadoop.test.GenericTestUtils;
 import org.junit.Assert;
 
 import org.apache.hadoop.fs.permission.FsPermission;
+import org.apache.hadoop.test.LambdaTestUtils;
 import org.apache.hadoop.util.Shell;
 import org.junit.After;
 import org.junit.Before;
@@ -129,8 +130,8 @@ public abstract class FileContextURIBase {
   }
 
   @Test
-  public void testCreateExistingFile() throws IOException {
-    String fileName = "testFile";
+  public void testCreateExistingFile() throws Exception {
+    String fileName = "testCreateExistingFile";
     Path testPath = qualifiedPath(fileName, fc2);
 
     // Ensure file does not exist
@@ -140,20 +141,16 @@ public abstract class FileContextURIBase {
     createFile(fc1, testPath);
 
     // Create same file with fc1
-    try {
-      createFile(fc2, testPath);
-      Assert.fail("Create existing file should throw an IOException.");
-    } catch (IOException e) {
-      // expected
-    }
+    LambdaTestUtils.intercept(IOException.class, () ->
+        createFile(fc2, testPath));
 
     // Ensure fc2 has the created file
-    Assert.assertTrue(exists(fc2, testPath));
+    fc2.getFileStatus(testPath);
   }
 
   @Test
   public void testCreateFileInNonExistingDirectory() throws IOException {
-    String fileName = "testDir/testFile";
+    String fileName = "testCreateFileInNonExistingDirectory/testFile";
 
     Path testPath = qualifiedPath(fileName, fc2);
 
@@ -165,8 +162,9 @@ public abstract class FileContextURIBase {
 
     // Ensure using fc2 that file is created
     Assert.assertTrue(isDir(fc2, testPath.getParent()));
-    Assert.assertEquals("testDir", testPath.getParent().getName());
-    Assert.assertTrue(exists(fc2, testPath));
+    Assert.assertEquals("testCreateFileInNonExistingDirectory",
+        testPath.getParent().getName());
+    fc2.getFileStatus(testPath);
 
   }
 
@@ -215,10 +213,11 @@ public abstract class FileContextURIBase {
     // TestCase - Create multiple directories
     String dirNames[] = { 
         "createTest/testDir", "createTest/test Dir",
-        "deleteTest/test*Dir", "deleteTest/test#Dir", 
-        "deleteTest/test1234", "deleteTest/test_DIr", 
-        "deleteTest/1234Test", "deleteTest/test)Dir",
-        "deleteTest/()&^%$#@!~_+}{><?", "  ", "^ " };
+        "createTest/test*Dir", "createTest/test#Dir",
+        "createTest/test1234", "createTest/test_DIr",
+        "createTest/1234Test", "createTest/test)Dir",
+        "createTest/()&^%$#@!~_+}{><?",
+        "createTest/  ", "createTest/^ " };
 
     for (String f : dirNames) {
       if (!isTestableFileNameOnPlatform(f)) {
@@ -236,6 +235,10 @@ public abstract class FileContextURIBase {
       Assert.assertTrue(exists(fc2, testPath));
       Assert.assertTrue(isDir(fc2, testPath));
     }
+    // delete the parent directory and verify that the dir no longer exists
+    final Path parent = qualifiedPath("createTest", fc2);
+    fc2.delete(parent, true);
+    Assert.assertFalse(exists(fc2, parent));
 
   }
 
@@ -293,7 +296,7 @@ public abstract class FileContextURIBase {
 
   @Test
   public void testDeleteFile() throws IOException {
-    Path testPath = qualifiedPath("testFile", fc2);
+    Path testPath = qualifiedPath("testDeleteFile", fc2);
 
     // Ensure file does not exist
     Assert.assertFalse(exists(fc2, testPath));
@@ -314,7 +317,7 @@ public abstract class FileContextURIBase {
 
   @Test
   public void testDeleteNonExistingFile() throws IOException {
-    String testFileName = "testFile";
+    String testFileName = "testDeleteNonExistingFile";
     Path testPath = qualifiedPath(testFileName, fc2);
 
     // TestCase1 : Test delete on file never existed
@@ -341,7 +344,7 @@ public abstract class FileContextURIBase {
 
   @Test
   public void testDeleteNonExistingFileInDir() throws IOException {
-    String testFileInDir = "testDir/testDir/TestFile";
+    String testFileInDir = "testDeleteNonExistingFileInDir/testDir/TestFile";
     Path testPath = qualifiedPath(testFileInDir, fc2);
 
     // TestCase1 : Test delete on file never existed
@@ -391,7 +394,9 @@ public abstract class FileContextURIBase {
         "deleteTest/test*Dir", "deleteTest/test#Dir", 
         "deleteTest/test1234", "deleteTest/1234Test", 
         "deleteTest/test)Dir", "deleteTest/test_DIr",
-        "deleteTest/()&^%$#@!~_+}{><?", "  ", "^ " };
+        "deleteTest/()&^%$#@!~_+}{><?",
+        "deleteTest/  ",
+        "deleteTest/^ " };
 
     for (String f : dirNames) {
       if (!isTestableFileNameOnPlatform(f)) {
@@ -418,7 +423,7 @@ public abstract class FileContextURIBase {
 
   @Test
   public void testDeleteNonExistingDirectory() throws IOException {
-    String testDirName = "testFile";
+    String testDirName = "testDeleteNonExistingDirectory";
     Path testPath = qualifiedPath(testDirName, fc2);
 
     // TestCase1 : Test delete on directory never existed
@@ -445,7 +450,7 @@ public abstract class FileContextURIBase {
 
   @Test
   public void testModificationTime() throws IOException {
-    String testFile = "file1";
+    String testFile = "testModificationTime";
     long fc2ModificationTime, fc1ModificationTime;
 
     Path testPath = qualifiedPath(testFile, fc2);
@@ -461,7 +466,7 @@ public abstract class FileContextURIBase {
 
   @Test
   public void testFileStatus() throws IOException {
-    String fileName = "file1";
+    String fileName = "testModificationTime";
     Path path2 = fc2.makeQualified(new Path(BASE, fileName));
 
     // Create a file on fc2's file system using fc1
