@@ -18,9 +18,8 @@
 
 package org.apache.hadoop.util;
 
+import static org.apache.hadoop.test.PlatformAssumptions.assumeWindows;
 import static org.junit.Assert.*;
-import static org.junit.Assume.assumeTrue;
-import static org.junit.matchers.JUnitMatchers.containsString;
 
 import java.io.File;
 import java.io.FileInputStream;
@@ -31,14 +30,13 @@ import java.io.IOException;
 import org.apache.commons.io.FileUtils;
 import org.apache.hadoop.fs.FileUtil;
 import org.apache.hadoop.test.GenericTestUtils;
+import org.assertj.core.api.Assertions;
 import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import static org.hamcrest.CoreMatchers.*;
-
 /**
  * Test cases for helper Windows winutils.exe utility.
  */
@@ -53,7 +51,7 @@ public class TestWinUtils {
   @Before
   public void setUp() throws IOException {
     // Not supported on non-Windows platforms
-    assumeTrue(Shell.WINDOWS);
+    assumeWindows();
     TEST_DIR.mkdirs();
     assertTrue("Failed to create Test directory " + TEST_DIR,
         TEST_DIR.isDirectory() );
@@ -496,12 +494,12 @@ public class TestWinUtils {
     String readLinkOutput = Shell.execCommand(winutils,
         "readlink",
         dirLink.toString());
-    assertThat(readLinkOutput, equalTo(dir1.toString()));
+    Assertions.assertThat(readLinkOutput).isEqualTo(dir1.toString());
 
     readLinkOutput = Shell.execCommand(winutils,
         "readlink",
         fileLink.toString());
-    assertThat(readLinkOutput, equalTo(file1.toString()));
+    Assertions.assertThat(readLinkOutput).isEqualTo(file1.toString());
 
     // Try a few invalid inputs and verify we get an ExitCodeException for each.
     //
@@ -511,7 +509,7 @@ public class TestWinUtils {
       Shell.execCommand(winutils, "readlink", "");
       fail("Failed to get Shell.ExitCodeException when reading bad symlink");
     } catch (Shell.ExitCodeException ece) {
-      assertThat(ece.getExitCode(), is(1));
+      Assertions.assertThat(ece.getExitCode()).isEqualTo(1);
     }
 
     try {
@@ -520,7 +518,7 @@ public class TestWinUtils {
       Shell.execCommand(winutils, "readlink", "ThereIsNoSuchLink");
       fail("Failed to get Shell.ExitCodeException when reading bad symlink");
     } catch (Shell.ExitCodeException ece) {
-      assertThat(ece.getExitCode(), is(1));
+      Assertions.assertThat(ece.getExitCode()).isEqualTo(1);
     }
 
     try {
@@ -529,7 +527,7 @@ public class TestWinUtils {
       Shell.execCommand(winutils, "readlink", dir1.toString());
       fail("Failed to get Shell.ExitCodeException when reading bad symlink");
     } catch (Shell.ExitCodeException ece) {
-      assertThat(ece.getExitCode(), is(1));
+      Assertions.assertThat(ece.getExitCode()).isEqualTo(1);
     }
 
     try {
@@ -538,7 +536,7 @@ public class TestWinUtils {
       Shell.execCommand(winutils, "readlink", file1.toString());
       fail("Failed to get Shell.ExitCodeException when reading bad symlink");
     } catch (Shell.ExitCodeException ece) {
-      assertThat(ece.getExitCode(), is(1));
+      Assertions.assertThat(ece.getExitCode()).isEqualTo(1);
     }
 
     try {
@@ -547,11 +545,10 @@ public class TestWinUtils {
       Shell.execCommand(winutils, "readlink", "a", "b");
       fail("Failed to get Shell.ExitCodeException with bad parameters");
     } catch (Shell.ExitCodeException ece) {
-      assertThat(ece.getExitCode(), is(1));
+      Assertions.assertThat(ece.getExitCode()).isEqualTo(1);
     }
   }
   
-  @SuppressWarnings("deprecation")
   @Test(timeout=10000)
   public void testTaskCreate() throws IOException {
     requireWinutils();
@@ -570,8 +567,8 @@ public class TestWinUtils {
     assertTrue(proof.exists());
     
     String outNumber = FileUtils.readFileToString(proof);
-    
-    assertThat(outNumber, containsString(testNumber));
+
+    Assertions.assertThat(outNumber).contains(testNumber);
   }
 
   @Test (timeout = 30000)
@@ -604,7 +601,7 @@ public class TestWinUtils {
           + jobId, "java -Xmx256m -version");
       fail("Failed to get Shell.ExitCodeException with insufficient memory");
     } catch (Shell.ExitCodeException ece) {
-      assertThat(ece.getExitCode(), is(1));
+      Assertions.assertThat(ece.getExitCode()).isEqualTo(1);
     }
 
     // Run tasks with wrong parameters
@@ -615,7 +612,7 @@ public class TestWinUtils {
           "-1", "foo", "job" + jobId, "cmd /c echo job" + jobId);
       fail("Failed to get Shell.ExitCodeException with bad parameters");
     } catch (Shell.ExitCodeException ece) {
-      assertThat(ece.getExitCode(), is(1639));
+      Assertions.assertThat(ece.getExitCode()).isEqualTo(1639);
     }
 
     try {
@@ -624,7 +621,7 @@ public class TestWinUtils {
           "job" + jobId, "cmd /c echo job" + jobId);
       fail("Failed to get Shell.ExitCodeException with bad parameters");
     } catch (Shell.ExitCodeException ece) {
-      assertThat(ece.getExitCode(), is(1639));
+      Assertions.assertThat(ece.getExitCode()).isEqualTo(1639);
     }
 
     try {
@@ -633,7 +630,7 @@ public class TestWinUtils {
           "job" + jobId, "cmd /c echo job" + jobId);
       fail("Failed to get Shell.ExitCodeException with bad parameters");
     } catch (Shell.ExitCodeException ece) {
-      assertThat(ece.getExitCode(), is(1639));
+      Assertions.assertThat(ece.getExitCode()).isEqualTo(1639);
     }
   }
 }
