@@ -20,8 +20,6 @@ package org.apache.hadoop.hdfs.web;
 import org.apache.hadoop.hdfs.client.HdfsClientConfigKeys;
 import org.apache.hadoop.io.Text;
 
-import com.google.common.annotations.VisibleForTesting;
-
 public class SWebHdfsFileSystem extends WebHdfsFileSystem {
 
   @Override
@@ -39,10 +37,8 @@ public class SWebHdfsFileSystem extends WebHdfsFileSystem {
     return WebHdfsConstants.SWEBHDFS_TOKEN_KIND;
   }
 
-  @VisibleForTesting
   @Override
-  public int getDefaultPort() {
-    return getConf().getInt(HdfsClientConfigKeys.DFS_NAMENODE_HTTPS_PORT_KEY,
-        HdfsClientConfigKeys.DFS_NAMENODE_HTTPS_PORT_DEFAULT);
+  protected int getDefaultPort() {
+    return HdfsClientConfigKeys.DFS_NAMENODE_HTTPS_PORT_DEFAULT;
   }
 }
