@@ -30,9 +30,15 @@ import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.hdfs.DFSClient;
 import org.apache.hadoop.hdfs.nfs.conf.NfsConfiguration;
 import org.apache.hadoop.security.UserGroupInformation;
+import org.junit.After;
 import org.junit.Test;
 
 public class TestDFSClientCache {
+  @After
+  public void cleanup() {
+    UserGroupInformation.reset();
+  }
+
   @Test
   public void testEviction() throws IOException {
     NfsConfiguration conf = new NfsConfiguration();
