@@ -42,8 +42,8 @@ import org.apache.hadoop.security.AccessControlException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.google.common.collect.ArrayListMultimap;
-import com.google.common.collect.ListMultimap;
+import org.apache.hadoop.thirdparty.com.google.common.collect.ArrayListMultimap;
+import org.apache.hadoop.thirdparty.com.google.common.collect.ListMultimap;
 
 /**
  * Module that implements the quota relevant RPC calls
@@ -75,11 +75,14 @@ public class Quota {
    * @param storagespaceQuota Storage space quota.
    * @param type StorageType that the space quota is intended to be set on.
    * @param checkMountEntry whether to check the path is a mount entry.
-   * @throws AccessControlException If the quota system is disabled or if
+   * @throws IOException If the quota system is disabled or if
    * checkMountEntry is true and the path is a mount entry.
    */
   public void setQuota(String path, long namespaceQuota, long storagespaceQuota,
       StorageType type, boolean checkMountEntry) throws IOException {
+    if (!router.isQuotaEnabled()) {
+      throw new IOException("The quota system is disabled in Router.");
+    }
     if (checkMountEntry && isMountEntry(path)) {
       throw new AccessControlException(
           "Permission denied: " + RouterRpcServer.getRemoteUser()
@@ -101,9 +104,6 @@ public class Quota {
       long namespaceQuota, long storagespaceQuota, StorageType type)
       throws IOException {
     rpcServer.checkOperation(OperationCategory.WRITE);
-    if (!router.isQuotaEnabled()) {
-      throw new IOException("The quota system is disabled in Router.");
-    }
 
     // Set quota for current path and its children mount table path.
     if (locations == null) {
@@ -328,7 +328,9 @@ public class Quota {
 
   /**
    * Invoke predicate by each storage type and bitwise inclusive OR the results.
+   *
    * @param predicate the function test the storage type.
+   * @return true if bitwise OR by all storage type returns true, false otherwise.
    */
   public static boolean orByStorageType(Predicate<StorageType> predicate) {
     boolean res = false;
@@ -340,10 +342,12 @@ public class Quota {
 
   /**
    * Invoke predicate by each storage type and bitwise AND the results.
+   *
    * @param predicate the function test the storage type.
+   * @return true if bitwise AND by all storage type returns true, false otherwise.
    */
   public static boolean andByStorageType(Predicate<StorageType> predicate) {
-    boolean res = false;
+    boolean res = true;
     for (StorageType type : StorageType.values()) {
       res &= predicate.test(type);
     }
