@@ -23,6 +23,8 @@ import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
 
+import org.apache.hadoop.classification.InterfaceAudience;
+import org.apache.hadoop.classification.InterfaceStability;
 import org.apache.hadoop.hdfs.server.federation.store.StateStoreUtils;
 import org.apache.hadoop.hdfs.server.federation.store.driver.StateStoreDriver;
 import org.apache.hadoop.hdfs.server.federation.store.records.BaseRecord;
@@ -39,6 +41,8 @@ import org.apache.hadoop.hdfs.server.federation.store.records.QueryResult;
  * optimization, such as custom get/put/remove queries, depending on the
  * capabilities of the data store.
  */
+@InterfaceAudience.Public
+@InterfaceStability.Evolving
 public abstract class StateStoreBaseImpl extends StateStoreDriver {
 
   @Override
@@ -71,7 +75,7 @@ public abstract class StateStoreBaseImpl extends StateStoreDriver {
       T record, boolean allowUpdate, boolean errorIfExists) throws IOException {
     List<T> singletonList = new ArrayList<>();
     singletonList.add(record);
-    return putAll(singletonList, allowUpdate, errorIfExists);
+    return putAll(singletonList, allowUpdate, errorIfExists).isOperationSuccessful();
   }
 
   @Override
