@@ -38,7 +38,7 @@ import org.apache.hadoop.hdfs.server.namenode.FileJournalManager;
 import org.apache.hadoop.hdfs.server.namenode.NNStorage;
 import org.apache.hadoop.hdfs.server.protocol.NamespaceInfo;
 
-import com.google.common.collect.ImmutableList;
+import org.apache.hadoop.thirdparty.com.google.common.collect.ImmutableList;
 
 /**
  * A {@link Storage} implementation for the {@link JournalNode}.
@@ -123,10 +123,6 @@ class JNStorage extends Storage {
     return new File(sd.getCurrentDir(), name);
   }
 
-  File getCurrentDir() {
-    return sd.getCurrentDir();
-  }
-
   /**
    * Directory {@code edits.sync} temporarily holds the log segments
    * downloaded through {@link JournalNodeSyncer} before they are moved to
