@@ -72,7 +72,7 @@ import org.apache.hadoop.hdfs.server.namenode.startupprogress.Step;
 import org.apache.hadoop.hdfs.util.EnumCounters;
 import org.apache.hadoop.hdfs.util.ReadOnlyList;
 
-import org.apache.hadoop.thirdparty.com.google.common.base.Preconditions;
+import org.apache.hadoop.util.Preconditions;
 import org.apache.hadoop.thirdparty.com.google.common.collect.ImmutableList;
 import org.apache.hadoop.thirdparty.protobuf.ByteString;
 
@@ -423,6 +423,7 @@ public final class FSImageFormatPBINode {
       CountDownLatch latch = new CountDownLatch(sections.size());
       AtomicInteger totalLoaded = new AtomicInteger(0);
       final List<IOException> exceptions = Collections.synchronizedList(new ArrayList<>());
+      Counter counter = prog.getCounter(Phase.LOADING_FSIMAGE, currentStep);
 
       for (int i=0; i < sections.size(); i++) {
         FileSummary.Section s = sections.get(i);
@@ -433,9 +434,7 @@ public final class FSImageFormatPBINode {
         }
         service.submit(() -> {
           try {
-            totalLoaded.addAndGet(loadINodesInSection(ins, null));
-            prog.setCount(Phase.LOADING_FSIMAGE, currentStep,
-                totalLoaded.get());
+            totalLoaded.addAndGet(loadINodesInSection(ins, counter));
           } catch (Exception e) {
             LOG.error("An exception occurred loading INodes in parallel", e);
             exceptions.add(new IOException(e));
