@@ -307,6 +307,7 @@ public class DebugAdmin extends Configured implements Tool {
                 smallBufferSize));
         BlockMetadataHeader.writeHeader(metaOut, checksum);
         metaOut.close();
+        metaOut = null;
         FsDatasetUtil.computeChecksum(
             srcMeta, srcMeta, blockFile, smallBufferSize, conf);
         System.out.println(
@@ -432,8 +433,13 @@ public class DebugAdmin extends Configured implements Tool {
 
     VerifyECCommand() {
       super("verifyEC",
-          "verifyEC -file <file>",
-          "  Verify HDFS erasure coding on all block groups of the file.");
+          "verifyEC -file <file> [-blockId <blk_Id>] [-skipFailureBlocks]",
+          "  -file Verify HDFS erasure coding on all block groups of the file." +
+              System.lineSeparator() +
+          "  -skipFailureBlocks specify will skip any block group failures during verify," +
+          "  and continues verify all block groups of the file," + System.lineSeparator() +
+          "  the default is not to skip failure blocks." + System.lineSeparator() +
+          "  -blockId specify blk_Id to verify for a specific one block group.");
     }
 
     int run(List<String> args) throws IOException {
@@ -480,30 +486,48 @@ public class DebugAdmin extends Configured implements Tool {
       this.parityBlkNum = ecPolicy.getNumParityUnits();
       this.cellSize = ecPolicy.getCellSize();
       this.encoder = CodecUtil.createRawEncoder(getConf(), ecPolicy.getCodecName(),
-          new ErasureCoderOptions(
-              ecPolicy.getNumDataUnits(), ecPolicy.getNumParityUnits()));
+          new ErasureCoderOptions(dataBlkNum, parityBlkNum));
       int blockNum = dataBlkNum + parityBlkNum;
       this.readService = new ExecutorCompletionService<>(
           DFSUtilClient.getThreadPoolExecutor(blockNum, blockNum, 60,
               new LinkedBlockingQueue<>(), "read-", false));
-      this.blockReaders = new BlockReader[dataBlkNum + parityBlkNum];
+      this.blockReaders = new BlockReader[blockNum];
+
+      String needToVerifyBlockId = StringUtils.popOptionWithArgument("-blockId", args);
+      boolean skipFailureBlocks = StringUtils.popOption("-skipFailureBlocks", args);
+      boolean isHealthy = true;
 
       for (LocatedBlock locatedBlock : locatedBlocks.getLocatedBlocks()) {
-        System.out.println("Checking EC block group: blk_" + locatedBlock.getBlock().getBlockId());
-        LocatedStripedBlock blockGroup = (LocatedStripedBlock) locatedBlock;
+        String blockName = locatedBlock.getBlock().getBlockName();
+        if (needToVerifyBlockId == null || needToVerifyBlockId.equals(blockName)) {
+          System.out.println("Checking EC block group: " + blockName);
+          LocatedStripedBlock blockGroup = (LocatedStripedBlock) locatedBlock;
 
-        try {
-          verifyBlockGroup(blockGroup);
-          System.out.println("Status: OK");
-        } catch (Exception e) {
-          System.err.println("Status: ERROR, message: " + e.getMessage());
-          return 1;
-        } finally {
-          closeBlockReaders();
+          try {
+            verifyBlockGroup(blockGroup);
+            System.out.println("Status: OK");
+          } catch (Exception e) {
+            System.err.println("Status: ERROR, message: " + e.getMessage());
+            isHealthy = false;
+            if (!skipFailureBlocks) {
+              break;
+            }
+          } finally {
+            closeBlockReaders();
+          }
+
+          if (needToVerifyBlockId != null) {
+            break;
+          }
         }
       }
-      System.out.println("\nAll EC block group status: OK");
-      return 0;
+      if (isHealthy) {
+        if (needToVerifyBlockId == null) {
+          System.out.println("\nAll EC block group status: OK");
+        }
+        return 0;
+      }
+      return 1;
     }
 
     private void verifyBlockGroup(LocatedStripedBlock blockGroup) throws Exception {
