@@ -328,7 +328,7 @@ public class ECAdmin extends Configured implements Tool {
           "Set the erasure coding policy for a file/directory.\n\n" +
           listing.toString() + "\n" +
           "-replicate and -policy are optional arguments. They cannot been " +
-          "used at the same time";
+          "used at the same time.\n";
     }
 
     @Override
@@ -642,6 +642,10 @@ public class ECAdmin extends Configured implements Tool {
           throw e;
         }
       } else {
+        if (args.size() > 0) {
+          System.err.println(getName() + ": Too many arguments");
+          return 1;
+        }
         result = dfs.getECTopologyResultForPolicies();
       }
       System.out.println(result.getResultMessage());
