@@ -24,7 +24,7 @@ import java.io.RandomAccessFile;
 
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.CommandLineParser;
-import org.apache.commons.cli.OptionBuilder;
+import org.apache.commons.cli.Option;
 import org.apache.commons.cli.Options;
 import org.apache.commons.cli.ParseException;
 import org.apache.commons.cli.PosixParser;
@@ -79,6 +79,16 @@ public class OfflineImageViewerPB {
       + "    to both inodes and inodes-under-construction, separated by a\n"
       + "    delimiter. The default delimiter is \\t, though this may be\n"
       + "    changed via the -delimiter argument.\n"
+      + "    -sp print storage policy, used by delimiter only.\n"
+      + "    -ec print erasure coding policy, used by delimiter only.\n"
+      + "    -m  defines multiThread to process sub-sections, \n"
+      + "    used by delimiter only.\n"
+      + "  * DetectCorruption: Detect potential corruption of the image by\n"
+      + "    selectively loading parts of it and actively searching for\n"
+      + "    inconsistencies. Outputs a summary of the found corruptions\n"
+      + "    in a delimited format.\n"
+      + "    Note that the check is not exhaustive, and only catches\n"
+      + "    missing nodes during the namespace reconstruction.\n"
       + "\n"
       + "Required command line arguments:\n"
       + "-i,--inputFile <arg>   FSImage or XML file to process.\n"
@@ -91,12 +101,32 @@ public class OfflineImageViewerPB {
       + "                       will also create an <outputFile>.md5 file.\n"
       + "-p,--processor <arg>   Select which type of processor to apply\n"
       + "                       against image file. (XML|FileDistribution|\n"
-      + "                       ReverseXML|Web|Delimited)\n"
+      + "                       ReverseXML|Web|Delimited|DetectCorruption)\n"
       + "                       The default is Web.\n"
-      + "-delimiter <arg>       Delimiting string to use with Delimited processor.  \n"
-      + "-t,--temp <arg>        Use temporary dir to cache intermediate result to generate\n"
-      + "                       Delimited outputs. If not set, Delimited processor constructs\n"
-      + "                       the namespace in memory before outputting text.\n"
+      + "-addr <arg>            Specify the address(host:port) to listen.\n"
+      + "                       (localhost:5978 by default). This option is\n"
+      + "                       used with Web processor.\n"
+      + "-maxSize <arg>         Specify the range [0, maxSize] of file sizes\n"
+      + "                       to be analyzed in bytes (128GB by default).\n"
+      + "                       This option is used with FileDistribution processor.\n"
+      + "-step <arg>            Specify the granularity of the distribution in bytes\n"
+      + "                       (2MB by default). This option is used\n"
+      + "                       with FileDistribution processor.\n"
+      + "-format                Format the output result in a human-readable fashion rather\n"
+      + "                       than a number of bytes. (false by default).\n"
+      + "                       This option is used with FileDistribution processor.\n"
+      + "-delimiter <arg>       Delimiting string to use with Delimited or \n"
+      + "                       DetectCorruption processor. \n"
+      + "-sp                    Whether to print storage policy (default is false). \n"
+      + "                       Is used by Delimited processor only. \n"
+      + "-ec                    Whether to print erasure coding policy (default is false). \n"
+      + "                       Is used by Delimited processor only. \n"
+      + "-t,--temp <arg>        Use temporary dir to cache intermediate\n"
+      + "                       result to generate DetectCorruption or\n"
+      + "                       Delimited outputs. If not set, the processor\n"
+      + "                       constructs the namespace in memory \n"
+      + "                       before outputting text.\n"
+      + "-m,--multiThread <arg> Use multiThread to process sub-sections.\n"
       + "-h,--help              Display usage information and exit\n";
 
   /**
@@ -107,10 +137,8 @@ public class OfflineImageViewerPB {
 
     // Build in/output file arguments, which are required, but there is no
     // addOption method that can specify this
-    OptionBuilder.isRequired();
-    OptionBuilder.hasArgs();
-    OptionBuilder.withLongOpt("inputFile");
-    options.addOption(OptionBuilder.create("i"));
+    Option optionInputFile = Option.builder("i").required().hasArgs().longOpt("inputFile").build();
+    options.addOption(optionInputFile);
 
     options.addOption("o", "outputFile", true, "");
     options.addOption("p", "processor", true, "");
@@ -120,7 +148,10 @@ public class OfflineImageViewerPB {
     options.addOption("format", false, "");
     options.addOption("addr", true, "");
     options.addOption("delimiter", true, "");
+    options.addOption("sp", false, "");
+    options.addOption("ec", false, "");
     options.addOption("t", "temp", true, "");
+    options.addOption("m", "multiThread", true, "");
 
     return options;
   }
@@ -172,8 +203,9 @@ public class OfflineImageViewerPB {
     String processor = cmd.getOptionValue("p", "Web");
     String outputFile = cmd.getOptionValue("o", "-");
     String delimiter = cmd.getOptionValue("delimiter",
-        PBImageDelimitedTextWriter.DEFAULT_DELIMITER);
+        PBImageTextWriter.DEFAULT_DELIMITER);
     String tempPath = cmd.getOptionValue("t", "");
+    int threads = Integer.parseInt(cmd.getOptionValue("m", "1"));
 
     Configuration conf = new Configuration();
     PrintStream out = null;
@@ -213,10 +245,19 @@ public class OfflineImageViewerPB {
         }
         break;
       case "DELIMITED":
+        boolean printStoragePolicy = cmd.hasOption("sp");
+        boolean printECPolicy = cmd.hasOption("ec");
         try (PBImageDelimitedTextWriter writer =
-            new PBImageDelimitedTextWriter(out, delimiter, tempPath);
-            RandomAccessFile r = new RandomAccessFile(inputFile, "r")) {
-          writer.visit(r);
+            new PBImageDelimitedTextWriter(out, delimiter,
+                tempPath, printStoragePolicy, printECPolicy, threads,
+                outputFile, conf)) {
+          writer.visit(inputFile);
+        }
+        break;
+      case "DETECTCORRUPTION":
+        try (PBImageCorruptionDetector detector =
+            new PBImageCorruptionDetector(out, delimiter, tempPath)) {
+          detector.visit(inputFile);
         }
         break;
       default:
