@@ -242,7 +242,7 @@ public class TestDFSInputStream {
       DFSInputStream dfsInputStream =
           (DFSInputStream) fs.open(filePath).getWrappedStream();
       LocatedBlock lb = mock(LocatedBlock.class);
-      when(lb.getCachedLocations()).thenReturn(new DatanodeInfo[0]);
+      when(lb.getCachedLocations()).thenReturn(DatanodeInfo.EMPTY_ARRAY);
       DatanodeID nodeId = new DatanodeID("localhost", "localhost", "dn0", 1111,
           1112, 1113, 1114);
       DatanodeInfo dnInfo = new DatanodeDescriptor(nodeId);
@@ -271,7 +271,7 @@ public class TestDFSInputStream {
       DFSInputStream dfsInputStream =
               (DFSInputStream) fs.open(filePath).getWrappedStream();
       LocatedBlock lb = mock(LocatedBlock.class);
-      when(lb.getCachedLocations()).thenReturn(new DatanodeInfo[0]);
+      when(lb.getCachedLocations()).thenReturn(DatanodeInfo.EMPTY_ARRAY);
       DatanodeID nodeId = new DatanodeID("localhost", "localhost", "dn0", 1111,
               1112, 1113, 1114);
       DatanodeInfo dnInfo = new DatanodeDescriptor(nodeId);
