@@ -23,11 +23,12 @@ import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
-import com.google.common.primitives.Bytes;
+import org.apache.hadoop.thirdparty.com.google.common.primitives.Bytes;
 import java.io.ByteArrayOutputStream;
 import java.io.DataOutputStream;
 import java.io.File;
 import java.io.IOException;
+import java.io.RandomAccessFile;
 
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileUtil;
@@ -82,7 +83,7 @@ public class TestJournal {
     conf.setBoolean(DFSConfigKeys.DFS_HA_TAILEDITS_INPROGRESS_KEY, true);
     journal = new Journal(conf, TEST_LOG_DIR, JID, StartupOption.REGULAR,
       mockErrorReporter);
-    journal.format(FAKE_NSINFO);
+    journal.format(FAKE_NSINFO, false);
   }
   
   @After
@@ -125,6 +126,48 @@ public class TestJournal {
     Assert.assertEquals(1, segmentState.getStartTxId());
   }
 
+  /**
+   * Test for HDFS-14557 to ensure that a edit file that failed to fully
+   * allocate and has a header byte of -1 is moved aside to allow startup
+   * to progress.
+   */
+  @Test
+  public void testEmptyEditsInProgressMovedAside() throws Exception {
+    // First, write 5 transactions to the journal
+    journal.startLogSegment(makeRI(1), 1,
+        NameNodeLayoutVersion.CURRENT_LAYOUT_VERSION - 1);
+    final int numTxns = 5;
+    byte[] ops = QJMTestUtil.createTxnData(1, 5);
+    journal.journal(makeRI(2), 1, 1, numTxns, ops);
+    // Now close the segment
+    journal.finalizeLogSegment(makeRI(3), 1, numTxns);
+
+    // Create a new segment creating a new edits_inprogress file
+    journal.startLogSegment(makeRI(4), 6,
+        NameNodeLayoutVersion.CURRENT_LAYOUT_VERSION - 1);
+    ops = QJMTestUtil.createTxnData(6, 5);
+    journal.journal(makeRI(5), 6, 6, numTxns, ops);
+    File eip = journal.getStorage().getInProgressEditLog(6);
+
+    // Now stop the journal without finalizing the segment
+    journal.close();
+
+    // Now "zero out" the EIP file with -1 bytes, similar to how it would
+    // appear if the pre-allocation failed
+    RandomAccessFile rwf = new RandomAccessFile(eip, "rw");
+    for (int i=0; i<rwf.length(); i++) {
+      rwf.write(-1);
+    }
+    rwf.close();
+
+    // Finally start the Journal again, and ensure the "zeroed out" file
+    // is renamed with a .empty extension
+    journal = new Journal(conf, TEST_LOG_DIR, JID, StartupOption.REGULAR,
+        mockErrorReporter);
+    File movedTo = new File(eip.getAbsolutePath()+".empty");
+    assertTrue(movedTo.exists());
+  }
+
   @Test (timeout = 10000)
   public void testEpochHandling() throws Exception {
     assertEquals(0, journal.getLastPromisedEpoch());
@@ -165,12 +208,12 @@ public class TestJournal {
     journal.startLogSegment(makeRI(1), 1,
         NameNodeLayoutVersion.CURRENT_LAYOUT_VERSION);
     // Send txids 1-3, with a request indicating only 0 committed
-    journal.journal(new RequestInfo(JID, 1, 2, 0), 1, 1, 3,
+    journal.journal(new RequestInfo(JID, null,  1, 2, 0), 1, 1, 3,
         QJMTestUtil.createTxnData(1, 3));
     assertEquals(0, journal.getCommittedTxnId());
     
     // Send 4-6, with request indicating that through 3 is committed.
-    journal.journal(new RequestInfo(JID, 1, 3, 3), 1, 4, 3,
+    journal.journal(new RequestInfo(JID, null, 1, 3, 3), 1, 4, 3,
         QJMTestUtil.createTxnData(4, 6));
     assertEquals(3, journal.getCommittedTxnId());
   }
@@ -204,7 +247,7 @@ public class TestJournal {
   @Test (timeout = 10000)
   public void testFormatResetsCachedValues() throws Exception {
     journal.newEpoch(FAKE_NSINFO, 12345L);
-    journal.startLogSegment(new RequestInfo(JID, 12345L, 1L, 0L), 1L,
+    journal.startLogSegment(new RequestInfo(JID, null, 12345L, 1L, 0L), 1L,
         NameNodeLayoutVersion.CURRENT_LAYOUT_VERSION);
 
     assertEquals(12345L, journal.getLastPromisedEpoch());
@@ -216,7 +259,7 @@ public class TestJournal {
     // Clear the storage directory before reformatting it
     journal.getStorage().getJournalManager()
         .getStorageDirectory().clearDirectory();
-    journal.format(FAKE_NSINFO_2);
+    journal.format(FAKE_NSINFO_2, false);
     
     assertEquals(0, journal.getLastPromisedEpoch());
     assertEquals(0, journal.getLastWriterEpoch());
@@ -413,7 +456,7 @@ public class TestJournal {
   }
   
   private static RequestInfo makeRI(int serial) {
-    return new RequestInfo(JID, 1, serial, 0);
+    return new RequestInfo(JID, null, 1, serial, 0);
   }
   
   @Test (timeout = 10000)
@@ -434,7 +477,7 @@ public class TestJournal {
     try {
       // Format again here and to format the non-empty directories in
       // journal node.
-      journal.format(FAKE_NSINFO);
+      journal.format(FAKE_NSINFO, false);
       fail("Did not fail to format non-empty directories in journal node.");
     } catch (IOException ioe) {
       GenericTestUtils.assertExceptionContains(
@@ -480,4 +523,16 @@ public class TestJournal {
             QJMTestUtil.createTxnData(startTxn, expectedTxnCount)),
         result.getEditLog().toByteArray());
   }
+
+  @Test
+  public void testFormatNonEmptyStorageDirectoriesWhenforceOptionIsTrue()
+      throws Exception {
+    try {
+      // Format again here and to format the non-empty directories in
+      // journal node.
+      journal.format(FAKE_NSINFO, true);
+    } catch (IOException ioe) {
+      fail("Format should be success with force option.");
+    }
+  }
 }
