@@ -721,7 +721,10 @@ public class TestDataNodeVolumeFailureReporting {
     String[] absolutePaths = new String[locations.length];
     for (int count = 0; count < locations.length; count++) {
       try {
-        absolutePaths[count] = new File(new URI(locations[count]))
+        String location = locations[count];
+        location = location.contains("]")
+            ? location.substring(location.indexOf("]") + 1) : location;
+        absolutePaths[count] = new File(new URI(location))
             .getAbsolutePath();
       } catch (URISyntaxException e) {
         //if the provided location is not an URI,
