@@ -18,7 +18,6 @@
 
 package org.apache.hadoop.hdfs.server.namenode;
 
-import org.apache.hadoop.thirdparty.com.google.common.collect.Lists;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileStatus;
@@ -39,7 +38,7 @@ import org.apache.hadoop.security.authorize.ProxyServers;
 import org.apache.hadoop.security.authorize.ProxyUsers;
 import org.apache.hadoop.test.GenericTestUtils;
 import org.apache.hadoop.test.GenericTestUtils.LogCapturer;
-import org.apache.log4j.Level;
+import org.apache.hadoop.util.Lists;
 
 import org.junit.Before;
 import org.junit.Test;
@@ -48,6 +47,7 @@ import org.mockito.Mockito;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import org.slf4j.event.Level;
 
 import java.io.IOException;
 import java.net.HttpURLConnection;
@@ -87,7 +87,7 @@ public class TestAuditLogger {
   private static final Logger LOG = LoggerFactory.getLogger(
       TestAuditLogger.class);
   static {
-    GenericTestUtils.setLogLevel(LOG, Level.ALL);
+    GenericTestUtils.setLogLevel(LOG, Level.TRACE);
   }
 
   private static final short TEST_PERMISSION = (short) 0654;
@@ -258,7 +258,7 @@ public class TestAuditLogger {
     conf.setInt(HADOOP_CALLER_CONTEXT_SIGNATURE_MAX_SIZE_KEY, 40);
 
     try (MiniDFSCluster cluster = new MiniDFSCluster.Builder(conf).build()) {
-      LogCapturer auditlog = LogCapturer.captureLogs(FSNamesystem.auditLog);
+      LogCapturer auditlog = LogCapturer.captureLogs(FSNamesystem.AUDIT_LOG);
       cluster.waitClusterUp();
       final FileSystem fs = cluster.getFileSystem();
       final long time = System.currentTimeMillis();
@@ -568,7 +568,7 @@ public class TestAuditLogger {
     Configuration conf = new HdfsConfiguration();
     MiniDFSCluster cluster1 = new MiniDFSCluster.Builder(conf).build();
     try {
-      LogCapturer auditLog = LogCapturer.captureLogs(FSNamesystem.auditLog);
+      LogCapturer auditLog = LogCapturer.captureLogs(FSNamesystem.AUDIT_LOG);
       cluster1.waitClusterUp();
       FileSystem fs = cluster1.getFileSystem();
       long time = System.currentTimeMillis();
@@ -585,7 +585,7 @@ public class TestAuditLogger {
     conf.setBoolean(HADOOP_CALLER_CONTEXT_ENABLED_KEY, true);
     MiniDFSCluster cluster2 = new MiniDFSCluster.Builder(conf).build();
     try {
-      LogCapturer auditLog = LogCapturer.captureLogs(FSNamesystem.auditLog);
+      LogCapturer auditLog = LogCapturer.captureLogs(FSNamesystem.AUDIT_LOG);
       cluster2.waitClusterUp();
       FileSystem fs = cluster2.getFileSystem();
       long time = System.currentTimeMillis();
@@ -606,7 +606,7 @@ public class TestAuditLogger {
     conf.setInt(HADOOP_CALLER_CONTEXT_SIGNATURE_MAX_SIZE_KEY, 40);
 
     try (MiniDFSCluster cluster = new MiniDFSCluster.Builder(conf).build()) {
-      LogCapturer auditlog = LogCapturer.captureLogs(FSNamesystem.auditLog);
+      LogCapturer auditlog = LogCapturer.captureLogs(FSNamesystem.AUDIT_LOG);
       cluster.waitClusterUp();
       final FileSystem fs = cluster.getFileSystem();
       final long time = System.currentTimeMillis();
