@@ -30,7 +30,7 @@ import org.apache.hadoop.hdfs.server.blockmanagement.BlockManagerTestUtil;
 import org.apache.hadoop.hdfs.server.datanode.DataNodeTestUtils;
 import org.apache.hadoop.test.GenericTestUtils;
 
-import com.google.common.base.Supplier;
+import java.util.function.Supplier;
 
 import org.junit.After;
 import org.junit.Before;
