@@ -25,6 +25,7 @@ import static org.mockito.Mockito.when;
 
 import java.io.File;
 import java.io.IOException;
+import java.io.InterruptedIOException;
 import java.net.BindException;
 import java.net.URI;
 import java.util.ArrayList;
@@ -56,13 +57,14 @@ import org.apache.hadoop.hdfs.server.namenode.NNStorage;
 import org.apache.hadoop.hdfs.server.namenode.NameNode;
 import org.apache.hadoop.hdfs.server.namenode.NameNodeAdapter;
 import org.apache.hadoop.test.GenericTestUtils;
+import org.apache.hadoop.util.FakeTimer;
 import org.slf4j.event.Level;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
 import org.junit.runners.Parameterized.Parameters;
 
-import com.google.common.base.Supplier;
+import java.util.function.Supplier;
 import org.mockito.Mockito;
 
 @RunWith(Parameterized.class)
@@ -394,13 +396,15 @@ public class TestEditLogTailer {
     // Time in seconds to wait before checking if edit logs are rolled while
     // expecting no edit log roll
     final int noLogRollWaitTime = 2;
+
     // Time in seconds to wait before checking if edit logs are rolled while
-    // expecting edit log roll
+    // expecting edit log roll.
     final int logRollWaitTime = 3;
 
+    final int logRollPeriod = standbyCatchupWaitTime + noLogRollWaitTime + 1;
+    final long logRollPeriodMs = TimeUnit.SECONDS.toMillis(logRollPeriod);
     Configuration conf = getConf();
-    conf.setInt(DFSConfigKeys.DFS_HA_LOGROLL_PERIOD_KEY,
-        standbyCatchupWaitTime + noLogRollWaitTime + 1);
+    conf.setInt(DFSConfigKeys.DFS_HA_LOGROLL_PERIOD_KEY, logRollPeriod);
     conf.setInt(DFSConfigKeys.DFS_HA_TAILEDITS_PERIOD_KEY, 1);
     conf.setBoolean(DFSConfigKeys.DFS_HA_TAILEDITS_INPROGRESS_KEY, true);
 
@@ -429,19 +433,29 @@ public class TestEditLogTailer {
       waitForStandbyToCatchUpWithInProgressEdits(standby, activeTxId,
           standbyCatchupWaitTime);
 
+      long curTime = standby.getNamesystem().getEditLogTailer().getTimer()
+          .monotonicNow();
+      long insufficientTimeForLogRoll = logRollPeriodMs / 3;
+      final FakeTimer testTimer =
+          new FakeTimer(curTime + insufficientTimeForLogRoll);
+      standby.getNamesystem().getEditLogTailer().setTimerForTest(testTimer);
+      Thread.sleep(2000);
+
       for (int i = DIRS_TO_MAKE / 2; i < DIRS_TO_MAKE; i++) {
         NameNodeAdapter.mkdirs(active, getDirPath(i),
             new PermissionStatus("test", "test",
             new FsPermission((short)00755)), true);
       }
 
-      boolean exceptionThrown = false;
       try {
         checkForLogRoll(active, origTxId, noLogRollWaitTime);
+        fail("Expected to timeout");
       } catch (TimeoutException e) {
-        exceptionThrown = true;
+        // expected
       }
-      assertTrue(exceptionThrown);
+
+      long sufficientTimeForLogRoll = logRollPeriodMs * 3;
+      testTimer.advance(sufficientTimeForLogRoll);
 
       checkForLogRoll(active, origTxId, logRollWaitTime);
     } finally {
@@ -449,29 +463,97 @@ public class TestEditLogTailer {
     }
   }
 
+  @Test
+  public void testRollEditLogHandleThreadInterruption()
+      throws IOException, InterruptedException, TimeoutException {
+    Configuration conf = getConf();
+    // RollEdits timeout 1s.
+    conf.setInt(DFSConfigKeys.DFS_HA_TAILEDITS_ROLLEDITS_TIMEOUT_KEY, 1);
+
+    MiniDFSCluster cluster = null;
+    try {
+      cluster = createMiniDFSCluster(conf, 3);
+      cluster.transitionToActive(2);
+      EditLogTailer tailer = Mockito.spy(
+          cluster.getNamesystem(0).getEditLogTailer());
+
+      // Stop the edit log tail thread for testing.
+      tailer.setShouldRunForTest(false);
+
+      final AtomicInteger invokedTimes = new AtomicInteger(0);
+
+      // For nn0 run triggerActiveLogRoll, nns is [nn1,nn2].
+      // Mock the NameNodeProxy for testing.
+      // An InterruptedIOException will be thrown when requesting to nn1.
+      when(tailer.getNameNodeProxy()).thenReturn(
+          tailer.new MultipleNameNodeProxy<Void>() {
+            @Override
+            protected Void doWork() throws IOException {
+              invokedTimes.getAndIncrement();
+              if (tailer.getCurrentNN().getNameNodeID().equals("nn1")) {
+                while (true) {
+                  Thread.yield();
+                  if (Thread.currentThread().isInterrupted()) {
+                    throw new InterruptedIOException("It is an Interrupted IOException.");
+                  }
+                }
+              } else {
+                tailer.getCachedActiveProxy().rollEditLog();
+                return null;
+              }
+            }
+          }
+      );
+
+      // Record the initial LastRollTimeMs value.
+      // This time will be updated only when triggerActiveLogRoll is executed successfully.
+      long initLastRollTimeMs = tailer.getLastRollTimeMs();
+
+      // Execute triggerActiveLogRoll for the first time.
+      // The MultipleNameNodeProxy uses round-robin to look for an active NN to roll the edit log.
+      // Here, a request will be made to nn1, and the main thread will trigger a Timeout and
+      // the doWork() method will throw an InterruptedIOException.
+      // The getActiveNodeProxy() method will determine that the thread is interrupted
+      // and will return null.
+      tailer.triggerActiveLogRoll();
+
+      // Execute triggerActiveLogRoll for the second time.
+      // A request will be made to nn2 and the rollEditLog will be successfully finished and
+      // lastRollTimeMs will be updated.
+      tailer.triggerActiveLogRoll();
+      GenericTestUtils.waitFor(new Supplier<Boolean>() {
+        @Override
+        public Boolean get() {
+          return tailer.getLastRollTimeMs() > initLastRollTimeMs;
+        }
+      }, 100, 10000);
+
+      // The total number of invoked times should be 2.
+      assertEquals(2, invokedTimes.get());
+    } finally {
+      if (cluster != null) {
+        cluster.shutdown();
+      }
+    }
+  }
+
   private static void waitForStandbyToCatchUpWithInProgressEdits(
       final NameNode standby, final long activeTxId,
       int maxWaitSec) throws Exception {
-    GenericTestUtils.waitFor(new Supplier<Boolean>() {
-      @Override
-      public Boolean get() {
-        long standbyTxId = standby.getNamesystem().getFSImage()
-            .getLastAppliedTxId();
-        return (standbyTxId >= activeTxId);
-      }
-    }, 100, maxWaitSec * 1000);
+    GenericTestUtils.waitFor(() -> {
+      long standbyTxId = standby.getNamesystem().getFSImage()
+          .getLastAppliedTxId();
+      return (standbyTxId >= activeTxId);
+    }, 100, TimeUnit.SECONDS.toMillis(maxWaitSec));
   }
 
   private static void checkForLogRoll(final NameNode active,
       final long origTxId, int maxWaitSec) throws Exception {
-    GenericTestUtils.waitFor(new Supplier<Boolean>() {
-      @Override
-      public Boolean get() {
-        long curSegmentTxId = active.getNamesystem().getFSImage().getEditLog()
-            .getCurSegmentTxId();
-        return (origTxId != curSegmentTxId);
-      }
-    }, 100, maxWaitSec * 1000);
+    GenericTestUtils.waitFor(() -> {
+      long curSegmentTxId = active.getNamesystem().getFSImage().getEditLog()
+          .getCurSegmentTxId();
+      return (origTxId != curSegmentTxId);
+    }, 100, TimeUnit.SECONDS.toMillis(maxWaitSec));
   }
 
   private static MiniDFSCluster createMiniDFSCluster(Configuration conf,
@@ -488,4 +570,5 @@ public class TestEditLogTailer {
         .build();
     return cluster;
   }
+
 }
