@@ -31,10 +31,12 @@ import java.net.ServerSocket;
 import java.net.Socket;
 import java.net.SocketTimeoutException;
 import java.nio.channels.SocketChannel;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.List;
+import java.util.concurrent.TimeoutException;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -47,6 +49,7 @@ import org.apache.hadoop.net.NetUtils;
 import org.apache.hadoop.security.authentication.client.ConnectionConfigurator;
 import org.apache.hadoop.test.GenericTestUtils;
 import org.junit.After;
+import org.junit.AssumptionViolatedException;
 import org.junit.Before;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
@@ -85,6 +88,7 @@ public class TestWebHdfsTimeouts {
       return conn;
     }
   });
+  private volatile boolean failedToConsumeBacklog;
 
   public enum TimeoutSource { ConnectionFactory, Configuration };
 
@@ -123,6 +127,7 @@ public class TestWebHdfsTimeouts {
 
     clients = new ArrayList<SocketChannel>();
     serverThread = null;
+    failedToConsumeBacklog = false;
   }
 
   @After
@@ -213,6 +218,7 @@ public class TestWebHdfsTimeouts {
       fs.getFileChecksum(new Path("/file"));
       fail("expected timeout");
     } catch (SocketTimeoutException e) {
+      assumeBacklogConsumed();
       GenericTestUtils.assertExceptionContains(
           fs.getUri().getAuthority() + ": connect timed out", e);
     }
@@ -246,6 +252,7 @@ public class TestWebHdfsTimeouts {
       os = fs.create(new Path("/file"));
       fail("expected timeout");
     } catch (SocketTimeoutException e) {
+      assumeBacklogConsumed();
       GenericTestUtils.assertExceptionContains(
           fs.getUri().getAuthority() + ": connect timed out", e);
     } finally {
@@ -326,7 +333,7 @@ public class TestWebHdfsTimeouts {
 
           // Write response.
           out = clientSocket.getOutputStream();
-          out.write(temporaryRedirect().getBytes("UTF-8"));
+          out.write(temporaryRedirect().getBytes(StandardCharsets.UTF_8));
         } catch (IOException e) {
           // Fail the test on any I/O error in the server thread.
           LOG.error("unexpected IOException in server thread", e);
@@ -359,6 +366,28 @@ public class TestWebHdfsTimeouts {
       client.connect(nnHttpAddress);
       clients.add(client);
     }
+    try {
+      GenericTestUtils.waitFor(() -> {
+        try (SocketChannel c = SocketChannel.open()) {
+          c.socket().connect(nnHttpAddress, 100);
+        } catch (SocketTimeoutException e) {
+          return true;
+        } catch (IOException e) {
+          LOG.debug("unexpected exception: " + e);
+        }
+        return false;
+      }, 100, 10000);
+    } catch (TimeoutException | InterruptedException e) {
+      failedToConsumeBacklog = true;
+      assumeBacklogConsumed();
+    }
+  }
+
+  private void assumeBacklogConsumed() {
+    if (failedToConsumeBacklog) {
+      throw new AssumptionViolatedException(
+          "failed to fill up connection backlog.");
+    }
   }
 
   /**
