@@ -78,7 +78,7 @@ import org.apache.hadoop.yarn.exceptions.YarnRuntimeException;
 import org.apache.hadoop.yarn.webapp.BadRequestException;
 import org.apache.hadoop.yarn.webapp.NotFoundException;
 
-import com.google.common.base.Preconditions;
+import org.apache.hadoop.util.Preconditions;
 import com.google.inject.Inject;
 
 @Path("/ws/v1/mapreduce")
