@@ -27,9 +27,8 @@ import java.util.Arrays;
 import java.util.jar.JarOutputStream;
 import java.util.zip.ZipEntry;
 
-import org.junit.Assert;
-import org.junit.Test;
-import static org.junit.Assert.*;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Timeout;
 
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.filecache.DistributedCache;
@@ -50,6 +49,13 @@ import org.apache.hadoop.mapreduce.lib.output.NullOutputFormat;
 import org.apache.hadoop.mapreduce.server.jobtracker.JTConfig;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+
 /**
  * Tests the use of the
  * {@link org.apache.hadoop.mapreduce.filecache.DistributedCache} within the
@@ -96,23 +102,23 @@ public class TestMRWithDistributedCache {
       FileSystem fs = LocalFileSystem.get(conf);
 
       // Check that 2 files and 2 archives are present
-      Assert.assertEquals(2, localFiles.length);
-      Assert.assertEquals(2, localArchives.length);
-      Assert.assertEquals(2, files.length);
-      Assert.assertEquals(2, archives.length);
+      assertEquals(2, localFiles.length);
+      assertEquals(2, localArchives.length);
+      assertEquals(2, files.length);
+      assertEquals(2, archives.length);
 
       // Check the file name
-      Assert.assertTrue(files[0].getPath().endsWith("distributed.first"));
-      Assert.assertTrue(files[1].getPath().endsWith("distributed.second.jar"));
+      assertTrue(files[0].getPath().endsWith("distributed.first"));
+      assertTrue(files[1].getPath().endsWith("distributed.second.jar"));
       
       // Check lengths of the files
-      Assert.assertEquals(1, fs.getFileStatus(localFiles[0]).getLen());
-      Assert.assertTrue(fs.getFileStatus(localFiles[1]).getLen() > 1);
+      assertEquals(1, fs.getFileStatus(localFiles[0]).getLen());
+      assertTrue(fs.getFileStatus(localFiles[1]).getLen() > 1);
 
       // Check extraction of the archive
-      Assert.assertTrue(fs.exists(new Path(localArchives[0],
+      assertTrue(fs.exists(new Path(localArchives[0],
           "distributed.jar.inside3")));
-      Assert.assertTrue(fs.exists(new Path(localArchives[1],
+      assertTrue(fs.exists(new Path(localArchives[1],
           "distributed.jar.inside4")));
 
       // Check the class loaders
@@ -120,19 +126,20 @@ public class TestMRWithDistributedCache {
       ClassLoader cl = Thread.currentThread().getContextClassLoader();
       // Both the file and the archive were added to classpath, so both
       // should be reachable via the class loader.
-      Assert.assertNotNull(cl.getResource("distributed.jar.inside2"));
-      Assert.assertNotNull(cl.getResource("distributed.jar.inside3"));
-      Assert.assertNull(cl.getResource("distributed.jar.inside4"));
+      assertNotNull(cl.getResource("distributed.jar.inside2"));
+      assertNotNull(cl.getResource("distributed.jar.inside3"));
+      assertNull(cl.getResource("distributed.jar.inside4"));
 
       // Check that the symlink for the renaming was created in the cwd;
-      Assert.assertTrue("symlink distributed.first.symlink doesn't exist",
-          symlinkFile.exists());
-      Assert.assertEquals("symlink distributed.first.symlink length not 1", 1,
-          symlinkFile.length());
+      assertTrue(symlinkFile.exists(),
+          "symlink distributed.first.symlink doesn't exist");
+      assertEquals(1,
+          symlinkFile.length(),
+          "symlink distributed.first.symlink length not 1");
       
       //This last one is a difference between MRv2 and MRv1
-      Assert.assertTrue("second file should be symlinked too",
-          expectedAbsentSymlinkFile.exists());
+      assertTrue(expectedAbsentSymlinkFile.exists(),
+          "second file should be symlinked too");
     }
 
   }
@@ -190,16 +197,16 @@ public class TestMRWithDistributedCache {
   @Test
   public void testLocalJobRunner() throws Exception {
     symlinkFile.delete(); // ensure symlink is not present (e.g. if test is
-                          // killed part way through)
+    // killed part way through)
     
     Configuration c = new Configuration();
     c.set(JTConfig.JT_IPC_ADDRESS, "local");
     c.set("fs.defaultFS", "file:///");
     testWithConf(c);
-    
-    assertFalse("Symlink not removed by local job runner",
-            // Symlink target will have gone so can't use File.exists()
-            Arrays.asList(new File(".").list()).contains(symlinkFile.getName()));
+
+    assertFalse(Arrays.asList(new File(".").list()).contains(symlinkFile.getName()),
+        // Symlink target will have gone so can't use File.exists()
+        "Symlink not removed by local job runner");
   }
 
   private Path createTempFile(String filename, String contents)
@@ -223,92 +230,93 @@ public class TestMRWithDistributedCache {
     return p;
   }
 
-  @Test (timeout = 10000)
+  @Test
+  @Timeout(10000)
   public void testDeprecatedFunctions() throws Exception {
     DistributedCache.addLocalArchives(conf, "Test Local Archives 1");
-    Assert.assertEquals("Test Local Archives 1",
+    assertEquals("Test Local Archives 1",
         conf.get(DistributedCache.CACHE_LOCALARCHIVES));
-    Assert.assertEquals(1,
-        DistributedCache.getLocalCacheArchives(conf).length);
-    Assert.assertEquals("Test Local Archives 1",
-        DistributedCache.getLocalCacheArchives(conf)[0].getName());
+    assertEquals(1,
+        JobContextImpl.getLocalCacheArchives(conf).length);
+    assertEquals("Test Local Archives 1",
+        JobContextImpl.getLocalCacheArchives(conf)[0].getName());
     DistributedCache.addLocalArchives(conf, "Test Local Archives 2");
-    Assert.assertEquals("Test Local Archives 1,Test Local Archives 2",
+    assertEquals("Test Local Archives 1,Test Local Archives 2",
         conf.get(DistributedCache.CACHE_LOCALARCHIVES));
-    Assert.assertEquals(2,
-        DistributedCache.getLocalCacheArchives(conf).length);
-    Assert.assertEquals("Test Local Archives 2",
-        DistributedCache.getLocalCacheArchives(conf)[1].getName());
+    assertEquals(2,
+        JobContextImpl.getLocalCacheArchives(conf).length);
+    assertEquals("Test Local Archives 2",
+        JobContextImpl.getLocalCacheArchives(conf)[1].getName());
     DistributedCache.setLocalArchives(conf, "Test Local Archives 3");
-    Assert.assertEquals("Test Local Archives 3",
+    assertEquals("Test Local Archives 3",
         conf.get(DistributedCache.CACHE_LOCALARCHIVES));
-    Assert.assertEquals(1,
-        DistributedCache.getLocalCacheArchives(conf).length);
-    Assert.assertEquals("Test Local Archives 3",
-        DistributedCache.getLocalCacheArchives(conf)[0].getName());
+    assertEquals(1,
+        JobContextImpl.getLocalCacheArchives(conf).length);
+    assertEquals("Test Local Archives 3",
+        JobContextImpl.getLocalCacheArchives(conf)[0].getName());
 
     DistributedCache.addLocalFiles(conf, "Test Local Files 1");
-    Assert.assertEquals("Test Local Files 1",
+    assertEquals("Test Local Files 1",
         conf.get(DistributedCache.CACHE_LOCALFILES));
-    Assert.assertEquals(1,
-        DistributedCache.getLocalCacheFiles(conf).length);
-    Assert.assertEquals("Test Local Files 1",
-        DistributedCache.getLocalCacheFiles(conf)[0].getName());
+    assertEquals(1,
+        JobContextImpl.getLocalCacheFiles(conf).length);
+    assertEquals("Test Local Files 1",
+        JobContextImpl.getLocalCacheFiles(conf)[0].getName());
     DistributedCache.addLocalFiles(conf, "Test Local Files 2");
-    Assert.assertEquals("Test Local Files 1,Test Local Files 2",
+    assertEquals("Test Local Files 1,Test Local Files 2",
         conf.get(DistributedCache.CACHE_LOCALFILES));
-    Assert.assertEquals(2,
-        DistributedCache.getLocalCacheFiles(conf).length);
-    Assert.assertEquals("Test Local Files 2",
-        DistributedCache.getLocalCacheFiles(conf)[1].getName());
+    assertEquals(2,
+        JobContextImpl.getLocalCacheFiles(conf).length);
+    assertEquals("Test Local Files 2",
+        JobContextImpl.getLocalCacheFiles(conf)[1].getName());
     DistributedCache.setLocalFiles(conf, "Test Local Files 3");
-    Assert.assertEquals("Test Local Files 3",
+    assertEquals("Test Local Files 3",
         conf.get(DistributedCache.CACHE_LOCALFILES));
-    Assert.assertEquals(1,
-        DistributedCache.getLocalCacheFiles(conf).length);
-    Assert.assertEquals("Test Local Files 3",
-        DistributedCache.getLocalCacheFiles(conf)[0].getName());
+    assertEquals(1,
+        JobContextImpl.getLocalCacheFiles(conf).length);
+    assertEquals("Test Local Files 3",
+        JobContextImpl.getLocalCacheFiles(conf)[0].getName());
 
     DistributedCache.setArchiveTimestamps(conf, "1234567890");
-    Assert.assertEquals(1234567890,
+    assertEquals(1234567890,
         conf.getLong(DistributedCache.CACHE_ARCHIVES_TIMESTAMPS, 0));
-    Assert.assertEquals(1,
-        DistributedCache.getArchiveTimestamps(conf).length);
-    Assert.assertEquals(1234567890,
-        DistributedCache.getArchiveTimestamps(conf)[0]);
+    assertEquals(1,
+        JobContextImpl.getArchiveTimestamps(conf).length);
+    assertEquals(1234567890,
+        JobContextImpl.getArchiveTimestamps(conf)[0]);
     DistributedCache.setFileTimestamps(conf, "1234567890");
-    Assert.assertEquals(1234567890,
+    assertEquals(1234567890,
         conf.getLong(DistributedCache.CACHE_FILES_TIMESTAMPS, 0));
-    Assert.assertEquals(1,
-        DistributedCache.getFileTimestamps(conf).length);
-    Assert.assertEquals(1234567890,
-        DistributedCache.getFileTimestamps(conf)[0]);
+    assertEquals(1,
+        JobContextImpl.getFileTimestamps(conf).length);
+    assertEquals(1234567890,
+        JobContextImpl.getFileTimestamps(conf)[0]);
 
     DistributedCache.createAllSymlink(conf, new File("Test Job Cache Dir"),
         new File("Test Work Dir"));
-    Assert.assertNull(conf.get(DistributedCache.CACHE_SYMLINK));
-    Assert.assertTrue(DistributedCache.getSymlink(conf));
+    assertNull(conf.get(DistributedCache.CACHE_SYMLINK));
+    assertTrue(DistributedCache.getSymlink(conf));
 
-    Assert.assertTrue(symlinkFile.createNewFile());
+    assertTrue(symlinkFile.createNewFile());
     FileStatus fileStatus =
         DistributedCache.getFileStatus(conf, symlinkFile.toURI());
-    Assert.assertNotNull(fileStatus);
-    Assert.assertEquals(fileStatus.getModificationTime(),
+    assertNotNull(fileStatus);
+    assertEquals(fileStatus.getModificationTime(),
         DistributedCache.getTimestamp(conf, symlinkFile.toURI()));
-    Assert.assertTrue(symlinkFile.delete());
+    assertTrue(symlinkFile.delete());
 
-    DistributedCache.addCacheArchive(symlinkFile.toURI(), conf);
-    Assert.assertEquals(symlinkFile.toURI().toString(),
+    Job.addCacheArchive(symlinkFile.toURI(), conf);
+    assertEquals(symlinkFile.toURI().toString(),
         conf.get(DistributedCache.CACHE_ARCHIVES));
-    Assert.assertEquals(1, DistributedCache.getCacheArchives(conf).length);
-    Assert.assertEquals(symlinkFile.toURI(),
-        DistributedCache.getCacheArchives(conf)[0]);
+    assertEquals(1, JobContextImpl.getCacheArchives(conf).length);
+    assertEquals(symlinkFile.toURI(),
+        JobContextImpl.getCacheArchives(conf)[0]);
 
-    DistributedCache.addCacheFile(symlinkFile.toURI(), conf);
-    Assert.assertEquals(symlinkFile.toURI().toString(),
+    Job.addCacheFile(symlinkFile.toURI(), conf);
+    assertEquals(symlinkFile.toURI().toString(),
         conf.get(DistributedCache.CACHE_FILES));
-    Assert.assertEquals(1, DistributedCache.getCacheFiles(conf).length);
-    Assert.assertEquals(symlinkFile.toURI(),
-        DistributedCache.getCacheFiles(conf)[0]);
+    assertEquals(1, JobContextImpl.getCacheFiles(conf).length);
+    assertEquals(symlinkFile.toURI(),
+        JobContextImpl.getCacheFiles(conf)[0]);
   }
 }
