@@ -34,7 +34,6 @@ import java.util.HashMap;
 import java.net.InetSocketAddress;
 import java.net.URI;
 
-import org.apache.commons.logging.Log;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.conf.Configured;
 import org.apache.hadoop.hdfs.MiniDFSCluster;
@@ -49,6 +48,10 @@ import org.apache.hadoop.mapred.lib.LongSumReducer;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.util.StringUtils;
 import org.junit.Test;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotSame;
@@ -57,7 +60,8 @@ import static org.junit.Assert.fail;
 
 
 public class TestFileSystem {
-  private static final Log LOG = FileSystem.LOG;
+  private static final Logger LOG =
+      LoggerFactory.getLogger(TestFileSystem.class);
 
   private static Configuration conf = new Configuration();
   private static int BUFFER_SIZE = conf.getInt("io.file.buffer.size", 4096);
@@ -101,7 +105,8 @@ public class TestFileSystem {
     // This should go to TestFsShell.java when it is added.
     CommandFormat cf;
     cf= new CommandFormat("copyToLocal", 2,2,"crc","ignoreCrc");
-    assertEquals(cf.parse(new String[] {"-get","file", "-"}, 1).get(1), "-");
+    assertThat(cf.parse(new String[] {"-get", "file", "-"}, 1).get(1))
+        .isEqualTo("-");
     try {
       cf.parse(new String[] {"-get","file","-ignoreCrc","/foo"}, 1);
       fail("Expected parsing to fail as it should stop at first non-option");
@@ -110,12 +115,16 @@ public class TestFileSystem {
       // Expected
     }  
     cf = new CommandFormat("tail", 1, 1, "f");
-    assertEquals(cf.parse(new String[] {"-tail","fileName"}, 1).get(0),"fileName");
-    assertEquals(cf.parse(new String[] {"-tail","-f","fileName"}, 1).get(0),"fileName");
+    assertThat(cf.parse(new String[] {"-tail", "fileName"}, 1).get(0))
+        .isEqualTo("fileName");
+    assertThat(cf.parse(new String[] {"-tail", "-f", "fileName"}, 1).get(0))
+        .isEqualTo("fileName");
     cf = new CommandFormat("setrep", 2, 2, "R", "w");
-    assertEquals(cf.parse(new String[] {"-setrep","-R","2","/foo/bar"}, 1).get(1), "/foo/bar");
+    assertThat(cf.parse(new String[] {"-setrep", "-R", "2", "/foo/bar"}, 1)
+        .get(1)).isEqualTo("/foo/bar");
     cf = new CommandFormat("put", 2, 10000);
-    assertEquals(cf.parse(new String[] {"-put", "-", "dest"}, 1).get(1), "dest"); 
+    assertThat(cf.parse(new String[] {"-put", "-", "dest"}, 1).get(1))
+        .isEqualTo("dest");
   }
 
   public static void createControlFile(FileSystem fs,
@@ -576,18 +585,6 @@ public class TestFileSystem {
       new Path("file:///").getFileSystem(conf);
       FileSystem.closeAll();
     }
-
-    {
-      Configuration conf = new Configuration();
-      new Path("hftp://localhost:12345/").getFileSystem(conf);
-      FileSystem.closeAll();
-    }
-
-    {
-      Configuration conf = new Configuration();
-      FileSystem fs = new Path("hftp://localhost:12345/").getFileSystem(conf);
-      FileSystem.closeAll();
-    }
   }
 
   @Test
@@ -629,12 +626,12 @@ public class TestFileSystem {
     Configuration conf = new Configuration();
     
     // check basic equality
-    FileSystem.Cache.Key lowercaseCachekey1 = new FileSystem.Cache.Key(new URI("hftp://localhost:12345/"), conf);
-    FileSystem.Cache.Key lowercaseCachekey2 = new FileSystem.Cache.Key(new URI("hftp://localhost:12345/"), conf);
+    FileSystem.Cache.Key lowercaseCachekey1 = new FileSystem.Cache.Key(new URI("hdfs://localhost:12345/"), conf);
+    FileSystem.Cache.Key lowercaseCachekey2 = new FileSystem.Cache.Key(new URI("hdfs://localhost:12345/"), conf);
     assertEquals( lowercaseCachekey1, lowercaseCachekey2 );
 
     // check insensitive equality    
-    FileSystem.Cache.Key uppercaseCachekey = new FileSystem.Cache.Key(new URI("HFTP://Localhost:12345/"), conf);
+    FileSystem.Cache.Key uppercaseCachekey = new FileSystem.Cache.Key(new URI("HDFS://Localhost:12345/"), conf);
     assertEquals( lowercaseCachekey2, uppercaseCachekey );
 
     // check behaviour with collections
