@@ -17,8 +17,6 @@
  */
 package org.apache.hadoop.mapreduce;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FSDataInputStream;
 import org.apache.hadoop.fs.FileStatus;
@@ -33,6 +31,8 @@ import org.apache.hadoop.mapreduce.lib.input.FileSplit;
 import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
 import org.apache.hadoop.util.ReflectionUtils;
 import org.junit.Test;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.io.BufferedReader;
 import java.io.BufferedWriter;
@@ -54,7 +54,8 @@ import static org.junit.Assert.assertTrue;
  */
 public class TestLocalRunner {
 
-  private static final Log LOG = LogFactory.getLog(TestLocalRunner.class);
+  private static final Logger LOG =
+      LoggerFactory.getLogger(TestLocalRunner.class);
 
   private static int INPUT_SIZES[] =
     new int[] { 50000, 500, 500, 20,  5000, 500};
@@ -330,9 +331,9 @@ public class TestLocalRunner {
     try {
       job.waitForCompletion(true);
     } catch (InterruptedException ie) {
-      LOG.fatal("Interrupted while waiting for job completion", ie);
+      LOG.error("Interrupted while waiting for job completion", ie);
       for (int i = 0; i < 10; i++) {
-        LOG.fatal("Dumping stacks");
+        LOG.error("Dumping stacks");
         ReflectionUtils.logThreadInfo(LOG, "multimap threads", 0);
         Thread.sleep(1000);
       }
