@@ -17,8 +17,6 @@
  */
 package org.apache.hadoop.mapreduce;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.LocatedFileStatus;
@@ -34,6 +32,8 @@ import org.codehaus.jettison.json.JSONException;
 import org.codehaus.jettison.json.JSONObject;
 import org.junit.BeforeClass;
 import org.junit.Test;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
@@ -47,6 +47,7 @@ import java.io.OutputStream;
 import java.io.PipedInputStream;
 import java.io.PipedOutputStream;
 import java.io.PrintStream;
+import java.nio.charset.StandardCharsets;
 import java.util.Arrays;
 
 import static org.junit.Assert.assertEquals;
@@ -61,7 +62,8 @@ import static org.junit.Assert.fail;
  */
 public class TestMRJobClient extends ClusterMapReduceTestCase {
 
-  private static final Log LOG = LogFactory.getLog(TestMRJobClient.class);
+  private static final Logger LOG =
+      LoggerFactory.getLogger(TestMRJobClient.class);
 
   @BeforeClass
   public static void setupClass() throws Exception {
@@ -192,7 +194,7 @@ public class TestMRJobClient extends ClusterMapReduceTestCase {
     assertEquals("Exit code", -1, exitCode);
 
     runTool(conf, jc, new String[] { "-fail-task", taid.toString() }, out);
-    String answer = new String(out.toByteArray(), "UTF-8");
+    String answer = new String(out.toByteArray(), StandardCharsets.UTF_8);
     assertTrue(answer.contains("Killed task " + taid + " by failing it"));
   }
 
@@ -210,7 +212,7 @@ public class TestMRJobClient extends ClusterMapReduceTestCase {
     assertEquals("Exit code", -1, exitCode);
 
     runTool(conf, jc, new String[] { "-kill-task", taid.toString() }, out);
-    String answer = new String(out.toByteArray(), "UTF-8");
+    String answer = new String(out.toByteArray(), StandardCharsets.UTF_8);
     assertTrue(answer.contains("Killed task " + taid));
   }
   
@@ -230,7 +232,7 @@ public class TestMRJobClient extends ClusterMapReduceTestCase {
     exitCode = runTool(conf, jc, new String[] { "-kill", jobId }, out);
     assertEquals("Exit code", 0, exitCode);
     
-    String answer = new String(out.toByteArray(), "UTF-8");
+    String answer = new String(out.toByteArray(), StandardCharsets.UTF_8);
     assertTrue(answer.contains("Killed job " + jobId));
   }
 
@@ -720,6 +722,50 @@ public class TestMRJobClient extends ClusterMapReduceTestCase {
     verifyJobPriority(jobId, "DEFAULT", conf, createJobClient());
   }
 
+  /**
+   * Test -list option displays job name.
+   * The name is capped to 20 characters for display.
+   */
+  @Test
+  public void testJobName() throws Exception {
+    Configuration conf = createJobConf();
+    CLI jc = createJobClient();
+    Job job = MapReduceTestUtil.createJob(conf, getInputDir(), getOutputDir(),
+        1, 1, "short_name");
+    job.setJobName("mapreduce");
+    job.setPriority(JobPriority.NORMAL);
+    job.waitForCompletion(true);
+    String jobId = job.getJobID().toString();
+    verifyJobName(jobId, "mapreduce", conf, jc);
+    Job job2 = MapReduceTestUtil.createJob(conf, getInputDir(), getOutputDir(),
+        1, 1, "long_name");
+    job2.setJobName("mapreduce_job_with_long_name");
+    job2.setPriority(JobPriority.NORMAL);
+    job2.waitForCompletion(true);
+    jobId = job2.getJobID().toString();
+    verifyJobName(jobId, "mapreduce_job_with_l", conf, jc);
+  }
+
+  protected void verifyJobName(String jobId, String name,
+      Configuration conf, CLI jc) throws Exception {
+    PipedInputStream pis = new PipedInputStream();
+    PipedOutputStream pos = new PipedOutputStream(pis);
+    int exitCode = runTool(conf, jc,
+        new String[] { "-list", "all" }, pos);
+    assertEquals("Exit code", 0, exitCode);
+    BufferedReader br = new BufferedReader(new InputStreamReader(pis));
+    String line = null;
+    while ((line = br.readLine()) != null) {
+      LOG.info("line = " + line);
+      if (!line.contains(jobId)) {
+        continue;
+      }
+      assertTrue(line.contains(name));
+      break;
+    }
+    pis.close();
+  }
+
   protected CLI createJobClient() throws IOException {
     return new CLI();
   }
