@@ -18,14 +18,14 @@
 
 package org.apache.hadoop.fs.s3a.auth;
 
-import com.amazonaws.AmazonClientException;
+import org.apache.hadoop.fs.s3a.CredentialInitializationException;
 
 /**
- * A specific subclass of {@code AmazonClientException} which can
- * be used in the retry logic to fail fast when there is any
+ * A specific subclass of {@code SdkException} which is
+ * used in the S3A retry policy to fail fast when there is any
  * authentication problem.
  */
-public class NoAuthWithAWSException extends AmazonClientException {
+public class NoAuthWithAWSException extends CredentialInitializationException {
 
   public NoAuthWithAWSException(final String message, final Throwable t) {
     super(message, t);
