@@ -26,21 +26,18 @@ import java.util.Optional;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 
-import com.amazonaws.ClientConfiguration;
-import com.amazonaws.auth.AWSCredentials;
-import com.amazonaws.auth.AWSSessionCredentials;
-import com.amazonaws.services.securitytoken.AWSSecurityTokenService;
-import org.apache.hadoop.thirdparty.com.google.common.annotations.VisibleForTesting;
+import software.amazon.awssdk.auth.credentials.AwsCredentials;
+import software.amazon.awssdk.auth.credentials.AwsSessionCredentials;
+import software.amazon.awssdk.services.sts.StsClient;
+import org.apache.hadoop.classification.VisibleForTesting;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.s3a.AWSCredentialProviderList;
-import org.apache.hadoop.fs.s3a.Constants;
 import org.apache.hadoop.fs.s3a.Invoker;
 import org.apache.hadoop.fs.s3a.Retries;
 import org.apache.hadoop.fs.s3a.S3ARetryPolicy;
-import org.apache.hadoop.fs.s3a.S3AUtils;
 import org.apache.hadoop.fs.s3a.auth.MarshalledCredentialProvider;
 import org.apache.hadoop.fs.s3a.auth.MarshalledCredentials;
 import org.apache.hadoop.fs.s3a.auth.RoleModel;
@@ -50,8 +47,8 @@ import org.apache.hadoop.io.Text;
 
 import static org.apache.hadoop.fs.s3a.Constants.AWS_CREDENTIALS_PROVIDER;
 import static org.apache.hadoop.fs.s3a.Invoker.once;
-import static org.apache.hadoop.fs.s3a.S3AUtils.STANDARD_AWS_PROVIDERS;
-import static org.apache.hadoop.fs.s3a.S3AUtils.buildAWSProviderList;
+import static org.apache.hadoop.fs.s3a.auth.CredentialProviderListFactory.STANDARD_AWS_PROVIDERS;
+import static org.apache.hadoop.fs.s3a.auth.CredentialProviderListFactory.buildAWSProviderList;
 import static org.apache.hadoop.fs.s3a.auth.MarshalledCredentialBinding.fromAWSCredentials;
 import static org.apache.hadoop.fs.s3a.auth.MarshalledCredentialBinding.fromSTSCredentials;
 import static org.apache.hadoop.fs.s3a.auth.delegation.DelegationConstants.*;
@@ -105,7 +102,8 @@ public class SessionTokenBinding extends AbstractDelegationTokenBinding {
   private boolean hasSessionCreds;
 
   /**
-   * The auth chain for the parent options.
+   * The parent authentication chain: that used to request
+   * session/role credentials when deployed unbonded.
    */
   private AWSCredentialProviderList parentAuthChain;
 
@@ -164,12 +162,14 @@ public class SessionTokenBinding extends AbstractDelegationTokenBinding {
         DEFAULT_DELEGATION_TOKEN_REGION);
 
     // create the provider set for session credentials.
-    parentAuthChain = buildAWSProviderList(
+    final AWSCredentialProviderList chain = buildAWSProviderList(
         getCanonicalUri(),
         conf,
         AWS_CREDENTIALS_PROVIDER,
         STANDARD_AWS_PROVIDERS,
         new HashSet<>());
+    LOG.debug("Setting parent authentication chain to {}", chain);
+    setParentAuthChain(chain);
   }
 
   @Override
@@ -192,7 +192,7 @@ public class SessionTokenBinding extends AbstractDelegationTokenBinding {
   public AWSCredentialProviderList deployUnbonded()
       throws IOException {
     requireServiceStarted();
-    return parentAuthChain;
+    return getParentAuthChain();
   }
 
   /**
@@ -292,23 +292,22 @@ public class SessionTokenBinding extends AbstractDelegationTokenBinding {
     // chain.
     // As no codepath (session propagation, STS creation) will work,
     // throw this.
-    final AWSCredentials parentCredentials = once("get credentials",
+    final AwsCredentials parentCredentials = once("get credentials",
         "",
-        () -> parentAuthChain.getCredentials());
-    hasSessionCreds = parentCredentials instanceof AWSSessionCredentials;
+        () -> getParentAuthChain().resolveCredentials());
+    hasSessionCreds = parentCredentials instanceof AwsSessionCredentials;
 
     if (!hasSessionCreds) {
       LOG.debug("Creating STS client for {}", getDescription());
 
       invoker = new Invoker(new S3ARetryPolicy(conf), LOG_EVENT);
-      ClientConfiguration awsConf =
-          S3AUtils.createAwsConf(conf, uri.getHost(),
-              Constants.AWS_SERVICE_IDENTIFIER_STS);
-      AWSSecurityTokenService tokenService =
-          STSClientFactory.builder(parentAuthChain,
-              awsConf,
+
+      StsClient tokenService =
+          STSClientFactory.builder(getParentAuthChain(),
+              conf,
               endpoint,
-              region)
+              region,
+              uri.getHost())
               .build();
       stsClient = Optional.of(
           STSClientFactory.createClientConnection(tokenService, invoker));
@@ -374,11 +373,11 @@ public class SessionTokenBinding extends AbstractDelegationTokenBinding {
             + " -duration unknown", getCanonicalUri());
       }
       origin += " " + CREDENTIALS_CONVERTED_TO_DELEGATION_TOKEN;
-      final AWSCredentials awsCredentials
-          = parentAuthChain.getCredentials();
-      if (awsCredentials instanceof AWSSessionCredentials) {
+      final AwsCredentials awsCredentials
+          = getParentAuthChain().resolveCredentials();
+      if (awsCredentials instanceof AwsSessionCredentials) {
         marshalledCredentials = fromAWSCredentials(
-            (AWSSessionCredentials) awsCredentials);
+            (AwsSessionCredentials) awsCredentials);
       } else {
         throw new DelegationTokenIOException(
             "AWS Authentication chain is no longer supplying session secrets");
@@ -425,4 +424,16 @@ public class SessionTokenBinding extends AbstractDelegationTokenBinding {
       tokenIdentifier) {
     this.tokenIdentifier = tokenIdentifier;
   }
+
+  /**
+   * The auth chain for the parent options.
+   * @return the parent authentication chain.
+   */
+  protected AWSCredentialProviderList getParentAuthChain() {
+    return parentAuthChain;
+  }
+
+  protected void setParentAuthChain(AWSCredentialProviderList parentAuthChain) {
+    this.parentAuthChain = parentAuthChain;
+  }
 }
