@@ -44,9 +44,6 @@ import static org.apache.hadoop.fs.s3a.Constants.STORE_CAPABILITY_DIRECTORY_MARK
 
 /**
  * Implementation of directory policy.
- * <p> </p>
- * As only the DELETE policy is supported, the policy logic here is relatively
- * straightforward.
  */
 public final class DirectoryPolicyImpl
     implements DirectoryPolicy {
@@ -65,7 +62,13 @@ public final class DirectoryPolicyImpl
    * All available policies.
    */
   private static final Set<MarkerPolicy> AVAILABLE_POLICIES =
-      EnumSet.of(MarkerPolicy.Delete);
+      EnumSet.allOf(MarkerPolicy.class);
+
+  /**
+   * Keep all markers.
+   */
+  public static final DirectoryPolicy KEEP = new DirectoryPolicyImpl(
+      MarkerPolicy.Keep, (p) -> false);
 
   /**
    * Delete all markers.
@@ -78,6 +81,12 @@ public final class DirectoryPolicyImpl
    */
   private final MarkerPolicy markerPolicy;
 
+  /**
+   * Callback to evaluate authoritativeness of a
+   * path.
+   */
+  private final Predicate<Path> authoritativeness;
+
   /**
    * Constructor.
    * @param markerPolicy marker policy
@@ -86,11 +95,20 @@ public final class DirectoryPolicyImpl
   public DirectoryPolicyImpl(final MarkerPolicy markerPolicy,
       final Predicate<Path> authoritativeness) {
     this.markerPolicy = markerPolicy;
+    this.authoritativeness = authoritativeness;
   }
 
   @Override
   public boolean keepDirectoryMarkers(final Path path) {
-    return false;
+    switch (markerPolicy) {
+    case Keep:
+      return true;
+    case Authoritative:
+      return authoritativeness.test(path);
+    case Delete:
+    default:   // which cannot happen
+      return false;
+    }
   }
 
   @Override
@@ -129,13 +147,19 @@ public final class DirectoryPolicyImpl
       return true;
 
     case STORE_CAPABILITY_DIRECTORY_MARKER_POLICY_KEEP:
-    case STORE_CAPABILITY_DIRECTORY_MARKER_ACTION_DELETE:
-      return true;
+      return markerPolicy == MarkerPolicy.Keep;
 
     case STORE_CAPABILITY_DIRECTORY_MARKER_POLICY_DELETE:
+      return markerPolicy == MarkerPolicy.Delete;
+
     case STORE_CAPABILITY_DIRECTORY_MARKER_POLICY_AUTHORITATIVE:
+      return markerPolicy == MarkerPolicy.Authoritative;
+
     case STORE_CAPABILITY_DIRECTORY_MARKER_ACTION_KEEP:
-      return false;
+      return keepDirectoryMarkers(path);
+
+    case STORE_CAPABILITY_DIRECTORY_MARKER_ACTION_DELETE:
+      return !keepDirectoryMarkers(path);
 
     default:
       throw new IllegalArgumentException("Unknown capability " + capability);
@@ -145,28 +169,36 @@ public final class DirectoryPolicyImpl
   /**
    * Create/Get the policy for this configuration.
    * @param conf config
+   * @param authoritativeness Callback to evaluate authoritativeness of a
+   * path.
    * @return a policy
    */
   public static DirectoryPolicy getDirectoryPolicy(
-      final Configuration conf) {
+      final Configuration conf,
+      final Predicate<Path> authoritativeness) {
+    DirectoryPolicy policy;
     String option = conf.getTrimmed(DIRECTORY_MARKER_POLICY,
-        DEFAULT_DIRECTORY_MARKER_POLICY)
-        .toLowerCase(Locale.ENGLISH);
-    switch (option) {
+        DEFAULT_DIRECTORY_MARKER_POLICY);
+    switch (option.toLowerCase(Locale.ENGLISH)) {
     case DIRECTORY_MARKER_POLICY_DELETE:
-      // only supported policy
+      // backwards compatible.
       LOG.debug("Directory markers will be deleted");
+      policy = DELETE;
       break;
     case DIRECTORY_MARKER_POLICY_KEEP:
+      LOG.debug("Directory markers will be kept");
+      policy = KEEP;
+      break;
     case DIRECTORY_MARKER_POLICY_AUTHORITATIVE:
-      // known but not available.
-      LOG.info("Directory marker policy \"{}\" is unsupported,"
-              + " using \"delete\"", option);
+      LOG.debug("Directory markers will be kept on authoritative"
+          + " paths");
+      policy = new DirectoryPolicyImpl(MarkerPolicy.Authoritative,
+          authoritativeness);
       break;
     default:
       throw new IllegalArgumentException(UNKNOWN_MARKER_POLICY + option);
     }
-    return DELETE;
+    return policy;
   }
 
   /**
