@@ -171,8 +171,11 @@ public class SDKStreamDrainer<TStream extends InputStream & Abortable>
           "duplicate invocation of drain operation");
     }
     boolean executeAbort = shouldAbort;
-    LOG.debug("drain or abort reason {} remaining={} abort={}",
-        reason, remaining, executeAbort);
+    if (remaining > 0 || executeAbort) {
+      // only log if there is a drain or an abort
+      LOG.debug("drain or abort reason {} remaining={} abort={}",
+          reason, remaining, executeAbort);
+    }
 
     if (!executeAbort) {
       try {
