@@ -19,6 +19,7 @@
 package org.apache.hadoop.fs.s3a.statistics;
 
 import org.apache.hadoop.fs.impl.prefetch.PrefetchingStatistics;
+import org.apache.hadoop.fs.s3a.impl.streams.InputStreamType;
 import org.apache.hadoop.fs.statistics.DurationTracker;
 
 /**
@@ -53,6 +54,13 @@ public interface S3AInputStreamStatistics extends AutoCloseable,
    */
   long streamOpened();
 
+  /**
+   * A stream of the given type was opened.
+   * @param type type of input stream
+   *  @return the previous count or zero if this is the first opening.
+   */
+  long streamOpened(InputStreamType type);
+
   /**
    * The inner stream was closed.
    * @param abortedConnection flag to indicate the stream was aborted,
@@ -210,4 +218,8 @@ public interface S3AInputStreamStatistics extends AutoCloseable,
    */
   DurationTracker initiateInnerStreamClose(boolean abort);
 
+  /**
+   * Stream was leaked.
+   */
+  default void streamLeaked() {};
 }
