@@ -19,43 +19,39 @@ package org.apache.hadoop.fs.s3a.auth;
 
 import java.io.Closeable;
 import java.io.IOException;
-import java.net.URI;
-import java.net.URISyntaxException;
 import java.nio.charset.StandardCharsets;
 import java.security.PrivilegedExceptionAction;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Objects;
-import java.util.concurrent.TimeUnit;
 
-import com.amazonaws.AmazonWebServiceRequest;
-import com.amazonaws.DefaultRequest;
-import com.amazonaws.SignableRequest;
-import com.amazonaws.auth.AWSCredentials;
-import com.amazonaws.auth.Signer;
-import com.amazonaws.auth.SignerFactory;
+import software.amazon.awssdk.core.interceptor.ExecutionAttributes;
+import software.amazon.awssdk.core.signer.Signer;
+import software.amazon.awssdk.http.SdkHttpFullRequest;
+import software.amazon.awssdk.http.SdkHttpMethod;
 import org.assertj.core.api.Assertions;
 import org.junit.Before;
-import org.junit.Rule;
 import org.junit.Test;
-import org.junit.rules.Timeout;
 
 import org.apache.hadoop.classification.InterfaceAudience.Private;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.s3a.auth.TestSignerManager.SignerInitializerForTest.StoreValue;
 import org.apache.hadoop.fs.s3a.auth.delegation.DelegationTokenProvider;
+import org.apache.hadoop.fs.s3a.impl.InstantiationIOException;
 import org.apache.hadoop.io.Text;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.security.token.Token;
 import org.apache.hadoop.security.token.TokenIdentifier;
-import org.apache.hadoop.test.LambdaTestUtils;
+import org.apache.hadoop.test.AbstractHadoopTestBase;
 
 import static org.apache.hadoop.fs.s3a.Constants.CUSTOM_SIGNERS;
+import static org.apache.hadoop.fs.s3a.auth.SignerFactory.S3_V2_SIGNER;
+import static org.apache.hadoop.test.LambdaTestUtils.intercept;
 
 /**
  * Tests for the SignerManager.
  */
-public class TestSignerManager {
+public class TestSignerManager extends AbstractHadoopTestBase {
 
   private static final Text TEST_TOKEN_KIND = new Text("TestTokenKind");
   private static final Text TEST_TOKEN_SERVICE = new Text("TestTokenService");
@@ -65,9 +61,6 @@ public class TestSignerManager {
   private static final String TESTUSER1 = "testuser1";
   private static final String TESTUSER2 = "testuser2";
 
-  @Rule public Timeout testTimeout = new Timeout(10_000L,
-      TimeUnit.MILLISECONDS);
-
   @Before
   public void beforeTest() {
     SignerForTest1.reset();
@@ -99,11 +92,8 @@ public class TestSignerManager {
     // Make sure the config is respected.
     signerManager.initCustomSigners();
     // Simulate a call from the AWS SDK to create the signer.
-    LambdaTestUtils.intercept(Exception.class,
+    intercept(InstantiationIOException.class,
         () -> SignerFactory.createSigner("testsignerUnregistered", null));
-    // Expecting generic Exception.class to handle future implementation
-    // changes.
-    // For now, this is an NPE
   }
 
   @Test
@@ -284,7 +274,7 @@ public class TestSignerManager {
       throws IOException, InterruptedException {
     ugi.doAs((PrivilegedExceptionAction<Void>) () -> {
       Signer signer = new SignerForInitializerTest();
-      SignableRequest<?> signableRequest = constructSignableRequest(bucket);
+      SdkHttpFullRequest signableRequest = constructSignableRequest(bucket);
       signer.sign(signableRequest, null);
       verifyStoreValueInSigner(expectNullStoreInfo, bucket, identifier);
       return null;
@@ -336,8 +326,10 @@ public class TestSignerManager {
     private static boolean initialized = false;
 
     @Override
-    public void sign(SignableRequest<?> request, AWSCredentials credentials) {
+    public SdkHttpFullRequest sign(SdkHttpFullRequest sdkHttpFullRequest,
+        ExecutionAttributes executionAttributes) {
       initialized = true;
+      return sdkHttpFullRequest;
     }
 
     public static void reset() {
@@ -354,8 +346,10 @@ public class TestSignerManager {
     private static boolean initialized = false;
 
     @Override
-    public void sign(SignableRequest<?> request, AWSCredentials credentials) {
+    public SdkHttpFullRequest sign(SdkHttpFullRequest sdkHttpFullRequest,
+        ExecutionAttributes executionAttributes) {
       initialized = true;
+      return sdkHttpFullRequest;
     }
 
     public static void reset() {
@@ -472,11 +466,15 @@ public class TestSignerManager {
     private static StoreValue retrievedStoreValue;
 
     @Override
-    public void sign(SignableRequest<?> request, AWSCredentials credentials) {
-      String bucketName = request.getEndpoint().getHost();
+    public SdkHttpFullRequest sign(SdkHttpFullRequest sdkHttpFullRequest,
+        ExecutionAttributes executionAttributes) {
+      String bucket = sdkHttpFullRequest.host().split("//")[1];
+      // remove trailing slash
+      String bucketName = bucket.substring(0, bucket.length() - 1);
       try {
         retrievedStoreValue = SignerInitializerForTest
             .getStoreInfo(bucketName, UserGroupInformation.getCurrentUser());
+        return sdkHttpFullRequest;
       } catch (IOException e) {
         throw new RuntimeException("Failed to get current ugi", e);
       }
@@ -579,12 +577,22 @@ public class TestSignerManager {
     return identifier + "_" + bucketName + "_" + user;
   }
 
-  private SignableRequest<?> constructSignableRequest(String bucketName)
-      throws URISyntaxException {
-    DefaultRequest signableRequest = new DefaultRequest(
-        AmazonWebServiceRequest.NOOP, "fakeservice");
-    URI uri = new URI("s3://" + bucketName + "/");
-    signableRequest.setEndpoint(uri);
-    return signableRequest;
+  private SdkHttpFullRequest constructSignableRequest(String bucketName) {
+    String host = "s3://" + bucketName + "/";
+    return SdkHttpFullRequest.builder().host(host).protocol("https").method(SdkHttpMethod.GET)
+        .build();
   }
+
+  @Test
+  public void testV2SignerRejected() throws Throwable {
+    intercept(InstantiationIOException.class, "no longer supported",
+        () -> SignerFactory.createSigner(S3_V2_SIGNER, "key"));
+  }
+
+  @Test
+  public void testUnknownSignerRejected() throws Throwable {
+    intercept(InstantiationIOException.class, "unknownSigner",
+        () -> SignerFactory.createSigner("unknownSigner", "key"));
+  }
+
 }
