@@ -22,64 +22,76 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.function.Predicate;
 
+import org.assertj.core.api.Assertions;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
 
-import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.Path;
-import org.apache.hadoop.test.HadoopTestBase;
+import org.apache.hadoop.test.AbstractHadoopTestBase;
 
-import static org.apache.hadoop.fs.s3a.Constants.DIRECTORY_MARKER_POLICY;
 import static org.apache.hadoop.fs.s3a.Constants.STORE_CAPABILITY_DIRECTORY_MARKER_ACTION_DELETE;
 import static org.apache.hadoop.fs.s3a.Constants.STORE_CAPABILITY_DIRECTORY_MARKER_ACTION_KEEP;
 
 /**
  * Unit tests for directory marker policies.
- * <p></p>
- * As this FS only supports "delete", this is more minimal test
- * suite than on later versions.
- * <p></p>
- * It helps ensure that there aren't unexpected problems if the site configuration
- * asks for retention of some form.
  */
 @RunWith(Parameterized.class)
-public class TestDirectoryMarkerPolicy extends HadoopTestBase {
+public class TestDirectoryMarkerPolicy extends AbstractHadoopTestBase {
 
   @Parameterized.Parameters(name = "{0}")
   public static Collection<Object[]> data() {
     return Arrays.asList(new Object[][]{
         {
             DirectoryPolicy.MarkerPolicy.Delete,
+            FAIL_IF_INVOKED,
+            false, false
         },
         {
             DirectoryPolicy.MarkerPolicy.Keep,
+            FAIL_IF_INVOKED,
+            true, true
         },
         {
             DirectoryPolicy.MarkerPolicy.Authoritative,
+            AUTH_PATH_ONLY,
+            false, true
         }
     });
   }
 
   private final DirectoryPolicy directoryPolicy;
 
-  private static final boolean EXPECT_MARKER_RETENTION = false;
+  private final boolean expectNonAuthDelete;
+
+  private final boolean expectAuthDelete;
 
   public TestDirectoryMarkerPolicy(
-      final DirectoryPolicy.MarkerPolicy markerPolicy) {
-    this.directoryPolicy = newPolicy(markerPolicy);
+      final DirectoryPolicy.MarkerPolicy markerPolicy,
+      final Predicate<Path> authoritativeness,
+      final boolean expectNonAuthDelete,
+      final boolean expectAuthDelete) {
+    this.directoryPolicy = newPolicy(markerPolicy, authoritativeness);
+    this.expectNonAuthDelete = expectNonAuthDelete;
+    this.expectAuthDelete = expectAuthDelete;
   }
 
   /**
    * Create a new retention policy.
    * @param markerPolicy policy option
+   * @param authoritativeness predicate for determining if
+   * a path is authoritative.
    * @return the retention policy.
    */
   private DirectoryPolicy newPolicy(
-      DirectoryPolicy.MarkerPolicy markerPolicy) {
-    return new DirectoryPolicyImpl(markerPolicy, FAIL_IF_INVOKED);
+      DirectoryPolicy.MarkerPolicy markerPolicy,
+      Predicate<Path> authoritativeness) {
+    return new DirectoryPolicyImpl(markerPolicy, authoritativeness);
   }
 
+  private static final Predicate<Path> AUTH_PATH_ONLY =
+      (p) -> p.toUri().getPath().startsWith("/auth/");
+
   private static final Predicate<Path> FAIL_IF_INVOKED = (p) -> {
     throw new RuntimeException("failed");
   };
@@ -96,9 +108,9 @@ public class TestDirectoryMarkerPolicy extends HadoopTestBase {
    * @param retain should the marker be retained
    */
   private void assertMarkerRetention(Path path, boolean retain) {
-    assertEquals("Retention of path " + path + " by " + directoryPolicy,
-        retain,
-        directoryPolicy.keepDirectoryMarkers(path));
+    Assertions.assertThat(directoryPolicy.keepDirectoryMarkers(path))
+        .describedAs("Retention of path %s by %s", path, directoryPolicy)
+        .isEqualTo(retain);
   }
 
   /**
@@ -107,54 +119,45 @@ public class TestDirectoryMarkerPolicy extends HadoopTestBase {
   private void assertPathCapability(Path path,
       String capability,
       boolean outcome) {
-    assertEquals(String.format(
-        "%s support for capability %s by path %s expected as %s",
-        directoryPolicy, capability, path, outcome),
-        outcome,
-        directoryPolicy.hasPathCapability(path, capability));
+    Assertions.assertThat(directoryPolicy)
+        .describedAs("%s support for capability %s by path %s"
+                + " expected as %s",
+            directoryPolicy, capability, path, outcome)
+        .matches(p -> p.hasPathCapability(path, capability) == outcome,
+            "pathCapability");
   }
 
   @Test
   public void testNonAuthPath() throws Throwable {
-    assertMarkerRetention(nonAuthPath, EXPECT_MARKER_RETENTION);
+    assertMarkerRetention(nonAuthPath, expectNonAuthDelete);
     assertPathCapability(nonAuthPath,
         STORE_CAPABILITY_DIRECTORY_MARKER_ACTION_DELETE,
-        !EXPECT_MARKER_RETENTION);
+        !expectNonAuthDelete);
     assertPathCapability(nonAuthPath,
         STORE_CAPABILITY_DIRECTORY_MARKER_ACTION_KEEP,
-        EXPECT_MARKER_RETENTION);
+        expectNonAuthDelete);
   }
 
   @Test
   public void testAuthPath() throws Throwable {
-    assertMarkerRetention(authPath, EXPECT_MARKER_RETENTION);
+    assertMarkerRetention(authPath, expectAuthDelete);
     assertPathCapability(authPath,
         STORE_CAPABILITY_DIRECTORY_MARKER_ACTION_DELETE,
-        !EXPECT_MARKER_RETENTION);
+        !expectAuthDelete);
     assertPathCapability(authPath,
         STORE_CAPABILITY_DIRECTORY_MARKER_ACTION_KEEP,
-        EXPECT_MARKER_RETENTION);
+        expectAuthDelete);
   }
 
   @Test
   public void testDeepAuthPath() throws Throwable {
-    assertMarkerRetention(deepAuth, EXPECT_MARKER_RETENTION);
+    assertMarkerRetention(deepAuth, expectAuthDelete);
     assertPathCapability(deepAuth,
         STORE_CAPABILITY_DIRECTORY_MARKER_ACTION_DELETE,
-        !EXPECT_MARKER_RETENTION);
+        !expectAuthDelete);
     assertPathCapability(deepAuth,
         STORE_CAPABILITY_DIRECTORY_MARKER_ACTION_KEEP,
-        EXPECT_MARKER_RETENTION);
+        expectAuthDelete);
   }
 
-  @Test
-  public void testInstantiate() throws Throwable {
-    Configuration conf = new Configuration(false);
-    conf.set(DIRECTORY_MARKER_POLICY,
-        directoryPolicy.getMarkerPolicy().getOptionName());
-    DirectoryPolicy policy = DirectoryPolicyImpl.getDirectoryPolicy(
-        conf);
-    assertEquals(DirectoryPolicyImpl.DELETE, policy);
-
-  }
 }
