@@ -21,11 +21,9 @@ package org.apache.hadoop.fs.s3a.scale;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.nio.ByteBuffer;
-import java.util.ArrayList;
+import java.time.Duration;
 import java.util.List;
 import java.util.NoSuchElementException;
-import java.util.concurrent.atomic.AtomicInteger;
-import java.util.concurrent.atomic.AtomicLong;
 import java.util.function.IntFunction;
 
 import org.assertj.core.api.Assertions;
@@ -47,20 +45,17 @@ import org.apache.hadoop.fs.s3a.Constants;
 import org.apache.hadoop.fs.s3a.S3AFileSystem;
 import org.apache.hadoop.fs.s3a.S3ATestUtils;
 import org.apache.hadoop.fs.s3a.Statistic;
-import org.apache.hadoop.fs.s3a.impl.ProgressListener;
-import org.apache.hadoop.fs.s3a.impl.ProgressListenerEvent;
 import org.apache.hadoop.fs.s3a.statistics.BlockOutputStreamStatistics;
 import org.apache.hadoop.fs.statistics.IOStatistics;
+import org.apache.hadoop.io.ElasticByteBufferPool;
+import org.apache.hadoop.io.WeakReferencedElasticByteBufferPool;
 import org.apache.hadoop.util.DurationInfo;
-import org.apache.hadoop.util.Progressable;
 
 import static java.util.Objects.requireNonNull;
 import static org.apache.hadoop.fs.Options.OpenFileOptions.FS_OPTION_OPENFILE_BUFFER_SIZE;
 import static org.apache.hadoop.fs.Options.OpenFileOptions.FS_OPTION_OPENFILE_LENGTH;
 import static org.apache.hadoop.fs.Options.OpenFileOptions.FS_OPTION_OPENFILE_READ_POLICY;
 import static org.apache.hadoop.fs.Options.OpenFileOptions.FS_OPTION_OPENFILE_READ_POLICY_WHOLE_FILE;
-import static org.apache.hadoop.fs.Options.OpenFileOptions.FS_OPTION_OPENFILE_SPLIT_END;
-import static org.apache.hadoop.fs.Options.OpenFileOptions.FS_OPTION_OPENFILE_SPLIT_START;
 import static org.apache.hadoop.fs.contract.ContractTestUtils.*;
 import static org.apache.hadoop.fs.contract.ContractTestUtils.validateVectoredReadResult;
 import static org.apache.hadoop.fs.s3a.Constants.*;
@@ -78,11 +73,11 @@ import static org.apache.hadoop.util.functional.RemoteIterators.filteringRemoteI
 
 /**
  * Scale test which creates a huge file.
- *
+ * <p>
  * <b>Important:</b> the order in which these tests execute is fixed to
  * alphabetical order. Test cases are numbered {@code test_123_} to impose
  * an ordering based on the numbers.
- *
+ * <p>
  * Having this ordering allows the tests to assume that the huge file
  * exists. Even so: they should all have a {@link #assumeHugeFileExists()}
  * check at the start, in case an individual test is executed.
@@ -113,6 +108,16 @@ public abstract class AbstractSTestS3AHugeFiles extends S3AScaleTestBase {
         DEFAULT_HUGE_FILESIZE);
   }
 
+  /**
+   * Test dir deletion is removed from test case teardown so the
+   * subsequent tests see the output.
+   * @throws IOException failure
+   */
+  @Override
+  protected void deleteTestDirInTeardown() throws IOException {
+    /* no-op */
+  }
+
   /**
    * Get the name of this test suite, which is used in path generation.
    * Base implementation uses {@link #getBlockOutputBufferName()} for this.
@@ -213,7 +218,7 @@ public abstract class AbstractSTestS3AHugeFiles extends S3AScaleTestBase {
     ContractTestUtils.NanoTimer timer = new ContractTestUtils.NanoTimer();
     BlockOutputStreamStatistics streamStatistics;
     long blocksPer10MB = blocksPerMB * 10;
-    ProgressCallback progress = new ProgressCallback(timer);
+    CountingProgressListener progress = new CountingProgressListener(timer);
     try (FSDataOutputStream out = fs.create(fileToCreate,
         true,
         uploadBlockSize,
@@ -378,84 +383,6 @@ public abstract class AbstractSTestS3AHugeFiles extends S3AScaleTestBase {
     return partitionSize;
   }
 
-  /**
-   * Progress callback.
-   */
-  private final class ProgressCallback implements Progressable, ProgressListener {
-    private AtomicLong bytesTransferred = new AtomicLong(0);
-    private AtomicLong uploadEvents = new AtomicLong(0);
-    private AtomicInteger failures = new AtomicInteger(0);
-    private final ContractTestUtils.NanoTimer timer;
-
-    private ProgressCallback(NanoTimer timer) {
-      this.timer = timer;
-    }
-
-    @Override
-    public void progress() {
-    }
-
-    @Override
-    public void progressChanged(ProgressListenerEvent eventType, long transferredBytes) {
-
-      switch (eventType) {
-      case TRANSFER_PART_FAILED_EVENT:
-        // failure
-        failures.incrementAndGet();
-        LOG.warn("Transfer failure");
-        break;
-      case TRANSFER_PART_COMPLETED_EVENT:
-        // completion
-        bytesTransferred.addAndGet(transferredBytes);
-        long elapsedTime = timer.elapsedTime();
-        double elapsedTimeS = elapsedTime / 1.0e9;
-        long written = bytesTransferred.get();
-        long writtenMB = written / _1MB;
-        LOG.info(String.format(
-            "Event %s; total uploaded=%d MB in %.1fs;" +
-                " effective upload bandwidth = %.2f MB/s",
-            eventType,
-            writtenMB, elapsedTimeS, writtenMB / elapsedTimeS));
-        break;
-      case REQUEST_BYTE_TRANSFER_EVENT:
-        uploadEvents.incrementAndGet();
-        break;
-      default:
-        // nothing
-        break;
-      }
-    }
-
-    public String toString() {
-      String sb = "ProgressCallback{"
-          + "bytesTransferred=" + bytesTransferred.get() +
-          ", uploadEvents=" + uploadEvents.get() +
-          ", failures=" + failures.get() +
-          '}';
-      return sb;
-    }
-
-    /**
-     * Get the number of bytes transferred.
-     * @return byte count
-     */
-    private long getBytesTransferred() {
-      return bytesTransferred.get();
-    }
-
-    /**
-     * Get the number of event callbacks.
-     * @return count of byte transferred events.
-     */
-    private long getUploadEvents() {
-      return uploadEvents.get();
-    }
-
-    private void verifyNoFailures(String operation) {
-      assertEquals("Failures in " + operation + ": " + this, 0, failures.get());
-    }
-  }
-
   /**
    * Assume that the huge file exists; skip the test if it does not.
    * @throws IOException IO failure
@@ -584,54 +511,94 @@ public abstract class AbstractSTestS3AHugeFiles extends S3AScaleTestBase {
         toHuman(timer.nanosPerOperation(ops)));
   }
 
+  /**
+   * Should this test suite use direct buffers for
+   * the Vector IO operations?
+   * @return true if direct buffers are desired.
+   */
+  protected boolean isDirectVectorBuffer() {
+    return false;
+  }
+
   @Test
   public void test_045_vectoredIOHugeFile() throws Throwable {
     assumeHugeFileExists();
-    List<FileRange> rangeList = new ArrayList<>();
-    rangeList.add(FileRange.createFileRange(5856368, 116770));
-    rangeList.add(FileRange.createFileRange(3520861, 116770));
-    rangeList.add(FileRange.createFileRange(8191913, 116770));
-    rangeList.add(FileRange.createFileRange(1520861, 116770));
-    rangeList.add(FileRange.createFileRange(2520861, 116770));
-    rangeList.add(FileRange.createFileRange(9191913, 116770));
-    rangeList.add(FileRange.createFileRange(2820861, 156770));
-    IntFunction<ByteBuffer> allocate = ByteBuffer::allocate;
+    final ElasticByteBufferPool pool =
+              new WeakReferencedElasticByteBufferPool();
+    boolean direct = isDirectVectorBuffer();
+    IntFunction<ByteBuffer> allocate = size -> pool.getBuffer(direct, size);
+
+    // build a list of ranges for both reads.
+    final int rangeLength = 116770;
+    long base = 1520861;
+    long pos = base;
+    List<FileRange> rangeList = range(pos, rangeLength);
+    pos += rangeLength;
+    range(rangeList, pos, rangeLength);
+    pos += rangeLength;
+    range(rangeList, pos, rangeLength);
+    pos += rangeLength;
+    range(rangeList, pos, rangeLength);
+    pos += rangeLength;
+    range(rangeList, pos, rangeLength);
+    pos += rangeLength;
+    range(rangeList, pos, rangeLength);
+
     FileSystem fs = getFileSystem();
 
-    // read into a buffer first
-    // using sequential IO
+    final int validateSize = (int) totalReadSize(rangeList);
 
-    int validateSize = (int) Math.min(filesize, 10 * _1MB);
-    byte[] readFullRes;
-    IOStatistics sequentialIOStats, vectorIOStats;
+    // read the same ranges using readFully into a buffer.
+    // this is to both validate the range resolution logic,
+    // and to compare performance of sequential GET requests
+    // with the vector IO.
+    byte[] readFullRes = new byte[validateSize];
+    IOStatistics readIOStats, vectorIOStats;
+    DurationInfo readFullyTime = new DurationInfo(LOG, true, "Sequential read of %,d bytes",
+        validateSize);
     try (FSDataInputStream in = fs.openFile(hugefile)
-        .optLong(FS_OPTION_OPENFILE_LENGTH, validateSize)  // lets us actually force a shorter read
-        .optLong(FS_OPTION_OPENFILE_SPLIT_START, 0)
-        .opt(FS_OPTION_OPENFILE_SPLIT_END, validateSize)
-        .opt(FS_OPTION_OPENFILE_READ_POLICY, "sequential")
+        .optLong(FS_OPTION_OPENFILE_LENGTH, filesize)
+        .opt(FS_OPTION_OPENFILE_READ_POLICY, "random")
         .opt(FS_OPTION_OPENFILE_BUFFER_SIZE, uploadBlockSize)
-        .build().get();
-         DurationInfo ignored = new DurationInfo(LOG, "Sequential read of %,d bytes",
-             validateSize)) {
-      readFullRes = new byte[validateSize];
-      in.readFully(0, readFullRes);
-      sequentialIOStats = in.getIOStatistics();
+        .build().get()) {
+      for (FileRange range : rangeList) {
+        in.readFully(range.getOffset(),
+            readFullRes,
+            (int)(range.getOffset() - base),
+            range.getLength());
+      }
+      readIOStats = in.getIOStatistics();
+    } finally {
+      readFullyTime.close();
     }
 
     // now do a vector IO read
+    DurationInfo vectorTime = new DurationInfo(LOG, true, "Vector Read");
     try (FSDataInputStream in = fs.openFile(hugefile)
         .optLong(FS_OPTION_OPENFILE_LENGTH, filesize)
         .opt(FS_OPTION_OPENFILE_READ_POLICY, "vector, random")
-        .build().get();
-         DurationInfo ignored = new DurationInfo(LOG, "Vector Read")) {
-
+        .build().get()) {
+      // initiate the read.
       in.readVectored(rangeList, allocate);
-      // Comparing vectored read results with read fully.
-      validateVectoredReadResult(rangeList, readFullRes);
+      // Wait for the results and compare with read fully.
+      validateVectoredReadResult(rangeList, readFullRes, base);
       vectorIOStats = in.getIOStatistics();
+    } finally {
+      vectorTime.close();
+      // release the pool
+      pool.release();
     }
 
-    LOG.info("Bulk read IOStatistics={}", ioStatisticsToPrettyString(sequentialIOStats));
+    final Duration readFullyDuration = readFullyTime.asDuration();
+    final Duration vectorDuration = vectorTime.asDuration();
+    final Duration diff = readFullyDuration.minus(vectorDuration);
+    double ratio = readFullyDuration.toNanos() / (double) vectorDuration.toNanos();
+    String format = String.format("Vector read to %s buffer taking %s was %s faster than"
+            + " readFully() (%s); ratio=%,.2fX",
+        direct ? "direct" : "heap",
+        vectorDuration, diff, readFullyDuration, ratio);
+    LOG.info(format);
+    LOG.info("Bulk read IOStatistics={}", ioStatisticsToPrettyString(readIOStats));
     LOG.info("Vector IOStatistics={}", ioStatisticsToPrettyString(vectorIOStats));
   }
 
@@ -791,4 +758,6 @@ public abstract class AbstractSTestS3AHugeFiles extends S3AScaleTestBase {
     timer.end("time to delete %s", path);
   }
 
+
+
 }
