@@ -19,13 +19,9 @@
 
 package org.apache.hadoop.fs.adl.live;
 
-import org.junit.Test;
-
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.contract.AbstractContractRenameTest;
 import org.apache.hadoop.fs.contract.AbstractFSContract;
-import org.apache.hadoop.security.AccessControlException;
-import org.apache.hadoop.test.LambdaTestUtils;
 
 /**
  * Test rename contract test cases on Adl file system.
@@ -37,14 +33,4 @@ public class TestAdlContractRenameLive extends AbstractContractRenameTest {
     return new AdlStorageContract(configuration);
   }
 
-  /**
-   * ADL throws an Access Control Exception rather than return false.
-   * This is caught and its error text checked, to catch regressions.
-   */
-  @Test
-  public void testRenameFileUnderFile() throws Exception {
-    LambdaTestUtils.intercept(AccessControlException.class,
-        "Parent path is not a folder.",
-        super::testRenameFileUnderFile);
-  }
 }
