@@ -20,7 +20,7 @@ package org.apache.hadoop.fs.azurebfs.oauth2;
 
 import java.io.IOException;
 
-import org.apache.hadoop.thirdparty.com.google.common.base.Preconditions;
+import org.apache.hadoop.util.Preconditions;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
