@@ -18,6 +18,10 @@
 
 package org.apache.hadoop.fs.azurebfs.services;
 
+/**
+ * Implementation of {@link AbfsThrottlingIntercept} that does not throttle
+ * the ABFS process.
+ */
 final class AbfsNoOpThrottlingIntercept implements AbfsThrottlingIntercept {
 
   public static final AbfsNoOpThrottlingIntercept INSTANCE = new AbfsNoOpThrottlingIntercept();
@@ -25,11 +29,13 @@ final class AbfsNoOpThrottlingIntercept implements AbfsThrottlingIntercept {
   private AbfsNoOpThrottlingIntercept() {
   }
 
+  /**{@inheritDoc}*/
   @Override
   public void updateMetrics(final AbfsRestOperationType operationType,
-      final AbfsHttpOperation abfsHttpOperation) {
+      final AbfsHttpOperation httpOperation) {
   }
 
+  /**{@inheritDoc}*/
   @Override
   public void sendingRequest(final AbfsRestOperationType operationType,
       final AbfsCounters abfsCounters) {
