@@ -23,5 +23,6 @@ package org.apache.hadoop.fs.azurebfs.services;
 public enum AuthType {
     SharedKey,
     OAuth,
-    Custom
+    Custom,
+    SAS
 }
