@@ -21,7 +21,6 @@ package org.apache.hadoop.fs.azurebfs.services;
 import javax.crypto.Mac;
 import javax.crypto.spec.SecretKeySpec;
 import java.io.UnsupportedEncodingException;
-import java.net.HttpURLConnection;
 import java.net.URL;
 import java.net.URLDecoder;
 import java.text.DateFormat;
@@ -80,7 +79,7 @@ public class SharedKeyCredentials {
     initializeMac();
   }
 
-  public void signRequest(HttpURLConnection connection, final long contentLength) throws UnsupportedEncodingException {
+  public void signRequest(AbfsHttpOperation connection, final long contentLength) throws UnsupportedEncodingException {
 
     String gmtTime = getGMTTime();
     connection.setRequestProperty(HttpHeaderConfigurations.X_MS_DATE, gmtTime);
@@ -117,7 +116,7 @@ public class SharedKeyCredentials {
    * @param conn                the HttpURLConnection for the operation
    * @param canonicalizedString the canonicalized string to add the canonicalized headerst to.
    */
-  private static void addCanonicalizedHeaders(final HttpURLConnection conn, final StringBuilder canonicalizedString) {
+  private static void addCanonicalizedHeaders(final AbfsHttpOperation conn, final StringBuilder canonicalizedString) {
     // Look for header names that start with
     // HeaderNames.PrefixForStorageHeader
     // Then sort them in case-insensitive manner.
@@ -205,13 +204,13 @@ public class SharedKeyCredentials {
    */
   private static String canonicalizeHttpRequest(final URL address,
       final String accountName, final String method, final String contentType,
-      final long contentLength, final String date, final HttpURLConnection conn)
+      final long contentLength, final String date, final AbfsHttpOperation conn)
       throws UnsupportedEncodingException {
 
     // The first element should be the Method of the request.
     // I.e. GET, POST, PUT, or HEAD.
     final StringBuilder canonicalizedString = new StringBuilder(EXPECTED_BLOB_QUEUE_CANONICALIZED_STRING_LENGTH);
-    canonicalizedString.append(conn.getRequestMethod());
+    canonicalizedString.append(conn.getMethod());
 
     // The next elements are
     // If any element is missing it may be empty.
@@ -450,7 +449,7 @@ public class SharedKeyCredentials {
     return value.substring(spaceDex);
   }
 
-  private static String getHeaderValue(final HttpURLConnection conn, final String headerName, final String defaultValue) {
+  private static String getHeaderValue(final AbfsHttpOperation conn, final String headerName, final String defaultValue) {
     final String headerValue = conn.getRequestProperty(headerName);
     return headerValue == null ? defaultValue : headerValue;
   }
@@ -465,7 +464,7 @@ public class SharedKeyCredentials {
    *                      -1 if unknown
    * @return a canonicalized string.
    */
-  private String canonicalize(final HttpURLConnection conn,
+  private String canonicalize(final AbfsHttpOperation conn,
                               final String accountName,
                               final Long contentLength) throws UnsupportedEncodingException {
 
@@ -476,8 +475,8 @@ public class SharedKeyCredentials {
 
     String contentType = getHeaderValue(conn, HttpHeaderConfigurations.CONTENT_TYPE, "");
 
-    return canonicalizeHttpRequest(conn.getURL(), accountName,
-        conn.getRequestMethod(), contentType, contentLength, null, conn);
+    return canonicalizeHttpRequest(conn.getConnUrl(), accountName,
+        conn.getMethod(), contentType, contentLength, null, conn);
   }
 
   /**
