@@ -128,7 +128,7 @@ public class TestDistCpOptions {
         new Path("hdfs://localhost:8020/target/"));
     Assert.assertFalse(builder.build().shouldSkipCRC());
 
-    final DistCpOptions options = builder.withSyncFolder(true).withCRC(true)
+    final DistCpOptions options = builder.withSyncFolder(true).withSkipCRC(true)
         .build();
     Assert.assertTrue(options.shouldSyncFolder());
     Assert.assertTrue(options.shouldSkipCRC());
@@ -289,7 +289,7 @@ public class TestDistCpOptions {
         "atomicWorkPath=null, logPath=null, sourceFileListing=abc, " +
         "sourcePaths=null, targetPath=xyz, filtersFile='null', " +
         "blocksPerChunk=0, copyBufferSize=8192, verboseLog=false, " +
-        "directWrite=false, useiterator=false}";
+        "directWrite=false, useiterator=false, updateRoot=false}";
     String optionString = option.toString();
     Assert.assertEquals(val, optionString);
     Assert.assertNotSame(DistCpOptionSwitch.ATOMIC_COMMIT.toString(),
@@ -391,7 +391,7 @@ public class TestDistCpOptions {
           new Path("hdfs://localhost:8020/target/"))
           .withSyncFolder(true)
           .withAppend(true)
-          .withCRC(true)
+          .withSkipCRC(true)
           .build();
       fail("Append should fail if skipCrc option is specified");
     } catch (IllegalArgumentException e) {
@@ -563,4 +563,15 @@ public class TestDistCpOptions {
             "otherwise it may not be fetched properly",
             expectedValForEmptyConfigKey, config.get(""));
   }
+
+  @Test
+  public void testUpdateRoot() {
+    final DistCpOptions options = new DistCpOptions.Builder(
+        Collections.singletonList(
+            new Path("hdfs://localhost:8020/source")),
+        new Path("hdfs://localhost:8020/target/"))
+        .withUpdateRoot(true)
+        .build();
+    Assert.assertTrue(options.shouldUpdateRoot());
+  }
 }
