@@ -261,7 +261,7 @@ public class TestCopyMapper {
       System.out.println(fileStatus.getReplication());
     }
     finally {
-      IOUtils.cleanup(null, outputStream);
+      IOUtils.cleanupWithLogger(null, outputStream);
     }
   }
 
