@@ -25,7 +25,9 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.BlockingQueue;
+import java.util.concurrent.ConcurrentSkipListSet;
 import java.util.concurrent.LinkedBlockingQueue;
 
 import org.apache.hadoop.classification.InterfaceAudience.Private;
@@ -50,6 +52,7 @@ import org.apache.hadoop.yarn.api.records.ContainerLaunchContext;
 import org.apache.hadoop.yarn.api.records.ExecutionType;
 import org.apache.hadoop.yarn.api.records.ExecutionTypeRequest;
 import org.apache.hadoop.yarn.api.records.FinalApplicationStatus;
+import org.apache.hadoop.yarn.api.records.NodeId;
 import org.apache.hadoop.yarn.api.records.Priority;
 import org.apache.hadoop.yarn.api.records.ReservationId;
 import org.apache.hadoop.yarn.api.records.Resource;
@@ -59,6 +62,7 @@ import org.apache.hadoop.yarn.factories.RecordFactory;
 import org.apache.hadoop.yarn.factory.providers.RecordFactoryProvider;
 import org.apache.hadoop.yarn.security.AMRMTokenIdentifier;
 import org.apache.hadoop.yarn.server.resourcemanager.ResourceManager;
+import org.apache.hadoop.yarn.sls.AMDefinition;
 import org.apache.hadoop.yarn.sls.scheduler.SchedulerMetrics;
 import org.apache.hadoop.yarn.util.Records;
 import org.apache.hadoop.yarn.sls.scheduler.ContainerSimulator;
@@ -72,6 +76,7 @@ import org.slf4j.LoggerFactory;
 @Private
 @Unstable
 public abstract class AMSimulator extends TaskRunner.Task {
+  private static final long FINISH_TIME_NOT_INITIALIZED = Long.MIN_VALUE;
   // resource manager
   protected ResourceManager rm;
   // main
@@ -99,7 +104,7 @@ public abstract class AMSimulator extends TaskRunner.Task {
   protected long traceStartTimeMS;
   protected long traceFinishTimeMS;
   protected long simulateStartTimeMS;
-  protected long simulateFinishTimeMS;
+  protected long simulateFinishTimeMS = FINISH_TIME_NOT_INITIALIZED;
   // whether tracked in Metrics
   protected boolean isTracked;
   // progress
@@ -118,31 +123,31 @@ public abstract class AMSimulator extends TaskRunner.Task {
 
   private Map<ApplicationId, AMSimulator> appIdToAMSim;
 
+  private Set<NodeId> ranNodes = new ConcurrentSkipListSet<NodeId>();
+
   public AMSimulator() {
     this.responseQueue = new LinkedBlockingQueue<>();
   }
 
-  @SuppressWarnings("checkstyle:parameternumber")
-  public void init(int heartbeatInterval,
-      List<ContainerSimulator> containerList, ResourceManager resourceManager,
-      SLSRunner slsRunnner, long startTime, long finishTime, String simUser,
-      String simQueue, boolean tracked, String oldApp, long baseTimeMS,
-      Resource amResource, String nodeLabelExpr, Map<String, String> params,
-      Map<ApplicationId, AMSimulator> appIdAMSim) {
-    super.init(startTime, startTime + 1000000L * heartbeatInterval,
-        heartbeatInterval);
-    this.user = simUser;
-    this.rm = resourceManager;
-    this.se = slsRunnner;
-    this.queue = simQueue;
-    this.oldAppId = oldApp;
+  public void init(AMDefinition amDef, ResourceManager rm, SLSRunner slsRunner,
+      boolean tracked, long baselineTimeMS, long heartbeatInterval,
+      Map<ApplicationId, AMSimulator> appIdToAMSim) {
+    long startTime = amDef.getJobStartTime();
+    long endTime = startTime + 1000000L * heartbeatInterval;
+    super.init(startTime, endTime, heartbeatInterval);
+
+    this.user = amDef.getUser();
+    this.queue = amDef.getQueue();
+    this.oldAppId = amDef.getOldAppId();
+    this.amContainerResource = amDef.getAmResource();
+    this.nodeLabelExpression = amDef.getLabelExpression();
+    this.traceStartTimeMS = amDef.getJobStartTime();
+    this.traceFinishTimeMS = amDef.getJobFinishTime();
+    this.rm = rm;
+    this.se = slsRunner;
     this.isTracked = tracked;
-    this.baselineTimeMS = baseTimeMS;
-    this.traceStartTimeMS = startTime;
-    this.traceFinishTimeMS = finishTime;
-    this.amContainerResource = amResource;
-    this.nodeLabelExpression = nodeLabelExpr;
-    this.appIdToAMSim = appIdAMSim;
+    this.baselineTimeMS = baselineTimeMS;
+    this.appIdToAMSim = appIdToAMSim;
   }
 
   /**
@@ -221,6 +226,16 @@ public abstract class AMSimulator extends TaskRunner.Task {
 
   @Override
   public void lastStep() throws Exception {
+    if (simulateFinishTimeMS != FINISH_TIME_NOT_INITIALIZED) {
+      // The finish time is already recorded.
+      // Different value from zero means lastStep was called before.
+      // We want to prevent lastStep to be called more than once.
+      // See YARN-10427 for more details.
+      LOG.warn("Method AMSimulator#lastStep was already called. " +
+          "Skipping execution of method for application: {}", appId);
+      return;
+    }
+
     LOG.info("Application {} is shutting down.", appId);
     // unregister tracking
     if (isTracked) {
@@ -236,6 +251,11 @@ public abstract class AMSimulator extends TaskRunner.Task {
       LOG.info("AM container is null");
     }
 
+    // Clear runningApps for ranNodes of this app
+    for (NodeId nodeId : ranNodes) {
+      se.getNmMap().get(nodeId).finishApplication(getApplicationId());
+    }
+
     if (null == appAttemptId) {
       // If appAttemptId == null, AM is not launched from RM's perspective, so
       // it's unnecessary to finish am as well
@@ -272,8 +292,8 @@ public abstract class AMSimulator extends TaskRunner.Task {
   }
 
   protected ResourceRequest createResourceRequest(Resource resource,
-      ExecutionType executionType, String host, int priority, int
-      numContainers) {
+      ExecutionType executionType, String host, int priority, long
+      allocationId, int numContainers) {
     ResourceRequest request = recordFactory
         .newRecordInstance(ResourceRequest.class);
     request.setCapability(resource);
@@ -284,6 +304,7 @@ public abstract class AMSimulator extends TaskRunner.Task {
     Priority prio = recordFactory.newRecordInstance(Priority.class);
     prio.setPriority(priority);
     request.setPriority(prio);
+    request.setAllocationRequestId(allocationId);
     return request;
   }
 
@@ -406,11 +427,22 @@ public abstract class AMSimulator extends TaskRunner.Task {
   protected List<ResourceRequest> packageRequests(
           List<ContainerSimulator> csList, int priority) {
     // create requests
-    Map<String, ResourceRequest> rackLocalRequestMap = new HashMap<String, ResourceRequest>();
-    Map<String, ResourceRequest> nodeLocalRequestMap = new HashMap<String, ResourceRequest>();
-    ResourceRequest anyRequest = null;
+    Map<Long, Map<String, ResourceRequest>> rackLocalRequests =
+        new HashMap<>();
+    Map<Long, Map<String, ResourceRequest>> nodeLocalRequests =
+        new HashMap<>();
+    Map<Long, ResourceRequest> anyRequests = new HashMap<>();
     for (ContainerSimulator cs : csList) {
+      long allocationId = cs.getAllocationId();
+      ResourceRequest anyRequest = anyRequests.get(allocationId);
       if (cs.getHostname() != null) {
+        Map<String, ResourceRequest> rackLocalRequestMap;
+        if (rackLocalRequests.containsKey(allocationId)) {
+          rackLocalRequestMap = rackLocalRequests.get(allocationId);
+        } else {
+          rackLocalRequestMap = new HashMap<>();
+          rackLocalRequests.put(allocationId, rackLocalRequestMap);
+        }
         String[] rackHostNames = SLSUtils.getRackHostName(cs.getHostname());
         // check rack local
         String rackname = "/" + rackHostNames[0];
@@ -419,34 +451,49 @@ public abstract class AMSimulator extends TaskRunner.Task {
               rackLocalRequestMap.get(rackname).getNumContainers() + 1);
         } else {
           ResourceRequest request = createResourceRequest(cs.getResource(),
-              cs.getExecutionType(), rackname, priority, 1);
+              cs.getExecutionType(), rackname, priority,
+              cs.getAllocationId(), 1);
           rackLocalRequestMap.put(rackname, request);
         }
         // check node local
+        Map<String, ResourceRequest> nodeLocalRequestMap;
+        if (nodeLocalRequests.containsKey(allocationId)) {
+          nodeLocalRequestMap = nodeLocalRequests.get(allocationId);
+        } else {
+          nodeLocalRequestMap = new HashMap<>();
+          nodeLocalRequests.put(allocationId, nodeLocalRequestMap);
+        }
         String hostname = rackHostNames[1];
         if (nodeLocalRequestMap.containsKey(hostname)) {
           nodeLocalRequestMap.get(hostname).setNumContainers(
               nodeLocalRequestMap.get(hostname).getNumContainers() + 1);
         } else {
           ResourceRequest request = createResourceRequest(cs.getResource(),
-              cs.getExecutionType(), hostname, priority, 1);
+              cs.getExecutionType(), hostname, priority,
+              cs.getAllocationId(), 1);
           nodeLocalRequestMap.put(hostname, request);
         }
       }
       // any
       if (anyRequest == null) {
         anyRequest = createResourceRequest(cs.getResource(),
-            cs.getExecutionType(), ResourceRequest.ANY, priority, 1);
+            cs.getExecutionType(), ResourceRequest.ANY, priority,
+            cs.getAllocationId(), 1);
+        anyRequests.put(allocationId, anyRequest);
       } else {
         anyRequest.setNumContainers(anyRequest.getNumContainers() + 1);
       }
     }
     List<ResourceRequest> ask = new ArrayList<ResourceRequest>();
-    ask.addAll(nodeLocalRequestMap.values());
-    ask.addAll(rackLocalRequestMap.values());
-    if (anyRequest != null) {
-      ask.add(anyRequest);
+    for (Map<String, ResourceRequest> nodeLocalRequestMap :
+        nodeLocalRequests.values()) {
+      ask.addAll(nodeLocalRequestMap.values());
     }
+    for (Map<String, ResourceRequest> rackLocalRequestMap :
+        rackLocalRequests.values()) {
+      ask.addAll(rackLocalRequestMap.values());
+    }
+    ask.addAll(anyRequests.values());
     return ask;
   }
 
@@ -470,4 +517,8 @@ public abstract class AMSimulator extends TaskRunner.Task {
   public ApplicationAttemptId getApplicationAttemptId() {
     return appAttemptId;
   }
+
+  public Set<NodeId> getRanNodes() {
+    return this.ranNodes;
+  }
 }
