@@ -22,13 +22,11 @@ import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.net.URI;
-import java.net.URISyntaxException;
 import java.net.URLEncoder;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.Map;
-import java.util.regex.Pattern;
 import java.util.TreeMap;
 import java.util.TreeSet;
 
@@ -36,17 +34,17 @@ import org.apache.commons.cli.BasicParser;
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.CommandLineParser;
 import org.apache.commons.cli.Option;
-import org.apache.commons.cli.OptionBuilder;
 import org.apache.commons.cli.Options;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.fs.permission.FsAction;
+import org.apache.hadoop.mapreduce.Job;
 import org.apache.hadoop.mapreduce.MRConfig;
 import org.apache.hadoop.mapreduce.MRJobConfig;
 import org.apache.hadoop.mapreduce.filecache.DistributedCache;
 import org.apache.hadoop.mapreduce.server.jobtracker.JTConfig;
 import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.FileUtil;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.fs.FileAlreadyExistsException;
 import org.apache.hadoop.mapred.FileInputFormat;
@@ -56,7 +54,6 @@ import org.apache.hadoop.mapred.JobClient;
 import org.apache.hadoop.mapred.JobConf;
 import org.apache.hadoop.mapred.JobID;
 import org.apache.hadoop.mapred.KeyValueTextInputFormat;
-import org.apache.hadoop.mapred.OutputFormat;
 import org.apache.hadoop.mapred.RunningJob;
 import org.apache.hadoop.mapred.SequenceFileAsTextInputFormat;
 import org.apache.hadoop.mapred.SequenceFileInputFormat;
@@ -65,6 +62,7 @@ import org.apache.hadoop.mapred.TextOutputFormat;
 import org.apache.hadoop.mapred.lib.LazyOutputFormat;
 import org.apache.hadoop.mapred.lib.aggregate.ValueAggregatorCombiner;
 import org.apache.hadoop.mapred.lib.aggregate.ValueAggregatorReducer;
+import org.apache.hadoop.security.AccessControlException;
 import org.apache.hadoop.streaming.io.IdentifierResolver;
 import org.apache.hadoop.streaming.io.InputWriter;
 import org.apache.hadoop.streaming.io.OutputReader;
@@ -74,12 +72,14 @@ import org.apache.hadoop.util.RunJar;
 import org.apache.hadoop.util.StringUtils;
 import org.apache.hadoop.util.Tool;
 
+import static org.apache.hadoop.util.RunJar.MATCH_ANY;
+
 /** All the client-side work happens here.
  * (Jar packaging, MapRed job submission and monitoring)
  */
 public class StreamJob implements Tool {
 
-  protected static final Log LOG = LogFactory.getLog(StreamJob.class.getName());
+  protected static final Logger LOG = LoggerFactory.getLogger(StreamJob.class.getName());
   final static String REDUCE_NONE = "NONE";
 
   /** -----------Streaming CLI Implementation  **/
@@ -156,7 +156,7 @@ public class StreamJob implements Tool {
 
   /**
    * This is the method that actually
-   * intializes the job conf and submits the job
+   * initializes the job conf and submits the job
    * to the jobtracker
    * @throws IOException
    * @deprecated use {@link #run(String[])} instead.
@@ -297,7 +297,10 @@ public class StreamJob implements Tool {
           try {
             Path path = new Path(file);
             FileSystem localFs = FileSystem.getLocal(config_);
-            String finalPath = path.makeQualified(localFs).toString();
+            Path qualifiedPath = path.makeQualified(
+                localFs.getUri(), localFs.getWorkingDirectory());
+            validate(qualifiedPath);
+            String finalPath = qualifiedPath.toString();
             if(fileList.length() > 0) {
               fileList.append(',');
             }
@@ -313,7 +316,6 @@ public class StreamJob implements Tool {
           tmpFiles = tmpFiles + "," + fileList;
         }
         config_.set("tmpfiles", tmpFiles);
-        validate(packageFiles_);
       }
 
       String fsName = cmdLine.getOptionValue("dfs");
@@ -379,26 +381,26 @@ public class StreamJob implements Tool {
 
   private Option createOption(String name, String desc,
                               String argName, int max, boolean required){
-    return OptionBuilder
-           .withArgName(argName)
-           .hasArgs(max)
-           .withDescription(desc)
-           .isRequired(required)
-           .create(name);
+    return Option.builder(name)
+           .argName(argName)
+           .hasArgs()
+           .numberOfArgs(max)
+           .desc(desc)
+           .required(required)
+           .build();
   }
 
   private Option createBoolOption(String name, String desc){
-    return OptionBuilder.withDescription(desc).create(name);
+    return Option.builder(name).desc(desc).build();
   }
 
-  private void validate(final List<String> values)
-  throws IllegalArgumentException {
-    for (String file : values) {
-      File f = new File(file);
-      if (!FileUtil.canRead(f)) {
-        fail("File: " + f.getAbsolutePath()
-          + " does not exist, or is not readable.");
-      }
+  private void validate(final Path path) throws IOException {
+    try {
+      path.getFileSystem(config_).access(path, FsAction.READ);
+    } catch (FileNotFoundException e) {
+      fail("File: " + path + " does not exist.");
+    } catch (AccessControlException e) {
+      fail("File: " + path + " is not readable.");
     }
   }
 
@@ -502,7 +504,7 @@ public class StreamJob implements Tool {
   }
 
   private void printUsage(boolean detailed) {
-    System.out.println("Usage: $HADOOP_PREFIX/bin/hadoop jar hadoop-streaming.jar"
+    System.out.println("Usage: $HADOOP_HOME/bin/hadoop jar hadoop-streaming.jar"
         + " [options]");
     System.out.println("Options:");
     System.out.println("  -input          <path> DFS input file(s) for the Map"
@@ -551,7 +553,7 @@ public class StreamJob implements Tool {
       System.out.println();
       System.out.println("For more details about these options:");
       System.out.println("Use " +
-          "$HADOOP_PREFIX/bin/hadoop jar hadoop-streaming.jar -info");
+          "$HADOOP_HOME/bin/hadoop jar hadoop-streaming.jar -info");
       return;
     }
     System.out.println();
@@ -611,7 +613,7 @@ public class StreamJob implements Tool {
     System.out.println("  -D stream.non.zero.exit.is.failure=false");
     System.out.println("Use a custom hadoop streaming build along with standard"
         + " hadoop install:");
-    System.out.println("  $HADOOP_PREFIX/bin/hadoop jar " +
+    System.out.println("  $HADOOP_HOME/bin/hadoop jar " +
         "/path/my-hadoop-streaming.jar [...]\\");
     System.out.println("    [...] -D stream.shipped.hadoopstreaming=" +
         "/path/my-hadoop-streaming.jar");
@@ -625,7 +627,7 @@ public class StreamJob implements Tool {
     System.out.println("   -cmdenv EXAMPLE_DIR=/home/example/dictionaries/");
     System.out.println();
     System.out.println("Shortcut:");
-    System.out.println("   setenv HSTREAMING \"$HADOOP_PREFIX/bin/hadoop jar " +
+    System.out.println("   setenv HSTREAMING \"$HADOOP_HOME/bin/hadoop jar " +
         "hadoop-streaming.jar\"");
     System.out.println();
     System.out.println("Example: $HSTREAMING -mapper " +
@@ -648,9 +650,9 @@ public class StreamJob implements Tool {
   // --------------------------------------------
 
   protected String getHadoopClientHome() {
-    String h = env_.getProperty("HADOOP_PREFIX"); // standard Hadoop
+    String h = env_.getProperty("HADOOP_HOME"); // standard Hadoop
     if (h == null) {
-      //fail("Missing required environment variable: HADOOP_PREFIX");
+      //fail("Missing required environment variable: HADOOP_HOME");
       h = "UNDEF";
     }
     return h;
@@ -674,8 +676,8 @@ public class StreamJob implements Tool {
     // usually found in: build/contrib or build/hadoop-<version>-dev-streaming.jar
 
     // First try an explicit spec: it's too hard to find our own location in this case:
-    // $HADOOP_PREFIX/bin/hadoop jar /not/first/on/classpath/custom-hadoop-streaming.jar
-    // where findInClasspath() would find the version of hadoop-streaming.jar in $HADOOP_PREFIX
+    // $HADOOP_HOME/bin/hadoop jar /not/first/on/classpath/custom-hadoop-streaming.jar
+    // where findInClasspath() would find the version of hadoop-streaming.jar in $HADOOP_HOME
     String runtimeClasses = config_.get("stream.shipped.hadoopstreaming"); // jar or class dir
 
     if (runtimeClasses == null) {
@@ -968,10 +970,12 @@ public class StreamJob implements Tool {
         fail(LINK_URI);
     }
     // set the jobconf for the caching parameters
-    if (cacheArchives != null)
-      DistributedCache.setCacheArchives(archiveURIs, jobConf_);
-    if (cacheFiles != null)
-      DistributedCache.setCacheFiles(fileURIs, jobConf_);
+    if (cacheArchives != null) {
+      Job.setCacheArchives(archiveURIs, jobConf_);
+    }
+    if (cacheFiles != null) {
+      Job.setCacheFiles(fileURIs, jobConf_);
+    }
 
     if (verbose_) {
       listJobConfProperties();
@@ -1007,7 +1011,7 @@ public class StreamJob implements Tool {
     if (jar_ != null && isLocalHadoop()) {
       // getAbs became required when shell and subvm have different working dirs...
       File wd = new File(".").getAbsoluteFile();
-      RunJar.unJar(new File(jar_), wd);
+      RunJar.unJar(new File(jar_), wd, MATCH_ANY);
     }
 
     // if jobConf_ changes must recreate a JobClient
