@@ -47,6 +47,28 @@ public interface ApplicationConstants {
   String CONTAINER_TOKEN_FILE_ENV_NAME =
       UserGroupInformation.HADOOP_TOKEN_FILE_LOCATION;
 
+  /**
+   * The file into which the keystore containing the AM's certificate is
+   * written.
+   */
+  String KEYSTORE_FILE_LOCATION_ENV_NAME = "KEYSTORE_FILE_LOCATION";
+
+  /**
+   * The password for the AM's keystore.
+   */
+  String KEYSTORE_PASSWORD_ENV_NAME = "KEYSTORE_PASSWORD";
+
+  /**
+   * The file into which the truststore containing the AM's certificate is
+   * written.
+   */
+  String TRUSTSTORE_FILE_LOCATION_ENV_NAME = "TRUSTSTORE_FILE_LOCATION";
+
+  /**
+   * The password for the AM's truststore.
+   */
+  String TRUSTSTORE_PASSWORD_ENV_NAME = "TRUSTSTORE_PASSWORD";
+
   /**
    * The environmental variable for APPLICATION_WEB_PROXY_BASE. Set in
    * ApplicationMaster's environment only. This states that for all non-relative
@@ -55,6 +77,13 @@ public interface ApplicationConstants {
   String APPLICATION_WEB_PROXY_BASE_ENV =
     "APPLICATION_WEB_PROXY_BASE";
 
+  /**
+   * The environmental variable for JDK17's add-opens workaround. This
+   * should be replaced either a correctly formatted add-opens option if JDK17 is used
+   * or an empty string if not on container launch.
+   */
+  String JVM_ADD_OPENS_VAR = "<ADD_OPENS>";
+
   /**
    * The temporary environmental variable for container log directory. This
    * should be replaced by real container log directory on container launch.
@@ -260,7 +289,14 @@ public interface ApplicationConstants {
      * Final, Docker run support ENTRY_POINT.
      */
     YARN_CONTAINER_RUNTIME_DOCKER_RUN_OVERRIDE_DISABLE(
-        "YARN_CONTAINER_RUNTIME_DOCKER_RUN_OVERRIDE_DISABLE");
+        "YARN_CONTAINER_RUNTIME_DOCKER_RUN_OVERRIDE_DISABLE"),
+
+    /**
+     * $YARN_CONTAINER_RUNTIME_YARN_SYSFS_ENABLE
+     * Final, expose cluster information to container.
+     */
+    YARN_CONTAINER_RUNTIME_YARN_SYSFS_ENABLE(
+        "YARN_CONTAINER_RUNTIME_YARN_SYSFS_ENABLE");
 
     private final String variable;
     Environment(String variable) {
