@@ -449,7 +449,7 @@ public abstract class ApplicationReport {
    * Get the AMRM token of the application.
    * <p>
    * The AMRM token is required for AM to RM scheduling operations. For 
-   * managed Application Masters Yarn takes care of injecting it. For unmanaged
+   * managed Application Masters YARN takes care of injecting it. For unmanaged
    * Applications Masters, the token must be obtained via this method and set
    * in the {@link org.apache.hadoop.security.UserGroupInformation} of the
    * current user.
@@ -539,4 +539,17 @@ public abstract class ApplicationReport {
   @Unstable
   public abstract void setApplicationTimeouts(
       Map<ApplicationTimeoutType, ApplicationTimeout> timeouts);
+
+  /**
+   * Get RM ClusterId.
+   *
+   * @return RM ClusterId
+   */
+  @Public
+  @Stable
+  public abstract String getRMClusterId();
+
+  @Public
+  @Stable
+  public abstract void setRMClusterId(String rmClusterId);
 }
