@@ -66,6 +66,7 @@ import org.apache.hadoop.yarn.api.records.ReservationDefinition;
 import org.apache.hadoop.yarn.api.records.ReservationId;
 import org.apache.hadoop.yarn.api.records.Resource;
 import org.apache.hadoop.yarn.api.records.ResourceTypeInfo;
+import org.apache.hadoop.yarn.api.records.ShellContainerCommand;
 import org.apache.hadoop.yarn.api.records.SignalContainerCommand;
 import org.apache.hadoop.yarn.api.records.Token;
 import org.apache.hadoop.yarn.api.records.YarnApplicationState;
@@ -463,11 +464,28 @@ public abstract class YarnClient extends AbstractService {
    * @throws YarnException
    *           in case of errors or if YARN rejects the request due to
    *           access-control restrictions.
-   * @throws IOException
+   * @throws IOException I/O exception has occurred.
    */
   public abstract QueueInfo getQueueInfo(String queueName) throws YarnException,
       IOException;
 
+  /**
+   * <p>
+   * Get information ({@link QueueInfo}) about a given <em>queue</em>.
+   * </p>
+   *
+   * @param queueName
+   *          Name of the queue whose information is needed.
+   * @param subClusterId sub-cluster Id.
+   * @return queue information.
+   * @throws YarnException
+   *           in case of errors or if YARN rejects the request due to
+   *           access-control restrictions.
+   * @throws IOException I/O exception has occurred.
+   */
+  public abstract QueueInfo getQueueInfo(String queueName, String subClusterId)
+      throws YarnException, IOException;
+
   /**
    * <p>
    * Get information ({@link QueueInfo}) about all queues, recursively if there
@@ -958,4 +976,18 @@ public abstract class YarnClient extends AbstractService {
   public abstract Map<String, Set<NodeAttribute>> getNodeToAttributes(
       Set<String> hostNames) throws YarnException, IOException;
 
+  /**
+   * <p>
+   * The interface used by client to get a shell to a container.
+   * </p>
+   *
+   * @param containerId Container ID
+   * @param command Shell type
+   * @throws IOException if connection fails.
+   */
+  @Public
+  @Unstable
+  public abstract void shellToContainer(ContainerId containerId,
+      ShellContainerCommand command) throws IOException;
+
 }
