@@ -81,6 +81,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.TreeSet;
 
+import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 
@@ -512,6 +513,7 @@ public class TestOpportunisticContainerAllocationE2E {
     assertEquals(0, amClient.ask.size());
     assertEquals(0, amClient.release.size());
 
+
     amClient.addContainerRequest(
         new AMRMClient.ContainerRequest(capability, nodes, racks, priority));
     amClient.addContainerRequest(
@@ -656,7 +658,7 @@ public class TestOpportunisticContainerAllocationE2E {
         for (ContainerStatus cStatus : allocResponse
             .getCompletedContainersStatuses()) {
           if (releases.contains(cStatus.getContainerId())) {
-            assertEquals(cStatus.getState(), ContainerState.COMPLETE);
+            assertThat(cStatus.getState()).isEqualTo(ContainerState.COMPLETE);
             assertEquals(-100, cStatus.getExitStatus());
             releases.remove(cStatus.getContainerId());
           }
@@ -691,10 +693,9 @@ public class TestOpportunisticContainerAllocationE2E {
             ExecutionTypeRequest.newInstance(
                 ExecutionType.OPPORTUNISTIC, true)));
 
-    int oppContainersRequestedAny =
-        amClient.getTable(0).get(priority3, ResourceRequest.ANY,
-            ExecutionType.OPPORTUNISTIC, capability).remoteRequest
-            .getNumContainers();
+    int oppContainersRequestedAny = amClient.getTable(0)
+        .get(priority3, ResourceRequest.ANY, ExecutionType.OPPORTUNISTIC,
+            capability).remoteRequest.getNumContainers();
 
     assertEquals(2, oppContainersRequestedAny);
 
