@@ -44,7 +44,7 @@ import org.apache.hadoop.yarn.proto.YarnProtos.LogAggregationStatusProto;
 import org.apache.hadoop.yarn.proto.YarnProtos.PriorityProto;
 import org.apache.hadoop.yarn.proto.YarnProtos.YarnApplicationStateProto;
 
-import com.google.protobuf.TextFormat;
+import org.apache.hadoop.thirdparty.protobuf.TextFormat;
 
 import java.util.HashMap;
 import java.util.HashSet;
@@ -787,4 +787,23 @@ public class ApplicationReportPBImpl extends ApplicationReport {
     this.builder.addAllAppTimeouts(values);
   }
 
+  @Override
+  public String getRMClusterId() {
+    ApplicationReportProtoOrBuilder p = viaProto ? proto : builder;
+    if (!p.hasRmClusterId()) {
+      return null;
+    }
+    return p.getRmClusterId();
+  }
+
+  @Override
+  public void setRMClusterId(String rmClusterId) {
+    maybeInitBuilder();
+    if (rmClusterId == null) {
+      builder.clearRmClusterId();
+      return;
+    }
+    builder.setRmClusterId((rmClusterId));
+  }
+
 }
