@@ -31,9 +31,9 @@ public class HeaderBlock extends HtmlBlock {
     html.
       div("#header.ui-widget").
         div("#user").
-          _(loggedIn)._().
+        __(loggedIn).__().
         div("#logo").
-          img("/static/hadoop-st.png")._().
-        h1($(TITLE))._();
+          img("/static/hadoop-st.png").__().
+        h1($(TITLE)).__();
   }
 }
