@@ -21,8 +21,8 @@ package org.apache.hadoop.yarn.webapp.view;
 import java.util.List;
 
 import org.apache.hadoop.classification.InterfaceAudience;
+import org.apache.hadoop.util.Lists;
 import org.apache.hadoop.yarn.webapp.hamlet2.HamletSpec.HTML;
-import org.apache.hadoop.thirdparty.com.google.common.collect.Lists;
 
 import static org.apache.commons.text.StringEscapeUtils.escapeEcmaScript;
 import static org.apache.hadoop.yarn.util.StringHelper.djoin;
@@ -65,12 +65,12 @@ public class JQueryUI extends HtmlBlock {
   @Override
   protected void render(Block html) {
     html.link(root_url("static/jquery/themes-1.9.1/base/jquery-ui.css"))
-        .link(root_url("static/dt-1.10.18/css/jquery.dataTables.css"))
-        .link(root_url("static/dt-1.10.18/css/jui-dt.css"))
-        .link(root_url("static/dt-1.10.18/css/custom_datatable.css"))
+        .link(root_url("static/dt-1.11.5/css/jquery.dataTables.css"))
+        .link(root_url("static/dt-1.11.5/css/jui-dt.css"))
+        .link(root_url("static/dt-1.11.5/css/custom_datatable.css"))
         .script(root_url("static/jquery/jquery-3.6.0.min.js"))
-        .script(root_url("static/jquery/jquery-ui-1.13.1.custom.min.js"))
-        .script(root_url("static/dt-1.10.18/js/jquery.dataTables.min.js"))
+        .script(root_url("static/jquery/jquery-ui-1.13.2.custom.min.js"))
+        .script(root_url("static/dt-1.11.5/js/jquery.dataTables.min.js"))
         .script(root_url("static/yarn.dt.plugins.js"))
         .script(root_url("static/dt-sorting/natural.js"))
         .style("#jsnotice { padding: 0.2em; text-align: center; }",
