@@ -18,11 +18,12 @@
 
 package org.apache.hadoop.yarn.factories.impl.pb;
 
-import org.apache.hadoop.conf.Configuration;
-import org.junit.Test;
-
 import java.net.InetSocketAddress;
 
+import org.junit.jupiter.api.Test;
+
+import org.apache.hadoop.conf.Configuration;
+
 import static org.mockito.ArgumentMatchers.anyString;
 import static org.mockito.Mockito.atLeastOnce;
 import static org.mockito.Mockito.mock;
@@ -33,7 +34,7 @@ import static org.mockito.Mockito.verify;
  */
 public class TestRpcClientFactoryPBImpl {
   @Test
-  public void testToUseCustomClassloader() throws Exception {
+  void testToUseCustomClassloader() throws Exception {
     Configuration configuration = mock(Configuration.class);
     RpcClientFactoryPBImpl rpcClientFactoryPB = RpcClientFactoryPBImpl.get();
     try {
