@@ -21,33 +21,33 @@ package org.apache.hadoop.yarn.util.resource;
 import java.util.Arrays;
 import java.util.Collection;
 
-import com.google.common.collect.ImmutableSet;
+import org.apache.hadoop.thirdparty.com.google.common.collect.ImmutableSet;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Timeout;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.MethodSource;
+
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.yarn.api.records.Resource;
 import org.apache.hadoop.yarn.api.records.ResourceInformation;
 import org.apache.hadoop.yarn.conf.YarnConfiguration;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.junit.runners.Parameterized;
 
-import static org.junit.Assert.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
-@RunWith(Parameterized.class)
 public class TestResourceCalculator {
   private static final String EXTRA_RESOURCE_NAME = "test";
 
-  private final ResourceCalculator resourceCalculator;
+  private ResourceCalculator resourceCalculator;
 
-  @Parameterized.Parameters(name = "{0}")
   public static Collection<Object[]> getParameters() {
-    return Arrays.asList(new Object[][] {
-        { "DefaultResourceCalculator", new DefaultResourceCalculator() },
-        { "DominantResourceCalculator", new DominantResourceCalculator() } });
+    return Arrays.asList(new Object[][]{
+        {"DefaultResourceCalculator", new DefaultResourceCalculator()},
+        {"DominantResourceCalculator", new DominantResourceCalculator()}});
   }
 
-  @Before
+  @BeforeEach
   public void setupNoExtraResource() {
     // This has to run before each test because we don't know when
     // setupExtraResource() might be called
@@ -61,34 +61,38 @@ public class TestResourceCalculator {
     ResourceUtils.resetResourceTypes(conf);
   }
 
-  public TestResourceCalculator(String name, ResourceCalculator rs) {
+  public void initTestResourceCalculator(String name, ResourceCalculator rs) {
     this.resourceCalculator = rs;
   }
-  
-  @Test(timeout = 10000)
-  public void testFitsIn() {
+
+  @MethodSource("getParameters")
+  @ParameterizedTest(name = "{0}")
+  @Timeout(10000)
+  void testFitsIn(String name, ResourceCalculator rs) {
+
+    initTestResourceCalculator(name, rs);
 
     if (resourceCalculator instanceof DefaultResourceCalculator) {
-      Assert.assertTrue(resourceCalculator.fitsIn(
+      assertTrue(resourceCalculator.fitsIn(
           Resource.newInstance(1, 2), Resource.newInstance(2, 1)));
-      Assert.assertTrue(resourceCalculator.fitsIn(
+      assertTrue(resourceCalculator.fitsIn(
           Resource.newInstance(1, 2), Resource.newInstance(2, 2)));
-      Assert.assertTrue(resourceCalculator.fitsIn(
+      assertTrue(resourceCalculator.fitsIn(
           Resource.newInstance(1, 2), Resource.newInstance(1, 2)));
-      Assert.assertTrue(resourceCalculator.fitsIn(
+      assertTrue(resourceCalculator.fitsIn(
           Resource.newInstance(1, 2), Resource.newInstance(1, 1)));
-      Assert.assertFalse(resourceCalculator.fitsIn(
+      assertFalse(resourceCalculator.fitsIn(
           Resource.newInstance(2, 1), Resource.newInstance(1, 2)));
     } else if (resourceCalculator instanceof DominantResourceCalculator) {
-      Assert.assertFalse(resourceCalculator.fitsIn(
+      assertFalse(resourceCalculator.fitsIn(
           Resource.newInstance(1, 2), Resource.newInstance(2, 1)));
-      Assert.assertTrue(resourceCalculator.fitsIn(
+      assertTrue(resourceCalculator.fitsIn(
           Resource.newInstance(1, 2), Resource.newInstance(2, 2)));
-      Assert.assertTrue(resourceCalculator.fitsIn(
+      assertTrue(resourceCalculator.fitsIn(
           Resource.newInstance(1, 2), Resource.newInstance(1, 2)));
-      Assert.assertFalse(resourceCalculator.fitsIn(
+      assertFalse(resourceCalculator.fitsIn(
           Resource.newInstance(1, 2), Resource.newInstance(1, 1)));
-      Assert.assertFalse(resourceCalculator.fitsIn(
+      assertFalse(resourceCalculator.fitsIn(
           Resource.newInstance(2, 1), Resource.newInstance(1, 2)));
     }
   }
@@ -121,22 +125,22 @@ public class TestResourceCalculator {
       int expected) {
     int actual = resourceCalculator.compare(cluster, res1, res2);
 
-    assertEquals(String.format("Resource comparison did not give the expected "
-        + "result for %s v/s %s", res1.toString(), res2.toString()),
-        expected, actual);
+    assertEquals(expected, actual, String.format("Resource comparison did not give the expected "
+        + "result for %s v/s %s", res1.toString(), res2.toString()));
 
     if (expected != 0) {
       // Try again with args in the opposite order and the negative of the
       // expected result.
       actual = resourceCalculator.compare(cluster, res2, res1);
-      assertEquals(String.format("Resource comparison did not give the "
-          + "expected result for %s v/s %s", res2.toString(), res1.toString()),
-          expected * -1, actual);
+      assertEquals(expected * -1, actual, String.format("Resource comparison did not give the "
+          + "expected result for %s v/s %s", res2.toString(), res1.toString()));
     }
   }
 
-  @Test
-  public void testCompareWithOnlyMandatory() {
+  @MethodSource("getParameters")
+  @ParameterizedTest(name = "{0}")
+  void testCompareWithOnlyMandatory(String name, ResourceCalculator rs) {
+    initTestResourceCalculator(name, rs);
     // This test is necessary because there are optimizations that are only
     // triggered when only the mandatory resources are configured.
 
@@ -173,8 +177,10 @@ public class TestResourceCalculator {
     assertComparison(cluster, newResource(3, 1), newResource(3, 0), 1);
   }
 
-  @Test
-  public void testCompare() {
+  @MethodSource("getParameters")
+  @ParameterizedTest(name = "{0}")
+  void testCompare(String name, ResourceCalculator rs) {
+    initTestResourceCalculator(name, rs);
     // Test with 3 resources
     setupExtraResource();
 
@@ -209,7 +215,7 @@ public class TestResourceCalculator {
   /**
    * Verify compare when one or all the resource are zero.
    */
-  private void testCompareDominantZeroValueResource(){
+  private void testCompareDominantZeroValueResource() {
     Resource cluster = newResource(4L, 4, 0);
     assertComparison(cluster, newResource(2, 1, 1), newResource(1, 1, 2), 1);
     assertComparison(cluster, newResource(2, 2, 1), newResource(1, 2, 2), 1);
@@ -261,8 +267,11 @@ public class TestResourceCalculator {
     assertComparison(cluster, newResource(3, 1, 1), newResource(3, 0, 0), 1);
   }
 
-  @Test(timeout = 10000)
-  public void testCompareWithEmptyCluster() {
+  @MethodSource("getParameters")
+  @ParameterizedTest(name = "{0}")
+  @Timeout(10000)
+  void testCompareWithEmptyCluster(String name, ResourceCalculator rs) {
+    initTestResourceCalculator(name, rs);
     Resource clusterResource = Resource.newInstance(0, 0);
 
     // For lhs == rhs
@@ -316,35 +325,39 @@ public class TestResourceCalculator {
       boolean greaterThan, boolean greaterThanOrEqual, Resource max,
       Resource min) {
 
-    assertEquals("Less Than operation is wrongly calculated.", lessThan,
-        Resources.lessThan(resourceCalculator, clusterResource, lhs, rhs));
+    assertEquals(lessThan,
+        Resources.lessThan(resourceCalculator, clusterResource, lhs, rhs),
+        "Less Than operation is wrongly calculated.");
 
     assertEquals(
-        "Less Than Or Equal To operation is wrongly calculated.",
         lessThanOrEqual, Resources.lessThanOrEqual(resourceCalculator,
-            clusterResource, lhs, rhs));
+            clusterResource, lhs, rhs), "Less Than Or Equal To operation is wrongly calculated.");
 
-    assertEquals("Greater Than operation is wrongly calculated.",
-        greaterThan,
-        Resources.greaterThan(resourceCalculator, clusterResource, lhs, rhs));
+    assertEquals(greaterThan,
+        Resources.greaterThan(resourceCalculator, clusterResource, lhs, rhs),
+        "Greater Than operation is wrongly calculated.");
 
-    assertEquals(
-        "Greater Than Or Equal To operation is wrongly calculated.",
-        greaterThanOrEqual, Resources.greaterThanOrEqual(resourceCalculator,
-            clusterResource, lhs, rhs));
+    assertEquals(greaterThanOrEqual,
+        Resources.greaterThanOrEqual(resourceCalculator, clusterResource, lhs, rhs),
+        "Greater Than Or Equal To operation is wrongly calculated.");
 
-    assertEquals("Max(value) Operation wrongly calculated.", max,
-        Resources.max(resourceCalculator, clusterResource, lhs, rhs));
+    assertEquals(max,
+        Resources.max(resourceCalculator, clusterResource, lhs, rhs),
+        "Max(value) Operation wrongly calculated.");
 
-    assertEquals("Min(value) operation is wrongly calculated.", min,
-        Resources.min(resourceCalculator, clusterResource, lhs, rhs));
+    assertEquals(min,
+        Resources.min(resourceCalculator, clusterResource, lhs, rhs),
+        "Min(value) operation is wrongly calculated.");
   }
 
   /**
    * Test resource normalization.
    */
-  @Test(timeout = 10000)
-  public void testNormalize() {
+  @MethodSource("getParameters")
+  @ParameterizedTest(name = "{0}")
+  @Timeout(10000)
+  void testNormalize(String name, ResourceCalculator rs) {
+    initTestResourceCalculator(name, rs);
     // requested resources value cannot be an arbitrary number.
     Resource ask = Resource.newInstance(1111, 2);
     Resource min = Resource.newInstance(1024, 1);
@@ -420,22 +433,28 @@ public class TestResourceCalculator {
     }
   }
 
-  @Test
-  public void testDivisionByZeroRatioDenominatorIsZero() {
+  @MethodSource("getParameters")
+  @ParameterizedTest(name = "{0}")
+  void testDivisionByZeroRatioDenominatorIsZero(String name, ResourceCalculator rs) {
+    initTestResourceCalculator(name, rs);
     float ratio = resourceCalculator.ratio(newResource(1, 1), newResource(0,
         0));
     assertEquals(Float.POSITIVE_INFINITY, ratio, 0.00001);
   }
 
-  @Test
-  public void testDivisionByZeroRatioNumeratorAndDenominatorIsZero() {
+  @MethodSource("getParameters")
+  @ParameterizedTest(name = "{0}")
+  void testDivisionByZeroRatioNumeratorAndDenominatorIsZero(String name, ResourceCalculator rs) {
+    initTestResourceCalculator(name, rs);
     float ratio = resourceCalculator.ratio(newResource(0, 0), newResource(0,
         0));
     assertEquals(0.0, ratio, 0.00001);
   }
 
-  @Test
-  public void testFitsInDiagnosticsCollector() {
+  @MethodSource("getParameters")
+  @ParameterizedTest(name = "{0}")
+  void testFitsInDiagnosticsCollector(String name, ResourceCalculator rs) {
+    initTestResourceCalculator(name, rs);
     if (resourceCalculator instanceof DefaultResourceCalculator) {
       // required-resource = (0, 0)
       assertEquals(ImmutableSet.of(),
@@ -551,8 +570,10 @@ public class TestResourceCalculator {
     }
   }
 
-  @Test
-  public void testRatioWithNoExtraResource() {
+  @MethodSource("getParameters")
+  @ParameterizedTest(name = "{0}")
+  void testRatioWithNoExtraResource(String name, ResourceCalculator rs) {
+    initTestResourceCalculator(name, rs);
     //setup
     Resource resource1 = newResource(1, 1);
     Resource resource2 = newResource(2, 1);
@@ -570,8 +591,10 @@ public class TestResourceCalculator {
     }
   }
 
-  @Test
-  public void testRatioWithExtraResource() {
+  @MethodSource("getParameters")
+  @ParameterizedTest(name = "{0}")
+  void testRatioWithExtraResource(String name, ResourceCalculator rs) {
+    initTestResourceCalculator(name, rs);
     //setup
     setupExtraResource();
     Resource resource1 = newResource(1, 1, 2);
