@@ -18,25 +18,10 @@
 
 package org.apache.hadoop.yarn.util.resource;
 
-import org.apache.commons.io.FileUtils;
-import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.yarn.api.protocolrecords.ResourceTypes;
-import org.apache.hadoop.yarn.api.records.Resource;
-import org.apache.hadoop.yarn.api.records.ResourceInformation;
-import org.apache.hadoop.yarn.conf.YarnConfiguration;
-import org.apache.hadoop.yarn.exceptions.YarnRuntimeException;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.ExpectedException;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
-
 import java.io.File;
 import java.io.IOException;
 import java.net.URL;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -44,6 +29,27 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import org.apache.commons.io.FileUtils;
+import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.yarn.api.protocolrecords.ResourceTypes;
+import org.apache.hadoop.yarn.api.records.Resource;
+import org.apache.hadoop.yarn.api.records.ResourceInformation;
+import org.apache.hadoop.yarn.api.records.ResourceTypeInfo;
+import org.apache.hadoop.yarn.conf.YarnConfiguration;
+import org.apache.hadoop.yarn.exceptions.YarnRuntimeException;
+
+import static org.assertj.core.api.Assertions.assertThat;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
+
 /**
  * Test class to verify all resource utility methods.
  */
@@ -59,39 +65,19 @@ public class TestResourceUtils {
     int resourceCount;
     Map<String, String> resourceNameUnitsMap;
 
-    public ResourceFileInformation(String name, int count) {
+    ResourceFileInformation(String name, int count) {
       filename = name;
       resourceCount = count;
       resourceNameUnitsMap = new HashMap<>();
     }
   }
 
-  @Rule
-  public ExpectedException expexted = ExpectedException.none();
-
-  public static void addNewTypesToResources(String... resourceTypes) {
-    // Initialize resource map
-    Map<String, ResourceInformation> riMap = new HashMap<>();
-
-    // Initialize mandatory resources
-    riMap.put(ResourceInformation.MEMORY_URI, ResourceInformation.MEMORY_MB);
-    riMap.put(ResourceInformation.VCORES_URI, ResourceInformation.VCORES);
-
-    for (String newResource : resourceTypes) {
-      riMap.put(newResource, ResourceInformation
-          .newInstance(newResource, "", 0, ResourceTypes.COUNTABLE, 0,
-              Integer.MAX_VALUE));
-    }
-
-    ResourceUtils.initializeResourcesFromResourceInformationMap(riMap);
-  }
-
-  @Before
+  @BeforeEach
   public void setup() {
     ResourceUtils.resetResourceTypes();
   }
 
-  @After
+  @AfterEach
   public void teardown() {
     if (nodeResourcesFile != null && nodeResourcesFile.exists()) {
       nodeResourcesFile.delete();
@@ -150,29 +136,28 @@ public class TestResourceUtils {
   private void testMemoryAndVcores(Map<String, ResourceInformation> res) {
     String memory = ResourceInformation.MEMORY_MB.getName();
     String vcores = ResourceInformation.VCORES.getName();
-    Assert.assertTrue("Resource 'memory' missing", res.containsKey(memory));
-    Assert.assertEquals("'memory' units incorrect",
-        ResourceInformation.MEMORY_MB.getUnits(), res.get(memory).getUnits());
-    Assert.assertEquals("'memory' types incorrect",
-        ResourceInformation.MEMORY_MB.getResourceType(),
-        res.get(memory).getResourceType());
-    Assert.assertTrue("Resource 'vcores' missing", res.containsKey(vcores));
-    Assert.assertEquals("'vcores' units incorrect",
-        ResourceInformation.VCORES.getUnits(), res.get(vcores).getUnits());
-    Assert.assertEquals("'vcores' type incorrect",
-        ResourceInformation.VCORES.getResourceType(),
-        res.get(vcores).getResourceType());
+    assertTrue(res.containsKey(memory), "Resource 'memory' missing");
+    assertEquals(ResourceInformation.MEMORY_MB.getUnits(), res.get(memory).getUnits(),
+        "'memory' units incorrect");
+    assertEquals(ResourceInformation.MEMORY_MB.getResourceType(), res.get(memory).getResourceType(),
+        "'memory' types incorrect");
+    assertTrue(res.containsKey(vcores), "Resource 'vcores' missing");
+    assertEquals(ResourceInformation.VCORES.getUnits(), res.get(vcores).getUnits(),
+        "'vcores' units incorrect");
+    assertEquals(ResourceInformation.VCORES.getResourceType(),
+        res.get(vcores).getResourceType(),
+        "'vcores' type incorrect");
   }
 
   @Test
-  public void testGetResourceTypes() {
+  void testGetResourceTypes() {
     Map<String, ResourceInformation> res = ResourceUtils.getResourceTypes();
-    Assert.assertEquals(2, res.size());
+    assertEquals(2, res.size());
     testMemoryAndVcores(res);
   }
 
   @Test
-  public void testGetResourceTypesConfigs() throws Exception {
+  void testGetResourceTypesConfigs() throws Exception {
     Configuration conf = new YarnConfiguration();
 
     ResourceFileInformation testFile1 =
@@ -196,19 +181,19 @@ public class TestResourceUtils {
       ResourceUtils.resetResourceTypes();
       res = setupResourceTypesInternal(conf, testInformation.filename);
       testMemoryAndVcores(res);
-      Assert.assertEquals(testInformation.resourceCount, res.size());
+      assertEquals(testInformation.resourceCount, res.size());
       for (Map.Entry<String, String> entry :
           testInformation.resourceNameUnitsMap.entrySet()) {
         String resourceName = entry.getKey();
-        Assert.assertTrue("Missing key " + resourceName,
-            res.containsKey(resourceName));
-        Assert.assertEquals(entry.getValue(), res.get(resourceName).getUnits());
+        assertTrue(res.containsKey(resourceName),
+            "Missing key " + resourceName);
+        assertEquals(entry.getValue(), res.get(resourceName).getUnits());
       }
     }
   }
 
   @Test
-  public void testGetRequestedResourcesFromConfig() {
+  void testGetRequestedResourcesFromConfig() {
     Configuration conf = new Configuration();
 
     //these resource type configurations should be recognised
@@ -242,14 +227,14 @@ public class TestResourceUtils {
     Set<String> expectedSet =
         new HashSet<>(Arrays.asList(expectedKeys));
 
-    Assert.assertEquals(properList.size(), expectedKeys.length);
+    assertEquals(properList.size(), expectedKeys.length);
     properList.forEach(
-        item -> Assert.assertTrue(expectedSet.contains(item.getName())));
+        item -> assertTrue(expectedSet.contains(item.getName())));
 
   }
 
   @Test
-  public void testGetResourceTypesConfigErrors() throws IOException {
+  void testGetResourceTypesConfigErrors() throws IOException {
     Configuration conf = new YarnConfiguration();
 
     String[] resourceFiles = {"resource-types-error-1.xml",
@@ -259,7 +244,7 @@ public class TestResourceUtils {
       ResourceUtils.resetResourceTypes();
       try {
         setupResourceTypesInternal(conf, resourceFile);
-        Assert.fail("Expected error with file " + resourceFile);
+        fail("Expected error with file " + resourceFile);
       } catch (YarnRuntimeException | IllegalArgumentException e) {
         //Test passed
       }
@@ -267,7 +252,7 @@ public class TestResourceUtils {
   }
 
   @Test
-  public void testInitializeResourcesMap() {
+  void testInitializeResourcesMap() {
     String[] empty = {"", ""};
     String[] res1 = {"resource1", "m"};
     String[] res2 = {"resource2", "G"};
@@ -304,31 +289,30 @@ public class TestResourceUtils {
         len = 4;
       }
 
-      Assert.assertEquals(len, ret.size());
+      assertEquals(len, ret.size());
       for (String[] resources : test) {
         if (resources[0].length() == 0) {
           continue;
         }
-        Assert.assertTrue(ret.containsKey(resources[0]));
+        assertTrue(ret.containsKey(resources[0]));
         ResourceInformation resInfo = ret.get(resources[0]);
-        Assert.assertEquals(resources[1], resInfo.getUnits());
-        Assert.assertEquals(ResourceTypes.COUNTABLE, resInfo.getResourceType());
+        assertEquals(resources[1], resInfo.getUnits());
+        assertEquals(ResourceTypes.COUNTABLE, resInfo.getResourceType());
       }
       // we must always have memory and vcores with their fixed units
-      Assert.assertTrue(ret.containsKey("memory-mb"));
+      assertTrue(ret.containsKey("memory-mb"));
       ResourceInformation memInfo = ret.get("memory-mb");
-      Assert.assertEquals("Mi", memInfo.getUnits());
-      Assert.assertEquals(ResourceTypes.COUNTABLE, memInfo.getResourceType());
-      Assert.assertTrue(ret.containsKey("vcores"));
+      assertEquals("Mi", memInfo.getUnits());
+      assertEquals(ResourceTypes.COUNTABLE, memInfo.getResourceType());
+      assertTrue(ret.containsKey("vcores"));
       ResourceInformation vcoresInfo = ret.get("vcores");
-      Assert.assertEquals("", vcoresInfo.getUnits());
-      Assert
-          .assertEquals(ResourceTypes.COUNTABLE, vcoresInfo.getResourceType());
+      assertEquals("", vcoresInfo.getUnits());
+      assertEquals(ResourceTypes.COUNTABLE, vcoresInfo.getResourceType());
     }
   }
 
   @Test
-  public void testInitializeResourcesMapErrors() {
+  void testInitializeResourcesMapErrors() {
     String[] mem1 = {"memory-mb", ""};
     String[] vcores1 = {"vcores", "M"};
 
@@ -359,7 +343,7 @@ public class TestResourceUtils {
       }
       try {
         ResourceUtils.initializeResourcesMap(conf);
-        Assert.fail("resource map initialization should fail");
+        fail("resource map initialization should fail");
       } catch (Exception e) {
         //Test passed
       }
@@ -367,7 +351,7 @@ public class TestResourceUtils {
   }
 
   @Test
-  public void testGetResourceInformation() throws Exception {
+  void testGetResourceInformation() throws Exception {
     Configuration conf = new YarnConfiguration();
     Map<String, Resource> testRun = new HashMap<>();
     setupResourceTypesInternal(conf, "resource-types-4.xml");
@@ -385,16 +369,16 @@ public class TestResourceUtils {
       ResourceUtils.resetNodeResources();
       Map<String, ResourceInformation> actual = setupNodeResources(conf,
           resourceFile);
-      Assert.assertEquals(actual.size(),
+      assertEquals(actual.size(),
           entry.getValue().getResources().length);
       for (ResourceInformation resInfo : entry.getValue().getResources()) {
-        Assert.assertEquals(resInfo, actual.get(resInfo.getName()));
+        assertEquals(resInfo, actual.get(resInfo.getName()));
       }
     }
   }
 
   @Test
-  public void testGetNodeResourcesConfigErrors() throws Exception {
+  void testGetNodeResourcesConfigErrors() throws Exception {
     Configuration conf = new YarnConfiguration();
     setupResourceTypesInternal(conf, "resource-types-4.xml");
     String[] invalidNodeResFiles = {"node-resources-error-1.xml"};
@@ -403,7 +387,7 @@ public class TestResourceUtils {
       ResourceUtils.resetNodeResources();
       try {
         setupNodeResources(conf, resourceFile);
-        Assert.fail("Expected error with file " + resourceFile);
+        fail("Expected error with file " + resourceFile);
       } catch (YarnRuntimeException e) {
         //Test passed
       }
@@ -411,26 +395,28 @@ public class TestResourceUtils {
   }
 
   @Test
-  public void testGetNodeResourcesRedefineFpgaErrors() throws Exception {
-    Configuration conf = new YarnConfiguration();
-    expexted.expect(YarnRuntimeException.class);
-    expexted.expectMessage("Defined mandatory resource type=yarn.io/fpga");
-    setupResourceTypesInternal(conf,
-        "resource-types-error-redefine-fpga-unit.xml");
+  void testGetNodeResourcesRedefineFpgaErrors() throws Exception {
+    Throwable exception = assertThrows(YarnRuntimeException.class, () -> {
+      Configuration conf = new YarnConfiguration();
+      setupResourceTypesInternal(conf,
+          "resource-types-error-redefine-fpga-unit.xml");
+    });
+    assertTrue(exception.getMessage().contains("Defined mandatory resource type=yarn.io/fpga"));
   }
 
   @Test
-  public void testGetNodeResourcesRedefineGpuErrors() throws Exception {
-    Configuration conf = new YarnConfiguration();
-    expexted.expect(YarnRuntimeException.class);
-    expexted.expectMessage("Defined mandatory resource type=yarn.io/gpu");
-    setupResourceTypesInternal(conf,
-        "resource-types-error-redefine-gpu-unit.xml");
+  void testGetNodeResourcesRedefineGpuErrors() throws Exception {
+    Throwable exception = assertThrows(YarnRuntimeException.class, () -> {
+      Configuration conf = new YarnConfiguration();
+      setupResourceTypesInternal(conf,
+          "resource-types-error-redefine-gpu-unit.xml");
+    });
+    assertTrue(exception.getMessage().contains("Defined mandatory resource type=yarn.io/gpu"));
   }
 
   @Test
-  public void testResourceNameFormatValidation() {
-    String[] validNames = new String[] {
+  void testResourceNameFormatValidation() {
+    String[] validNames = new String[]{
         "yarn.io/gpu",
         "gpu",
         "g_1_2",
@@ -440,7 +426,7 @@ public class TestResourceUtils {
         "a....b",
     };
 
-    String[] invalidNames = new String[] {
+    String[] invalidNames = new String[]{
         "asd/resource/-name",
         "prefix/-resource_1",
         "prefix/0123resource",
@@ -456,7 +442,7 @@ public class TestResourceUtils {
     for (String invalidName : invalidNames) {
       try {
         ResourceUtils.validateNameOfResourceNameAndThrowException(invalidName);
-        Assert.fail("Expected to fail name check, the name=" + invalidName
+        fail("Expected to fail name check, the name=" + invalidName
             + " is illegal.");
       } catch (YarnRuntimeException e) {
         // Expected
@@ -465,7 +451,7 @@ public class TestResourceUtils {
   }
 
   @Test
-  public void testGetResourceInformationWithDiffUnits() throws Exception {
+  void testGetResourceInformationWithDiffUnits() throws Exception {
     Configuration conf = new YarnConfiguration();
     Map<String, Resource> testRun = new HashMap<>();
     setupResourceTypesInternal(conf, "resource-types-4.xml");
@@ -489,11 +475,128 @@ public class TestResourceUtils {
       ResourceUtils.resetNodeResources();
       Map<String, ResourceInformation> actual = setupNodeResources(conf,
           resourceFile);
-      Assert.assertEquals(actual.size(),
+      assertEquals(actual.size(),
           entry.getValue().getResources().length);
       for (ResourceInformation resInfo : entry.getValue().getResources()) {
-        Assert.assertEquals(resInfo, actual.get(resInfo.getName()));
+        assertEquals(resInfo, actual.get(resInfo.getName()));
       }
     }
   }
+
+  @Test
+  void testResourceUnitParsing() throws Exception {
+    Resource res = ResourceUtils.createResourceFromString("memory=20g,vcores=3",
+        ResourceUtils.getResourcesTypeInfo());
+    assertEquals(Resources.createResource(20 * 1024, 3), res);
+
+    res = ResourceUtils.createResourceFromString("memory=20G,vcores=3",
+        ResourceUtils.getResourcesTypeInfo());
+    assertEquals(Resources.createResource(20 * 1024, 3), res);
+
+    res = ResourceUtils.createResourceFromString("memory=20M,vcores=3",
+        ResourceUtils.getResourcesTypeInfo());
+    assertEquals(Resources.createResource(20, 3), res);
+
+    res = ResourceUtils.createResourceFromString("memory=20m,vcores=3",
+        ResourceUtils.getResourcesTypeInfo());
+    assertEquals(Resources.createResource(20, 3), res);
+
+    res = ResourceUtils.createResourceFromString("memory-mb=20,vcores=3",
+        ResourceUtils.getResourcesTypeInfo());
+    assertEquals(Resources.createResource(20, 3), res);
+
+    res = ResourceUtils.createResourceFromString("memory-mb=20m,vcores=3",
+        ResourceUtils.getResourcesTypeInfo());
+    assertEquals(Resources.createResource(20, 3), res);
+
+    res = ResourceUtils.createResourceFromString("memory-mb=20G,vcores=3",
+        ResourceUtils.getResourcesTypeInfo());
+    assertEquals(Resources.createResource(20 * 1024, 3), res);
+
+    // W/o unit for memory means bits, and 20 bits will be rounded to 0
+    res = ResourceUtils.createResourceFromString("memory=20,vcores=3",
+        ResourceUtils.getResourcesTypeInfo());
+    assertEquals(Resources.createResource(0, 3), res);
+
+    // Test multiple resources
+    List<ResourceTypeInfo> resTypes = new ArrayList<>(
+        ResourceUtils.getResourcesTypeInfo());
+    resTypes.add(ResourceTypeInfo.newInstance(ResourceInformation.GPU_URI, ""));
+    ResourceUtils.reinitializeResources(resTypes);
+    res = ResourceUtils.createResourceFromString("memory=2G,vcores=3,gpu=0",
+        resTypes);
+    assertEquals(2 * 1024, res.getMemorySize());
+    assertEquals(0, res.getResourceValue(ResourceInformation.GPU_URI));
+
+    res = ResourceUtils.createResourceFromString("memory=2G,vcores=3,gpu=3",
+        resTypes);
+    assertEquals(2 * 1024, res.getMemorySize());
+    assertEquals(3, res.getResourceValue(ResourceInformation.GPU_URI));
+
+    res = ResourceUtils.createResourceFromString("memory=2G,vcores=3",
+        resTypes);
+    assertEquals(2 * 1024, res.getMemorySize());
+    assertEquals(0, res.getResourceValue(ResourceInformation.GPU_URI));
+
+    res = ResourceUtils.createResourceFromString(
+        "memory=2G,vcores=3,yarn.io/gpu=0", resTypes);
+    assertEquals(2 * 1024, res.getMemorySize());
+    assertEquals(0, res.getResourceValue(ResourceInformation.GPU_URI));
+
+    res = ResourceUtils.createResourceFromString(
+        "memory=2G,vcores=3,yarn.io/gpu=3", resTypes);
+    assertEquals(2 * 1024, res.getMemorySize());
+    assertEquals(3, res.getResourceValue(ResourceInformation.GPU_URI));
+  }
+
+  @Test
+  void testMultipleOpsForResourcesWithTags() throws Exception {
+
+    Configuration conf = new YarnConfiguration();
+    setupResourceTypes(conf, "resource-types-6.xml");
+    Resource resourceA = Resource.newInstance(2, 4);
+    Resource resourceB = Resource.newInstance(3, 6);
+
+    resourceA.setResourceInformation("resource1",
+        ResourceInformation.newInstance("resource1", "T", 5L));
+
+    resourceA.setResourceInformation("resource2",
+        ResourceInformation.newInstance("resource2", "M", 2L));
+    resourceA.setResourceInformation("yarn.io/gpu",
+        ResourceInformation.newInstance("yarn.io/gpu", "", 1));
+    resourceA.setResourceInformation("yarn.io/test-volume",
+        ResourceInformation.newInstance("yarn.io/test-volume", "", 2));
+
+    resourceB.setResourceInformation("resource1",
+        ResourceInformation.newInstance("resource1", "T", 3L));
+
+    resourceB.setResourceInformation("resource2",
+        ResourceInformation.newInstance("resource2", "M", 4L));
+    resourceB.setResourceInformation("yarn.io/gpu",
+        ResourceInformation.newInstance("yarn.io/gpu", "", 2));
+    resourceB.setResourceInformation("yarn.io/test-volume",
+        ResourceInformation.newInstance("yarn.io/test-volume", "", 3));
+
+    Resource addedResource = Resources.add(resourceA, resourceB);
+    assertThat(addedResource.getMemorySize()).isEqualTo(5);
+    assertThat(addedResource.getVirtualCores()).isEqualTo(10);
+    assertThat(addedResource.getResourceInformation("resource1").getValue()).
+        isEqualTo(8);
+
+    // Verify that value of resourceA and resourceB is not added up for
+    // "yarn.io/test-volume".
+    assertThat(addedResource.getResourceInformation("yarn.io/test-volume").
+        getValue()).isEqualTo(2);
+
+    Resource mulResource = Resources.multiplyAndRoundDown(resourceA, 3);
+    assertThat(mulResource.getMemorySize()).isEqualTo(6);
+    assertThat(mulResource.getVirtualCores()).isEqualTo(12);
+    assertThat(mulResource.getResourceInformation("resource1").getValue()).
+        isEqualTo(15);
+
+    // Verify that value of resourceA is not multiplied up for
+    // "yarn.io/test-volume".
+    assertThat(mulResource.getResourceInformation("yarn.io/test-volume").
+        getValue()).isEqualTo(2);
+  }
 }
