@@ -25,7 +25,7 @@ import org.apache.hadoop.yarn.server.federation.policies.exceptions.FederationPo
 import org.apache.hadoop.yarn.server.federation.policies.router.LocalityRouterPolicy;
 import org.apache.hadoop.yarn.server.federation.store.records.SubClusterPolicyConfiguration;
 
-import com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.classification.VisibleForTesting;
 
 /**
  * Policy that allows operator to configure "weights" for routing. This picks a
@@ -65,4 +65,8 @@ public class WeightedLocalityPolicyManager
     this.weightedPolicyInfo = weightedPolicyInfo;
   }
 
+  @Override
+  public boolean isSupportWeightedPolicyInfo() {
+    return true;
+  }
 }
