@@ -55,6 +55,7 @@ public class ContainerInfo {
   protected String nodeHttpAddress;
   protected String nodeId;
   protected Map<String, Long> allocatedResources;
+  private String exposedPorts;
 
   public ContainerInfo() {
     // JAXB needs this
@@ -76,9 +77,9 @@ public class ContainerInfo {
     containerState = container.getContainerState();
     nodeHttpAddress = container.getNodeHttpAddress();
     nodeId = container.getAssignedNode().toString();
+    exposedPorts = container.getExposedPorts();
 
     Resource allocated = container.getAllocatedResource();
-
     if (allocated != null) {
       allocatedMB = allocated.getMemorySize();
       allocatedVCores = allocated.getVirtualCores();
@@ -160,6 +161,10 @@ public class ContainerInfo {
     return Collections.unmodifiableMap(allocatedResources);
   }
 
+  public String getExposedPorts() {
+    return exposedPorts;
+  }
+
   public boolean hasCustomResources() {
     return allocatedResources.size() > 2;
   }
