@@ -23,12 +23,14 @@ import org.apache.hadoop.security.Credentials;
 import org.apache.hadoop.yarn.api.records.ContainerId;
 import org.apache.hadoop.yarn.api.records.ContainerLaunchContext;
 import org.apache.hadoop.yarn.api.records.ContainerStatus;
+import org.apache.hadoop.yarn.api.records.LocalizationStatus;
 import org.apache.hadoop.yarn.api.records.Priority;
 import org.apache.hadoop.yarn.api.records.Resource;
 import org.apache.hadoop.yarn.event.EventHandler;
 import org.apache.hadoop.yarn.security.ContainerTokenIdentifier;
 import org.apache.hadoop.yarn.server.api.protocolrecords.NMContainerStatus;
 import org.apache.hadoop.yarn.server.nodemanager.containermanager.localizer.ResourceSet;
+import org.apache.hadoop.yarn.server.nodemanager.containermanager.runtime.ContainerExecutionException;
 
 import java.util.List;
 import java.util.Map;
@@ -75,12 +77,18 @@ public interface Container extends EventHandler<ContainerEvent> {
 
   void setWorkDir(String workDir);
 
+  String getCsiVolumesRootDir();
+
+  void setCsiVolumesRootDir(String volumesRootDir);
+
   String getLogDir();
 
   void setLogDir(String logDir);
 
   void setIpAndHost(String[] ipAndHost);
 
+  void setExposedPorts(String ports);
+
   String toString();
 
   Priority getPriority();
@@ -105,6 +113,11 @@ public interface Container extends EventHandler<ContainerEvent> {
 
   boolean isRecovering();
 
+  void setContainerRuntimeData(Object containerRuntimeData);
+
+  <T> T getContainerRuntimeData(Class<T> runtimeClazz)
+      throws ContainerExecutionException;
+
   /**
    * Get assigned resource mappings to the container.
    *
@@ -120,6 +133,12 @@ public interface Container extends EventHandler<ContainerEvent> {
    */
   boolean isContainerInFinalStates();
 
+  /**
+   * Get the localization statuses.
+   * @return localization statuses.
+   */
+  List<LocalizationStatus> getLocalizationStatuses();
+
   /**
    * Vector of localization counters to be passed from NM to application
    * container via environment variable {@code $LOCALIZATION_COUNTERS}. See
