@@ -40,9 +40,12 @@ public final class ContainerStartContext {
   private final Map<Path, List<String>> localizedResources;
   private final Path nmPrivateContainerScriptPath;
   private final Path nmPrivateTokensPath;
+  private final Path nmPrivateKeystorePath;
+  private final Path nmPrivateTruststorePath;
   private final String user;
   private final String appId;
   private final Path containerWorkDir;
+  private final Path csiVolumesRootDir;
   private final List<String> localDirs;
   private final List<String> logDirs;
   private final List<String> filecacheDirs;
@@ -57,9 +60,12 @@ public final class ContainerStartContext {
     private Map<Path, List<String>> localizedResources;
     private Path nmPrivateContainerScriptPath;
     private Path nmPrivateTokensPath;
+    private Path nmPrivateKeystorePath;
+    private Path nmPrivateTruststorePath;
     private String user;
     private String appId;
     private Path containerWorkDir;
+    private Path csiVolumesRoot;
     private List<String> localDirs;
     private List<String> logDirs;
     private List<String> filecacheDirs;
@@ -94,6 +100,16 @@ public final class ContainerStartContext {
       return this;
     }
 
+    public Builder setNmPrivateKeystorePath(Path nmPrivateKeystorePath) {
+      this.nmPrivateKeystorePath = nmPrivateKeystorePath;
+      return this;
+    }
+
+    public Builder setNmPrivateTruststorePath(Path nmPrivateTruststorePath) {
+      this.nmPrivateTruststorePath = nmPrivateTruststorePath;
+      return this;
+    }
+
     public Builder setUser(String user) {
       this.user = user;
       return this;
@@ -104,6 +120,11 @@ public final class ContainerStartContext {
       return this;
     }
 
+    public Builder setContainerCsiVolumesRootDir(Path csiVolumesRootDir) {
+      this.csiVolumesRoot = csiVolumesRootDir;
+      return this;
+    }
+
     public Builder setContainerWorkDir(Path containerWorkDir) {
       this.containerWorkDir = containerWorkDir;
       return this;
@@ -161,6 +182,8 @@ public final class ContainerStartContext {
     this.localizedResources = builder.localizedResources;
     this.nmPrivateContainerScriptPath = builder.nmPrivateContainerScriptPath;
     this.nmPrivateTokensPath = builder.nmPrivateTokensPath;
+    this.nmPrivateKeystorePath = builder.nmPrivateKeystorePath;
+    this.nmPrivateTruststorePath = builder.nmPrivateTruststorePath;
     this.user = builder.user;
     this.appId = builder.appId;
     this.containerWorkDir = builder.containerWorkDir;
@@ -172,6 +195,7 @@ public final class ContainerStartContext {
     this.containerLogDirs = builder.containerLogDirs;
     this.userFilecacheDirs = builder.userFilecacheDirs;
     this.applicationLocalDirs = builder.applicationLocalDirs;
+    this.csiVolumesRootDir = builder.csiVolumesRoot;
   }
 
   public Container getContainer() {
@@ -194,6 +218,14 @@ public final class ContainerStartContext {
     return this.nmPrivateTokensPath;
   }
 
+  public Path getNmPrivateKeystorePath() {
+    return this.nmPrivateKeystorePath;
+  }
+
+  public Path getNmPrivateTruststorePath() {
+    return this.nmPrivateTruststorePath;
+  }
+
   public String getUser() {
     return this.user;
   }
@@ -238,4 +270,8 @@ public final class ContainerStartContext {
   public List<String> getApplicationLocalDirs() {
     return Collections.unmodifiableList(this.applicationLocalDirs);
   }
+
+  public Path getCsiVolumesRootDir() {
+    return this.csiVolumesRootDir;
+  }
 }
\ No newline at end of file
