@@ -104,7 +104,12 @@ enum errorcodes {
   ERROR_RUNC_SETUP_FAILED = 76,
   ERROR_RUNC_RUN_FAILED = 77,
   ERROR_RUNC_REAP_LAYER_MOUNTS_FAILED = 78,
-  ERROR_DOCKER_CONTAINER_EXEC_FAILED = 79
+  ERROR_DOCKER_CONTAINER_EXEC_FAILED = 79,
+  CANNOT_GET_EXECUTABLE_NAME_FROM_READLINK = 80,
+  TOO_LONG_EXECUTOR_PATH = 81,
+  CANNOT_GET_EXECUTABLE_NAME_FROM_KERNEL = 82,
+  CANNOT_GET_EXECUTABLE_NAME_FROM_PID = 83,
+  WRONG_PATH_OF_EXECUTABLE = 84
 };
 
 /* Macros for min/max. */
