@@ -31,7 +31,7 @@ import java.util.Set;
 import java.util.TimerTask;
 import java.util.concurrent.TimeoutException;
 
-import com.google.common.collect.ImmutableSet;
+import org.apache.hadoop.thirdparty.com.google.common.collect.ImmutableSet;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.net.ServerSocketUtil;
 import org.apache.hadoop.service.ServiceOperations;
