@@ -29,10 +29,12 @@ import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.security.token.Token;
 import org.apache.hadoop.yarn.api.records.ApplicationAttemptId;
 import org.apache.hadoop.yarn.api.records.ApplicationId;
+import org.apache.hadoop.yarn.api.records.ApplicationSubmissionContext;
 import org.apache.hadoop.yarn.conf.YarnConfiguration;
 import org.apache.hadoop.yarn.exceptions.YarnException;
 import org.apache.hadoop.yarn.security.AMRMTokenIdentifier;
 import org.apache.hadoop.yarn.server.AMHeartbeatRequestHandler;
+import org.apache.hadoop.yarn.server.AMRMClientRelayer;
 import org.apache.hadoop.yarn.server.MockResourceManagerFacade;
 import org.apache.hadoop.yarn.server.uam.UnmanagedAMPoolManager;
 import org.apache.hadoop.yarn.server.uam.UnmanagedApplicationManager;
@@ -52,9 +54,16 @@ public class TestableFederationInterceptor extends FederationInterceptor {
   private AtomicInteger runningIndex = new AtomicInteger(0);
   private MockResourceManagerFacade mockRm;
 
+  private boolean isClientRPC = false;
+  private int retryCount = 0;
+
   public TestableFederationInterceptor() {
   }
 
+  public TestableFederationInterceptor(MockResourceManagerFacade homeRM) {
+    mockRm = homeRM;
+  }
+
   public TestableFederationInterceptor(MockResourceManagerFacade homeRM,
       ConcurrentHashMap<String, MockResourceManagerFacade> secondaries) {
     mockRm = homeRM;
@@ -68,15 +77,19 @@ public class TestableFederationInterceptor extends FederationInterceptor {
   }
 
   @Override
-  protected AMHeartbeatRequestHandler createHomeHeartbeartHandler(
-      Configuration conf, ApplicationId appId) {
-    return new TestableAMRequestHandlerThread(conf, appId);
+  protected AMHeartbeatRequestHandler createHomeHeartbeatHandler(
+      Configuration conf, ApplicationId appId,
+      AMRMClientRelayer rmProxyRelayer) {
+    return new TestableAMRequestHandlerThread(conf, appId, rmProxyRelayer);
   }
 
   @SuppressWarnings("unchecked")
   @Override
   protected <T> T createHomeRMProxy(AMRMProxyApplicationContext appContext,
       Class<T> protocol, UserGroupInformation user) {
+    if (isClientRPC) {
+      return super.createHomeRMProxy(appContext, protocol, user);
+    }
     synchronized (this) {
       if (mockRm == null) {
         mockRm = new MockResourceManagerFacade(
@@ -129,7 +142,7 @@ public class TestableFederationInterceptor extends FederationInterceptor {
   }
 
   /**
-   * Drain all aysnc heartbeat threads, comes in two favors:
+   * Drain all async heartbeat threads, comes in two favors:
    *
    * 1. waitForAsyncHBThreadFinish == false. Only wait for the async threads to
    * pick up all pending heartbeat requests. Not necessarily wait for all
@@ -146,9 +159,9 @@ public class TestableFederationInterceptor extends FederationInterceptor {
 
     LOG.info("waiting to drain home heartbeat handler");
     if (waitForAsyncHBThreadFinish) {
-      getHomeHeartbeartHandler().drainHeartbeatThread();
+      getHomeHeartbeatHandler().drainHeartbeatThread();
     } else {
-      while (getHomeHeartbeartHandler().getRequestQueueSize() > 0) {
+      while (getHomeHeartbeatHandler().getRequestQueueSize() > 0) {
         try {
           Thread.sleep(10);
         } catch (InterruptedException e) {
@@ -203,9 +216,10 @@ public class TestableFederationInterceptor extends FederationInterceptor {
     public UnmanagedApplicationManager createUAM(Configuration conf,
         ApplicationId appId, String queueName, String submitter,
         String appNameSuffix, boolean keepContainersAcrossApplicationAttempts,
-        String rmId) {
+        String rmId, ApplicationSubmissionContext originalAppSubmissionContext) {
       return new TestableUnmanagedApplicationManager(conf, appId, queueName,
-          submitter, appNameSuffix, keepContainersAcrossApplicationAttempts);
+          submitter, appNameSuffix, keepContainersAcrossApplicationAttempts,
+          rmId, originalAppSubmissionContext);
     }
   }
 
@@ -218,10 +232,17 @@ public class TestableFederationInterceptor extends FederationInterceptor {
 
     public TestableUnmanagedApplicationManager(Configuration conf,
         ApplicationId appId, String queueName, String submitter,
-        String appNameSuffix, boolean keepContainersAcrossApplicationAttempts) {
+        String appNameSuffix, boolean keepContainersAcrossApplicationAttempts,
+        String rmName, ApplicationSubmissionContext originalAppSubmissionContext) {
       super(conf, appId, queueName, submitter, appNameSuffix,
-          keepContainersAcrossApplicationAttempts, "TEST");
-      setHandlerThread(new TestableAMRequestHandlerThread(conf, appId));
+          keepContainersAcrossApplicationAttempts, rmName, originalAppSubmissionContext);
+    }
+
+    @Override
+    protected AMHeartbeatRequestHandler createAMHeartbeatRequestHandler(
+        Configuration conf, ApplicationId appId,
+        AMRMClientRelayer rmProxyRelayer) {
+      return new TestableAMRequestHandlerThread(conf, appId, rmProxyRelayer);
     }
 
     /**
@@ -238,29 +259,48 @@ public class TestableFederationInterceptor extends FederationInterceptor {
     }
   }
 
+  @Override
+  protected TokenAndRegisterResponse launchUAMAndRegisterApplicationMaster(YarnConfiguration config,
+      String subClusterId, ApplicationId applicationId) throws IOException, YarnException {
+    if (retryCount > 0) {
+      retryCount--;
+      throw new YarnException("launchUAMAndRegisterApplicationMaster will retry");
+    }
+    return super.launchUAMAndRegisterApplicationMaster(config, subClusterId, applicationId);
+  }
+
+  public void setRetryCount(int retryCount) {
+    this.retryCount = retryCount;
+  }
+
+  public int getRetryCount() {
+    return retryCount;
+  }
+
   /**
-   * Wrap the handler thread so it calls from the same user.
+   * Wrap the handler thread, so it calls from the same user.
    */
   protected class TestableAMRequestHandlerThread
       extends AMHeartbeatRequestHandler {
     public TestableAMRequestHandlerThread(Configuration conf,
-        ApplicationId applicationId) {
-      super(conf, applicationId);
+        ApplicationId applicationId, AMRMClientRelayer rmProxyRelayer) {
+      super(conf, applicationId, rmProxyRelayer);
     }
 
     @Override
     public void run() {
       try {
         getUGIWithToken(getAttemptId())
-            .doAs(new PrivilegedExceptionAction<Object>() {
-              @Override
-              public Object run() {
-                TestableAMRequestHandlerThread.super.run();
-                return null;
-              }
+            .doAs((PrivilegedExceptionAction<Object>) () -> {
+              TestableAMRequestHandlerThread.super.run();
+              return null;
             });
       } catch (Exception e) {
       }
     }
   }
+
+  public void setClientRPC(boolean clientRPC) {
+    this.isClientRPC = clientRPC;
+  }
 }
