@@ -23,12 +23,9 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import org.apache.hadoop.thirdparty.com.google.common.annotations.VisibleForTesting;
-import org.apache.hadoop.thirdparty.com.google.common.collect.Lists;
-import org.apache.hadoop.thirdparty.com.google.common.collect.Maps;
 import org.apache.commons.lang3.StringUtils;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
+import org.apache.hadoop.util.Lists;
+import org.apache.hadoop.util.Time;
 import org.apache.hadoop.classification.InterfaceAudience.Private;
 import org.apache.hadoop.classification.InterfaceStability.Unstable;
 import org.apache.hadoop.conf.Configuration;
@@ -62,6 +59,12 @@ import org.apache.hadoop.yarn.util.resource.ResourceCalculator;
 import org.apache.hadoop.yarn.util.resource.ResourceUtils;
 import org.apache.hadoop.yarn.util.resource.Resources;
 
+import org.apache.hadoop.classification.VisibleForTesting;
+import org.apache.hadoop.thirdparty.com.google.common.collect.Maps;
+
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
 import static org.apache.hadoop.yarn.exceptions
         .InvalidResourceRequestException
         .GREATER_THAN_MAX_RESOURCE_MESSAGE_TEMPLATE;
@@ -196,6 +199,12 @@ public class SchedulerUtils {
   /**
    * Utility method to normalize a resource request, by ensuring that the
    * requested memory is a multiple of minMemory and is not zero.
+   *
+   * @param ask resource request.
+   * @param resourceCalculator {@link ResourceCalculator} the resource
+   * calculator to use.
+   * @param minimumResource minimum Resource.
+   * @param maximumResource maximum Resource.
    */
   @VisibleForTesting
   public static void normalizeRequest(
@@ -212,6 +221,12 @@ public class SchedulerUtils {
    * Utility method to normalize a resource request, by ensuring that the
    * requested memory is a multiple of increment resource and is not zero.
    *
+   * @param ask resource request.
+   * @param resourceCalculator {@link ResourceCalculator} the resource
+   * calculator to use.
+   * @param minimumResource minimum Resource.
+   * @param maximumResource maximum Resource.
+   * @param incrementResource increment Resource.
    * @return normalized resource
    */
   public static Resource getNormalizedResource(
@@ -528,7 +543,14 @@ public class SchedulerUtils {
 
   /**
    * Check queue label expression, check if node label in queue's
-   * node-label-expression existed in clusterNodeLabels if rmContext != null
+   * node-label-expression existed in clusterNodeLabels if rmContext != null.
+   *
+   * @param queueLabels queue Labels.
+   * @param labelExpression label expression.
+   * @param rmContext rmContext.
+   * @return true, if node label in queue's node-label-expression existed in clusterNodeLabels;
+   * otherwise false.
+   *
    */
   public static boolean checkQueueLabelExpression(Set<String> queueLabels,
       String labelExpression, RMContext rmContext) {
@@ -602,4 +624,11 @@ public class SchedulerUtils {
     node.allocateContainer(rmContainer);
     return rmContainer;
   }
+
+  public static boolean isNodeHeartbeated(SchedulerNode node,
+      long skipNodeInterval) {
+    long timeElapsedFromLastHeartbeat =
+        Time.monotonicNow() - node.getLastHeartbeatMonotonicTime();
+    return timeElapsedFromLastHeartbeat <= skipNodeInterval;
+  }
 }
