@@ -18,6 +18,7 @@
 package org.apache.hadoop.yarn.server.resourcemanager.webapp.dao;
 
 import java.util.ArrayList;
+import java.util.Collection;
 
 import javax.xml.bind.annotation.XmlAccessType;
 import javax.xml.bind.annotation.XmlAccessorType;
@@ -43,4 +44,8 @@ public class NodesInfo {
   public void addAll(ArrayList<NodeInfo> nodesInfo) {
     node.addAll(nodesInfo);
   }
+
+  public void addAll(Collection<NodeInfo> nodesInfo) {
+    node.addAll(nodesInfo);
+  }
 }
