@@ -106,7 +106,7 @@ public abstract class RMHATestBase extends ClientBaseWithFixes{
     return am;
   }
 
-  private MockRM initMockRMWithOldConf(final Configuration confForRM1) {
+  private MockRM initMockRMWithOldConf(Configuration confForRM1) {
     return new MockRM(confForRM1, null, false, false) {
       @Override
       protected AdminService createAdminService() {
@@ -122,7 +122,8 @@ public abstract class RMHATestBase extends ClientBaseWithFixes{
           }
 
           @Override
-          protected Configuration loadNewConfiguration() throws IOException, YarnException {
+          protected Configuration loadNewConfiguration()
+              throws IOException, YarnException {
             return confForRM1;
           }
         };
