@@ -19,7 +19,7 @@
 package org.apache.hadoop.yarn.server.resourcemanager;
 
 import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.ipc.ProtobufRpcEngine;
+import org.apache.hadoop.ipc.ProtobufRpcEngine2;
 import org.apache.hadoop.ipc.RPC;
 import org.apache.hadoop.ipc.Server;
 import org.apache.hadoop.net.NetUtils;
@@ -52,6 +52,7 @@ import org.apache.hadoop.yarn.api.protocolrecords.AllocateRequest;
 import org.apache.hadoop.yarn.api.protocolrecords.AllocateResponse;
 import org.apache.hadoop.yarn.server.api.protocolrecords.DistributedSchedulingAllocateRequest;
 import org.apache.hadoop.yarn.server.api.protocolrecords.DistributedSchedulingAllocateResponse;
+import org.apache.hadoop.yarn.server.api.protocolrecords.NMContainerStatus;
 import org.apache.hadoop.yarn.server.api.protocolrecords.NodeHeartbeatResponse;
 import org.apache.hadoop.yarn.server.api.protocolrecords.RegisterDistributedSchedulingAMResponse;
 import org.apache.hadoop.yarn.api.protocolrecords.FinishApplicationMasterRequest;
@@ -71,6 +72,8 @@ import org.apache.hadoop.yarn.server.api.protocolrecords.impl.pb.DistributedSche
 import org.apache.hadoop.yarn.server.api.protocolrecords.impl.pb.DistributedSchedulingAllocateResponsePBImpl;
 import org.apache.hadoop.yarn.server.api.protocolrecords.impl.pb.RegisterDistributedSchedulingAMResponsePBImpl;
 import org.apache.hadoop.yarn.server.api.records.OpportunisticContainersStatus;
+import org.apache.hadoop.yarn.server.metrics.OpportunisticSchedulerMetrics;
+import org.apache.hadoop.yarn.server.resourcemanager.recovery.MemoryRMStateStore;
 import org.apache.hadoop.yarn.server.resourcemanager.rmapp.RMApp;
 import org.apache.hadoop.yarn.server.resourcemanager.rmapp.attempt.AMLivelinessMonitor;
 import org.apache.hadoop.yarn.server.resourcemanager.rmapp.attempt.RMAppAttemptState;
@@ -78,7 +81,6 @@ import org.apache.hadoop.yarn.server.resourcemanager.rmcontainer.RMContainer;
 import org.apache.hadoop.yarn.server.resourcemanager.rmcontainer.RMContainerImpl;
 import org.apache.hadoop.yarn.server.resourcemanager.rmcontainer.RMContainerState;
 import org.apache.hadoop.yarn.server.resourcemanager.rmnode.RMNode;
-import org.apache.hadoop.yarn.server.resourcemanager.rmnode.RMNodeImpl;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.QueueMetrics;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.ResourceScheduler;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.SchedulerApplication;
@@ -89,12 +91,9 @@ import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.Capacity
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.TestUtils;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.common.fica.FiCaSchedulerApp;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.event.AppAttemptRemovedSchedulerEvent;
-import org.apache.hadoop.yarn.server.resourcemanager.scheduler.event.NodeAddedSchedulerEvent;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.event.NodeRemovedSchedulerEvent;
-import org.apache.hadoop.yarn.server.resourcemanager.scheduler.event.NodeUpdateSchedulerEvent;
 
-import org.apache.hadoop.yarn.server.resourcemanager.scheduler.fifo
-    .FifoScheduler;
+import org.apache.hadoop.yarn.server.resourcemanager.scheduler.fifo.FifoScheduler;
 import org.apache.hadoop.yarn.server.resourcemanager.security.RMContainerTokenSecretManager;
 import org.apache.hadoop.yarn.server.scheduler.OpportunisticContainerContext;
 import org.apache.hadoop.yarn.server.scheduler.SchedulerRequestKey;
@@ -103,19 +102,14 @@ import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
-import org.mockito.Mockito;
-
-import com.google.common.base.Supplier;
-
-import static org.junit.Assert.fail;
 
 import java.io.IOException;
 import java.net.InetSocketAddress;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
-import java.util.concurrent.TimeoutException;
 
 /**
  * Test cases for {@link OpportunisticContainerAllocatorAMService}.
@@ -127,6 +121,9 @@ public class TestOpportunisticContainerAllocatorAMService {
   private MockRM rm;
   private DrainDispatcher dispatcher;
 
+  private OpportunisticContainersStatus oppContainersStatus =
+      getOpportunisticStatus();
+
   @Before
   public void createAndStartRM() {
     CapacitySchedulerConfiguration csConf =
@@ -138,6 +135,11 @@ public class TestOpportunisticContainerAllocatorAMService {
         YarnConfiguration.OPPORTUNISTIC_CONTAINER_ALLOCATION_ENABLED, true);
     conf.setInt(
         YarnConfiguration.NM_CONTAINER_QUEUING_SORTING_NODES_INTERVAL_MS, 100);
+    conf.setBoolean(YarnConfiguration.RECOVERY_ENABLED, true);
+    conf.setBoolean(
+        YarnConfiguration.RM_WORK_PRESERVING_RECOVERY_ENABLED, true);
+    conf.set(
+        YarnConfiguration.RM_STORE, MemoryRMStateStore.class.getName());
     startRM(conf);
   }
 
@@ -171,6 +173,8 @@ public class TestOpportunisticContainerAllocatorAMService {
     if (rm != null) {
       rm.stop();
     }
+
+    OpportunisticSchedulerMetrics.resetMetrics();
   }
 
   @Test(timeout = 600000)
@@ -189,40 +193,34 @@ public class TestOpportunisticContainerAllocatorAMService {
     nm3.registerNode();
     nm4.registerNode();
 
+    nm1.nodeHeartbeat(oppContainersStatus, true);
+    nm2.nodeHeartbeat(oppContainersStatus, true);
+    nm3.nodeHeartbeat(oppContainersStatus, true);
+    nm4.nodeHeartbeat(oppContainersStatus, true);
+
     OpportunisticContainerAllocatorAMService amservice =
         (OpportunisticContainerAllocatorAMService) rm
             .getApplicationMasterService();
-    RMApp app1 = rm.submitApp(1 * GB, "app", "user", null, "default");
-    ApplicationAttemptId attemptId =
-        app1.getCurrentAppAttempt().getAppAttemptId();
+    MockRMAppSubmissionData data =
+        MockRMAppSubmissionData.Builder.createWithMemory(1 * GB, rm)
+            .withAppName("app")
+            .withUser("user")
+            .withAcls(null)
+            .withQueue("default")
+            .withUnmanagedAM(false)
+            .build();
+    RMApp app1 = MockRMAppSubmitter.submit(rm, data);
     MockAM am1 = MockRM.launchAndRegisterAM(app1, rm, nm2);
     ResourceScheduler scheduler = rm.getResourceScheduler();
-    RMNode rmNode1 = rm.getRMContext().getRMNodes().get(nm1.getNodeId());
-    RMNode rmNode2 = rm.getRMContext().getRMNodes().get(nm2.getNodeId());
-    RMNode rmNode3 = rm.getRMContext().getRMNodes().get(nm3.getNodeId());
-    RMNode rmNode4 = rm.getRMContext().getRMNodes().get(nm4.getNodeId());
 
-    nm1.nodeHeartbeat(true);
-    nm2.nodeHeartbeat(true);
-    nm3.nodeHeartbeat(true);
-    nm4.nodeHeartbeat(true);
-
-    // Send add and update node events to AM Service.
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode1));
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode2));
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode3));
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode4));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode1));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode2));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode3));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode4));
     // All nodes 1 - 4 will be applicable for scheduling.
-    nm1.nodeHeartbeat(true);
-    nm2.nodeHeartbeat(true);
-    nm3.nodeHeartbeat(true);
-    nm4.nodeHeartbeat(true);
+    nm1.nodeHeartbeat(oppContainersStatus, true);
+    nm2.nodeHeartbeat(oppContainersStatus, true);
+    nm3.nodeHeartbeat(oppContainersStatus, true);
+    nm4.nodeHeartbeat(oppContainersStatus, true);
 
-    Thread.sleep(1000);
+    GenericTestUtils.waitFor(() ->
+        amservice.getLeastLoadedNodes().size() == 4, 10, 10 * 100);
 
     QueueMetrics metrics = ((CapacityScheduler) scheduler).getRootQueue()
         .getMetrics();
@@ -257,7 +255,7 @@ public class TestOpportunisticContainerAllocatorAMService {
             container.getId(), ContainerUpdateType.PROMOTE_EXECUTION_TYPE,
             null, ExecutionType.GUARANTEED)));
     // Node on same host should not result in allocation
-    sameHostDiffNode.nodeHeartbeat(true);
+    sameHostDiffNode.nodeHeartbeat(oppContainersStatus, true);
     rm.drainEvents();
     allocateResponse =  am1.allocate(new ArrayList<>(), new ArrayList<>());
     Assert.assertEquals(0, allocateResponse.getUpdatedContainers().size());
@@ -300,7 +298,7 @@ public class TestOpportunisticContainerAllocatorAMService {
             .getUpdateContainerRequest().getContainerId());
 
     // Ensure after correct node heartbeats, we should get the allocation
-    allocNode.nodeHeartbeat(true);
+    allocNode.nodeHeartbeat(oppContainersStatus, true);
     rm.drainEvents();
     allocateResponse =  am1.allocate(new ArrayList<>(), new ArrayList<>());
     Assert.assertEquals(1, allocateResponse.getUpdatedContainers().size());
@@ -314,10 +312,10 @@ public class TestOpportunisticContainerAllocatorAMService {
     // Allocated cores+mem should have increased, available should decrease
     verifyMetrics(metrics, 14336, 14, 2048, 2, 2);
 
-    nm1.nodeHeartbeat(true);
-    nm2.nodeHeartbeat(true);
-    nm3.nodeHeartbeat(true);
-    nm4.nodeHeartbeat(true);
+    nm1.nodeHeartbeat(oppContainersStatus, true);
+    nm2.nodeHeartbeat(oppContainersStatus, true);
+    nm3.nodeHeartbeat(oppContainersStatus, true);
+    nm4.nodeHeartbeat(oppContainersStatus, true);
     rm.drainEvents();
 
     // Verify that the container is still in ACQUIRED state wrt the RM.
@@ -356,38 +354,30 @@ public class TestOpportunisticContainerAllocatorAMService {
     nm1.registerNode();
     nm2.registerNode();
 
+    nm1.nodeHeartbeat(oppContainersStatus, true);
+    nm2.nodeHeartbeat(oppContainersStatus, true);
+
     OpportunisticContainerAllocatorAMService amservice =
         (OpportunisticContainerAllocatorAMService) rm
             .getApplicationMasterService();
-    RMApp app1 = rm.submitApp(1 * GB, "app", "user", null, "default");
-    ApplicationAttemptId attemptId =
-        app1.getCurrentAppAttempt().getAppAttemptId();
+    MockRMAppSubmissionData data =
+        MockRMAppSubmissionData.Builder.createWithMemory(1 * GB, rm)
+            .withAppName("app")
+            .withUser("user")
+            .withAcls(null)
+            .withQueue("default")
+            .withUnmanagedAM(false)
+            .build();
+    RMApp app1 = MockRMAppSubmitter.submit(rm, data);
     MockAM am1 = MockRM.launchAndRegisterAM(app1, rm, nm2);
     ResourceScheduler scheduler = rm.getResourceScheduler();
-    RMNode rmNode1 = rm.getRMContext().getRMNodes().get(nm1.getNodeId());
-    RMNode rmNode2 = rm.getRMContext().getRMNodes().get(nm2.getNodeId());
-
-    nm1.nodeHeartbeat(true);
-    nm2.nodeHeartbeat(true);
-
-    ((RMNodeImpl) rmNode1)
-        .setOpportunisticContainersStatus(getOppurtunisticStatus(-1, 100));
-    ((RMNodeImpl) rmNode2)
-        .setOpportunisticContainersStatus(getOppurtunisticStatus(-1, 100));
-
-    OpportunisticContainerContext ctxt = ((CapacityScheduler) scheduler)
-        .getApplicationAttempt(attemptId).getOpportunisticContainerContext();
-    // Send add and update node events to AM Service.
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode1));
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode2));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode1));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode2));
 
     // All nodes 1 to 2 will be applicable for scheduling.
-    nm1.nodeHeartbeat(true);
-    nm2.nodeHeartbeat(true);
+    nm1.nodeHeartbeat(oppContainersStatus, true);
+    nm2.nodeHeartbeat(oppContainersStatus, true);
 
-    Thread.sleep(1000);
+    GenericTestUtils.waitFor(() ->
+        amservice.getLeastLoadedNodes().size() == 2, 10, 10 * 100);
 
     QueueMetrics metrics = ((CapacityScheduler) scheduler).getRootQueue()
         .getMetrics();
@@ -481,38 +471,31 @@ public class TestOpportunisticContainerAllocatorAMService {
     nm1.registerNode();
     nm2.registerNode();
 
+    nm1.nodeHeartbeat(oppContainersStatus, true);
+    nm2.nodeHeartbeat(oppContainersStatus, true);
+
     OpportunisticContainerAllocatorAMService amservice =
         (OpportunisticContainerAllocatorAMService) rm
             .getApplicationMasterService();
-    RMApp app1 = rm.submitApp(1 * GB, "app", "user", null, "default");
-    ApplicationAttemptId attemptId =
-        app1.getCurrentAppAttempt().getAppAttemptId();
+    MockRMAppSubmissionData data =
+        MockRMAppSubmissionData.Builder.createWithMemory(1 * GB, rm)
+            .withAppName("app")
+            .withUser("user")
+            .withAcls(null)
+            .withQueue("default")
+            .withUnmanagedAM(false)
+            .build();
+    RMApp app1 = MockRMAppSubmitter.submit(rm, data);
     MockAM am1 = MockRM.launchAndRegisterAM(app1, rm, nm2);
     ResourceScheduler scheduler = rm.getResourceScheduler();
-    RMNode rmNode1 = rm.getRMContext().getRMNodes().get(nm1.getNodeId());
-    RMNode rmNode2 = rm.getRMContext().getRMNodes().get(nm2.getNodeId());
 
-    nm1.nodeHeartbeat(true);
-    nm2.nodeHeartbeat(true);
-
-    ((RMNodeImpl) rmNode1)
-        .setOpportunisticContainersStatus(getOppurtunisticStatus(-1, 100));
-    ((RMNodeImpl) rmNode2)
-        .setOpportunisticContainersStatus(getOppurtunisticStatus(-1, 100));
-
-    OpportunisticContainerContext ctxt = ((CapacityScheduler) scheduler)
-        .getApplicationAttempt(attemptId).getOpportunisticContainerContext();
-    // Send add and update node events to AM Service.
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode1));
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode2));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode1));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode2));
 
     // All nodes 1 to 2 will be applicable for scheduling.
-    nm1.nodeHeartbeat(true);
-    nm2.nodeHeartbeat(true);
+    nm1.nodeHeartbeat(oppContainersStatus, true);
+    nm2.nodeHeartbeat(oppContainersStatus, true);
 
-    Thread.sleep(1000);
+    GenericTestUtils.waitFor(() ->
+        amservice.getLeastLoadedNodes().size() == 2, 10, 10 * 100);
 
     QueueMetrics metrics = ((CapacityScheduler) scheduler).getRootQueue()
         .getMetrics();
@@ -593,31 +576,28 @@ public class TestOpportunisticContainerAllocatorAMService {
     createAndStartRMWithAutoUpdateContainer();
     MockNM nm1 = new MockNM("h1:1234", 4096, rm.getResourceTrackerService());
     nm1.registerNode();
+    nm1.nodeHeartbeat(oppContainersStatus, true);
 
     OpportunisticContainerAllocatorAMService amservice =
         (OpportunisticContainerAllocatorAMService) rm
             .getApplicationMasterService();
-    RMApp app1 = rm.submitApp(1 * GB, "app", "user", null, "default");
-    ApplicationAttemptId attemptId =
-        app1.getCurrentAppAttempt().getAppAttemptId();
+    MockRMAppSubmissionData data =
+        MockRMAppSubmissionData.Builder.createWithMemory(1 * GB, rm)
+            .withAppName("app")
+            .withUser("user")
+            .withAcls(null)
+            .withQueue("default")
+            .withUnmanagedAM(false)
+            .build();
+    RMApp app1 = MockRMAppSubmitter.submit(rm, data);
     MockAM am1 = MockRM.launchAndRegisterAM(app1, rm, nm1);
     ResourceScheduler scheduler = rm.getResourceScheduler();
     RMNode rmNode1 = rm.getRMContext().getRMNodes().get(nm1.getNodeId());
 
-    nm1.nodeHeartbeat(true);
+    nm1.nodeHeartbeat(oppContainersStatus, true);
 
-    ((RMNodeImpl) rmNode1)
-        .setOpportunisticContainersStatus(getOppurtunisticStatus(-1, 100));
-
-    OpportunisticContainerContext ctxt =
-        ((CapacityScheduler) scheduler).getApplicationAttempt(attemptId)
-            .getOpportunisticContainerContext();
-    // Send add and update node events to AM Service.
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode1));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode1));
-
-    nm1.nodeHeartbeat(true);
-    Thread.sleep(1000);
+    GenericTestUtils.waitFor(() ->
+        amservice.getLeastLoadedNodes().size() == 1, 10, 10 * 100);
 
     AllocateResponse allocateResponse = am1.allocate(Arrays.asList(
         ResourceRequest.newInstance(Priority.newInstance(1), "*",
@@ -713,7 +693,7 @@ public class TestOpportunisticContainerAllocatorAMService {
     Assert.assertEquals(Resource.newInstance(1 * GB, 1),
         response.getContainersToUpdate().get(0).getResource());
 
-    nm1.nodeHeartbeat(true);
+    nm1.nodeHeartbeat(oppContainersStatus, true);
     // DEMOTE the container
     allocateResponse = am1.sendContainerUpdateRequest(Arrays.asList(
         UpdateContainerRequest.newInstance(3, container.getId(),
@@ -735,7 +715,7 @@ public class TestOpportunisticContainerAllocatorAMService {
         uc.getContainer().getExecutionType());
     // Check that the container is updated in NM through NM heartbeat response
     if (response.getContainersToUpdate().size() == 0) {
-      response = nm1.nodeHeartbeat(true);
+      response = nm1.nodeHeartbeat(oppContainersStatus, true);
     }
     Assert.assertEquals(1, response.getContainersToUpdate().size());
     Assert.assertEquals(ExecutionType.OPPORTUNISTIC,
@@ -752,12 +732,237 @@ public class TestOpportunisticContainerAllocatorAMService {
     Assert.assertEquals(allocatedContainers, metrics.getAllocatedContainers());
   }
 
+  @Test(timeout = 60000)
+  public void testOpportunisticSchedulerMetrics() throws Exception {
+    HashMap<NodeId, MockNM> nodes = new HashMap<>();
+    MockNM nm1 = new MockNM("h1:1234", 4096, rm.getResourceTrackerService());
+    nodes.put(nm1.getNodeId(), nm1);
+    MockNM nm2 = new MockNM("h2:1234", 4096, rm.getResourceTrackerService());
+    nodes.put(nm2.getNodeId(), nm2);
+    nm1.registerNode();
+    nm2.registerNode();
+
+    nm1.nodeHeartbeat(oppContainersStatus, true);
+    nm2.nodeHeartbeat(oppContainersStatus, true);
+
+    OpportunisticSchedulerMetrics metrics =
+        OpportunisticSchedulerMetrics.getMetrics();
+
+    int allocContainers = metrics.getAllocatedContainers();
+    long aggrAllocatedContainers = metrics.getAggregatedAllocatedContainers();
+    long aggrOffSwitchContainers = metrics.getAggregatedOffSwitchContainers();
+    long aggrReleasedContainers = metrics.getAggregatedReleasedContainers();
+
+    OpportunisticContainerAllocatorAMService amservice =
+        (OpportunisticContainerAllocatorAMService) rm
+            .getApplicationMasterService();
+    RMApp app1 = MockRMAppSubmitter.submit(rm,
+        MockRMAppSubmissionData.Builder.createWithMemory(1 * GB, rm)
+            .withAppName("app")
+            .withUser("user")
+            .withAcls(null)
+            .withQueue("default")
+            .build());
+    ApplicationAttemptId attemptId =
+        app1.getCurrentAppAttempt().getAppAttemptId();
+    MockAM am1 = MockRM.launchAndRegisterAM(app1, rm, nm2);
+    ResourceScheduler scheduler = rm.getResourceScheduler();
+
+    // All nodes 1 to 2 will be applicable for scheduling.
+    nm1.nodeHeartbeat(oppContainersStatus, true);
+    nm2.nodeHeartbeat(oppContainersStatus, true);
+
+    GenericTestUtils.waitFor(() ->
+        amservice.getLeastLoadedNodes().size() == 2, 10, 10 * 100);
+
+    AllocateResponse allocateResponse = am1.allocate(Arrays.asList(
+        ResourceRequest.newInstance(Priority.newInstance(1), "*",
+            Resources.createResource(1 * GB), 2, true, null,
+            ExecutionTypeRequest
+                .newInstance(ExecutionType.OPPORTUNISTIC, true))), null);
+
+    List<Container> allocatedContainers = allocateResponse
+        .getAllocatedContainers();
+    Assert.assertEquals(2, allocatedContainers.size());
+
+    Assert.assertEquals(allocContainers + 2, metrics.getAllocatedContainers());
+    Assert.assertEquals(aggrAllocatedContainers + 2,
+        metrics.getAggregatedAllocatedContainers());
+    Assert.assertEquals(aggrOffSwitchContainers + 2,
+        metrics.getAggregatedOffSwitchContainers());
+
+    Container container = allocatedContainers.get(0);
+    MockNM allocNode = nodes.get(container.getNodeId());
+
+    // Start Container in NM
+    allocNode.nodeHeartbeat(Arrays.asList(
+        ContainerStatus.newInstance(container.getId(),
+            ExecutionType.OPPORTUNISTIC, ContainerState.RUNNING, "", 0)),
+        true);
+    rm.drainEvents();
+
+    // Verify that container is actually running wrt the RM..
+    RMContainer rmContainer = ((CapacityScheduler) scheduler)
+        .getApplicationAttempt(
+            container.getId().getApplicationAttemptId()).getRMContainer(
+            container.getId());
+    Assert.assertEquals(RMContainerState.RUNNING, rmContainer.getState());
+
+    // Container Completed in the NM
+    allocNode.nodeHeartbeat(Arrays.asList(
+        ContainerStatus.newInstance(container.getId(),
+            ExecutionType.OPPORTUNISTIC, ContainerState.COMPLETE, "", 0)),
+        true);
+    rm.drainEvents();
+
+    // Verify that container has been removed..
+    rmContainer = ((CapacityScheduler) scheduler)
+        .getApplicationAttempt(
+            container.getId().getApplicationAttemptId()).getRMContainer(
+            container.getId());
+    Assert.assertNull(rmContainer);
+
+    Assert.assertEquals(allocContainers + 1, metrics.getAllocatedContainers());
+    Assert.assertEquals(aggrReleasedContainers + 1,
+        metrics.getAggregatedReleasedContainers());
+  }
+
+  /**
+   * Tests that, if a node has running opportunistic containers when the RM
+   * is down, RM is able to reflect the opportunistic containers
+   * in its metrics upon RM recovery.
+   */
+  @Test
+  public void testMetricsRetainsAllocatedOpportunisticAfterRMRestart()
+      throws Exception {
+    final MockRMAppSubmissionData appSubmissionData =
+        MockRMAppSubmissionData.Builder.createWithMemory(GB, rm)
+            .withAppName("app")
+            .withUser("user")
+            .withAcls(null)
+            .withQueue("default")
+            .build();
+
+    MockNM nm1 = new MockNM("h:1234", 4096, rm.getResourceTrackerService());
+    nm1.registerNode();
+
+    final RMApp app = MockRMAppSubmitter.submit(rm, appSubmissionData);
+
+    final ApplicationAttemptId appAttemptId =
+        app.getCurrentAppAttempt().getAppAttemptId();
+
+    MockRM.launchAndRegisterAM(app, rm, nm1);
+
+    final OpportunisticSchedulerMetrics metrics =
+        OpportunisticSchedulerMetrics.getMetrics();
+
+    // We start with ID 2, since AMContainer is ID 1
+    final ContainerId recoverOContainerId2 = ContainerId.newContainerId(
+        appAttemptId, 2);
+
+    final Resource fakeResource = Resource.newInstance(1024, 1);
+    final String fakeDiagnostics = "recover container";
+    final Priority fakePriority = Priority.newInstance(0);
+
+    final NMContainerStatus recoverOContainerReport1 =
+        NMContainerStatus.newInstance(
+            recoverOContainerId2, 0, ContainerState.RUNNING,
+            fakeResource, fakeDiagnostics, 0,
+            fakePriority, 0, null,
+            ExecutionType.OPPORTUNISTIC, -1);
+
+    // Make sure that numbers start with 0
+    Assert.assertEquals(0, metrics.getAllocatedContainers());
+
+    // Recover one OContainer only
+    rm.registerNode("h2:1234", 4096, 1,
+        Collections.singletonList(
+            appAttemptId.getApplicationId()),
+        Collections.singletonList(recoverOContainerReport1));
+
+    Assert.assertEquals(1, metrics.getAllocatedContainers());
+
+    // Recover two OContainers at once
+    final ContainerId recoverOContainerId3 = ContainerId.newContainerId(
+        appAttemptId, 3);
+
+    final ContainerId recoverOContainerId4 = ContainerId.newContainerId(
+        appAttemptId, 4);
+
+    final NMContainerStatus recoverOContainerReport2 =
+        NMContainerStatus.newInstance(
+            recoverOContainerId2, 0, ContainerState.RUNNING,
+            fakeResource, fakeDiagnostics, 0,
+            fakePriority, 0, null,
+            ExecutionType.OPPORTUNISTIC, -1);
+
+    final NMContainerStatus recoverOContainerReport3 =
+        NMContainerStatus.newInstance(
+            recoverOContainerId3, 0, ContainerState.RUNNING,
+            fakeResource, fakeDiagnostics, 0,
+            fakePriority, 0, null,
+            ExecutionType.OPPORTUNISTIC, -1);
+
+    rm.registerNode(
+        "h3:1234", 4096, 10,
+        Collections.singletonList(
+            appAttemptId.getApplicationId()),
+        Arrays.asList(recoverOContainerReport2, recoverOContainerReport3));
+
+    Assert.assertEquals(3, metrics.getAllocatedContainers());
+
+    // Make sure that the recovered GContainer
+    // does not increment OContainer count
+    final ContainerId recoverGContainerId = ContainerId.newContainerId(
+        appAttemptId, 5);
+
+    final NMContainerStatus recoverGContainerReport =
+        NMContainerStatus.newInstance(
+            recoverGContainerId, 0, ContainerState.RUNNING,
+            fakeResource, fakeDiagnostics, 0,
+            fakePriority, 0, null,
+            ExecutionType.GUARANTEED, -1);
+
+    rm.registerNode(
+        "h4:1234", 4096, 10,
+        Collections.singletonList(
+            appAttemptId.getApplicationId()),
+        Collections.singletonList(recoverGContainerReport));
+
+    Assert.assertEquals(3, metrics.getAllocatedContainers());
+
+    final ContainerId completedOContainerId = ContainerId.newContainerId(
+        appAttemptId, 6);
+
+    final NMContainerStatus completedOContainerReport =
+        NMContainerStatus.newInstance(
+            completedOContainerId, 0, ContainerState.COMPLETE,
+            fakeResource, fakeDiagnostics, 0,
+            fakePriority, 0, null,
+            ExecutionType.OPPORTUNISTIC, -1);
+
+    // Tests that completed containers are not recorded
+    rm.registerNode(
+        "h5:1234", 4096, 10,
+        Collections.singletonList(
+            appAttemptId.getApplicationId()),
+        Collections.singletonList(completedOContainerReport));
+
+    Assert.assertEquals(3, metrics.getAllocatedContainers());
+  }
+
   @Test(timeout = 60000)
   public void testAMCrashDuringAllocate() throws Exception {
     MockNM nm = new MockNM("h:1234", 4096, rm.getResourceTrackerService());
     nm.registerNode();
 
-    RMApp app = rm.submitApp(1 * GB, "app", "user", null, "default");
+    RMApp app = MockRMAppSubmitter.submit(rm,
+        MockRMAppSubmissionData.Builder.createWithMemory(1 * GB, rm)
+            .withAppName("app")
+            .withUser("user")
+            .withAcls(null)
+            .withQueue("default")
+            .build());
     ApplicationAttemptId attemptId0 =
         app.getCurrentAppAttempt().getAppAttemptId();
     MockAM am = MockRM.launchAndRegisterAM(app, rm, nm);
@@ -787,30 +992,36 @@ public class TestOpportunisticContainerAllocatorAMService {
     MockNM nm2 = new MockNM("h2:1234", 4096, rm.getResourceTrackerService());
     nm1.registerNode();
     nm2.registerNode();
+
+    nm1.nodeHeartbeat(oppContainersStatus, true);
+    nm2.nodeHeartbeat(oppContainersStatus, true);
+
     OpportunisticContainerAllocatorAMService amservice =
         (OpportunisticContainerAllocatorAMService) rm
             .getApplicationMasterService();
-    RMApp app1 = rm.submitApp(1 * GB, "app", "user", null, "default");
+    MockRMAppSubmissionData data =
+        MockRMAppSubmissionData.Builder.createWithMemory(1 * GB, rm)
+            .withAppName("app")
+            .withUser("user")
+            .withAcls(null)
+            .withQueue("default")
+            .withUnmanagedAM(false)
+            .build();
+    RMApp app1 = MockRMAppSubmitter.submit(rm, data);
     ApplicationAttemptId attemptId =
         app1.getCurrentAppAttempt().getAppAttemptId();
     MockAM am1 = MockRM.launchAndRegisterAM(app1, rm, nm2);
     ResourceScheduler scheduler = rm.getResourceScheduler();
     RMNode rmNode1 = rm.getRMContext().getRMNodes().get(nm1.getNodeId());
     RMNode rmNode2 = rm.getRMContext().getRMNodes().get(nm2.getNodeId());
-    nm1.nodeHeartbeat(true);
-    nm2.nodeHeartbeat(true);
-    ((RMNodeImpl) rmNode1)
-        .setOpportunisticContainersStatus(getOppurtunisticStatus(-1, 100));
-    ((RMNodeImpl) rmNode2)
-        .setOpportunisticContainersStatus(getOppurtunisticStatus(-1, 100));
+
     OpportunisticContainerContext ctxt = ((CapacityScheduler) scheduler)
         .getApplicationAttempt(attemptId).getOpportunisticContainerContext();
-    // Send add and update node events to AM Service.
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode1));
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode2));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode1));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode2));
+
     // Both node 1 and node 2 will be applicable for scheduling.
+    nm1.nodeHeartbeat(oppContainersStatus, true);
+    nm2.nodeHeartbeat(oppContainersStatus, true);
+
     for (int i = 0; i < 10; i++) {
       am1.allocate(
           Arrays.asList(ResourceRequest.newInstance(Priority.newInstance(1),
@@ -845,11 +1056,17 @@ public class TestOpportunisticContainerAllocatorAMService {
   @Test(timeout = 60000)
   public void testAppAttemptRemovalAfterNodeRemoval() throws Exception {
     MockNM nm = new MockNM("h:1234", 4096, rm.getResourceTrackerService());
+
     nm.registerNode();
-    OpportunisticContainerAllocatorAMService amservice =
-        (OpportunisticContainerAllocatorAMService) rm
-            .getApplicationMasterService();
-    RMApp app = rm.submitApp(1 * GB, "app", "user", null, "default");
+    nm.nodeHeartbeat(oppContainersStatus, true);
+
+    RMApp app = MockRMAppSubmitter.submit(rm,
+        MockRMAppSubmissionData.Builder.createWithMemory(1 * GB, rm)
+            .withAppName("app")
+            .withUser("user")
+            .withAcls(null)
+            .withQueue("default")
+            .build());
     ApplicationAttemptId attemptId =
         app.getCurrentAppAttempt().getAppAttemptId();
     MockAM am = MockRM.launchAndRegisterAM(app, rm, nm);
@@ -857,21 +1074,12 @@ public class TestOpportunisticContainerAllocatorAMService {
     SchedulerApplicationAttempt schedulerAttempt =
         ((CapacityScheduler)scheduler).getApplicationAttempt(attemptId);
     RMNode rmNode1 = rm.getRMContext().getRMNodes().get(nm.getNodeId());
-    nm.nodeHeartbeat(true);
-    ((RMNodeImpl) rmNode1)
-        .setOpportunisticContainersStatus(getOppurtunisticStatus(-1, 100));
-    // Send add and update node events to AM Service.
-    amservice.handle(new NodeAddedSchedulerEvent(rmNode1));
-    amservice.handle(new NodeUpdateSchedulerEvent(rmNode1));
-    try {
-      GenericTestUtils.waitFor(new Supplier<Boolean>() {
-        @Override public Boolean get() {
-          return scheduler.getNumClusterNodes() == 1;
-        }
-      }, 10, 200 * 100);
-    }catch (TimeoutException e) {
-      fail("timed out while waiting for NM to add.");
-    }
+
+    nm.nodeHeartbeat(oppContainersStatus, true);
+
+    GenericTestUtils.waitFor(() ->
+        scheduler.getNumClusterNodes() == 1, 10, 200 * 100);
+
     AllocateResponse allocateResponse = am.allocate(
             Arrays.asList(ResourceRequest.newInstance(Priority.newInstance(1),
                 "*", Resources.createResource(1 * GB), 2, true, null,
@@ -882,15 +1090,10 @@ public class TestOpportunisticContainerAllocatorAMService {
         .getAllocatedContainers();
     Container container = allocatedContainers.get(0);
     scheduler.handle(new NodeRemovedSchedulerEvent(rmNode1));
-    try {
-      GenericTestUtils.waitFor(new Supplier<Boolean>() {
-        @Override public Boolean get() {
-          return scheduler.getNumClusterNodes() == 0;
-        }
-      }, 10, 200 * 100);
-    }catch (TimeoutException e) {
-      fail("timed out while waiting for NM to remove.");
-    }
+
+    GenericTestUtils.waitFor(() ->
+        scheduler.getNumClusterNodes() == 0, 10, 200 * 100);
+
     //test YARN-9165
     RMContainer rmContainer = null;
     rmContainer = SchedulerUtils.createOpportunisticRmContainer(
@@ -901,20 +1104,24 @@ public class TestOpportunisticContainerAllocatorAMService {
         schedulerAttempt.getApplicationAttemptId(), container.getNodeId(),
         schedulerAttempt.getUser(), rm.getRMContext(), true);
     }
-    assert(rmContainer!=null);
     //test YARN-9164
     schedulerAttempt.addRMContainer(container.getId(), rmContainer);
     scheduler.handle(new AppAttemptRemovedSchedulerEvent(attemptId,
         RMAppAttemptState.FAILED, false));
   }
 
+  private OpportunisticContainersStatus getOpportunisticStatus() {
+    return getOppurtunisticStatus(-1, 100, 1000);
+  }
+
   private OpportunisticContainersStatus getOppurtunisticStatus(int waitTime,
-      int queueLength) {
-    OpportunisticContainersStatus status1 =
-        Mockito.mock(OpportunisticContainersStatus.class);
-    Mockito.when(status1.getEstimatedQueueWaitTime()).thenReturn(waitTime);
-    Mockito.when(status1.getWaitQueueLength()).thenReturn(queueLength);
-    return status1;
+      int queueLength, int queueCapacity) {
+    OpportunisticContainersStatus status =
+        OpportunisticContainersStatus.newInstance();
+    status.setEstimatedQueueWaitTime(waitTime);
+    status.setOpportQueueCapacity(queueCapacity);
+    status.setWaitQueueLength(queueLength);
+    return status;
   }
 
   // Test if the OpportunisticContainerAllocatorAMService can handle both
@@ -972,7 +1179,7 @@ public class TestOpportunisticContainerAllocatorAMService {
     // Verify that the OpportunisticContainerAllocatorAMSercvice can handle
     // vanilla ApplicationMasterProtocol clients
     RPC.setProtocolEngine(conf, ApplicationMasterProtocolPB.class,
-        ProtobufRpcEngine.class);
+        ProtobufRpcEngine2.class);
     ApplicationMasterProtocolPB ampProxy =
         RPC.getProxy(ApplicationMasterProtocolPB
             .class, 1, NetUtils.getConnectAddress(server), conf);
@@ -1007,7 +1214,7 @@ public class TestOpportunisticContainerAllocatorAMService {
     // Verify that the DistrubutedSchedulingService can handle the
     // DistributedSchedulingAMProtocol clients as well
     RPC.setProtocolEngine(conf, DistributedSchedulingAMProtocolPB.class,
-        ProtobufRpcEngine.class);
+        ProtobufRpcEngine2.class);
     DistributedSchedulingAMProtocolPB dsProxy =
         RPC.getProxy(DistributedSchedulingAMProtocolPB
             .class, 1, NetUtils.getConnectAddress(server), conf);
