@@ -26,6 +26,7 @@ import java.util.Map;
 import java.util.Set;
 
 import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.util.Sets;
 import org.apache.hadoop.yarn.api.records.ApplicationResourceUsageReport;
 import org.apache.hadoop.yarn.api.records.ContainerExitStatus;
 import org.apache.hadoop.yarn.api.records.ContainerId;
@@ -54,9 +55,8 @@ import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
 
-import com.google.common.collect.ImmutableMap;
-import com.google.common.collect.ImmutableSet;
-import com.google.common.collect.Sets;
+import org.apache.hadoop.thirdparty.com.google.common.collect.ImmutableMap;
+import org.apache.hadoop.thirdparty.com.google.common.collect.ImmutableSet;
 
 public class TestCapacitySchedulerNodeLabelUpdate {
   private final int GB = 1024;
@@ -674,6 +674,7 @@ public class TestCapacitySchedulerNodeLabelUpdate {
     checkAMResourceLimit(rm, "a", 640, "y");
     checkAMResourceLimit(rm, "a", 0, "z");
     checkAMResourceLimit(rm, "a", 0, "");
+    rm.stop();
   }
 
   @Test(timeout = 60000)
@@ -952,5 +953,6 @@ public class TestCapacitySchedulerNodeLabelUpdate {
     // Last node with label x is replaced by CLI or REST.
     Assert.assertEquals(0,
         waitForNodeLabelSchedulerEventUpdate(rm, "x", 0, 3000L));
+    rm.stop();
   }
 }
