@@ -24,7 +24,6 @@ import java.util.Collection;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import org.apache.hadoop.yarn.api.records.Resource;
 import org.junit.Assert;
 import org.junit.Test;
 import org.junit.runner.RunWith;
@@ -47,7 +46,9 @@ public class TestQueueCapacities {
         { "AbsoluteMaximumCapacity" },
         { "MaxAMResourcePercentage" },
         { "ReservedCapacity" },
-        { "AbsoluteReservedCapacity" }});
+        { "AbsoluteReservedCapacity" },
+        { "Weight" },
+        { "NormalizedWeight" }});
   }
 
   public TestQueueCapacities(String suffix) {
@@ -105,9 +106,6 @@ public class TestQueueCapacities {
   private void internalTestModifyAndRead(String label) throws Exception {
     QueueCapacities qc = new QueueCapacities(false);
 
-    // First get returns 0 always
-    Assert.assertEquals(0f, get(qc, suffix, label), 1e-8);
-
     // Set to 1, and check
     set(qc, suffix, label, 1f);
     Assert.assertEquals(1f, get(qc, suffix, label), 1e-8);
@@ -117,15 +115,19 @@ public class TestQueueCapacities {
     Assert.assertEquals(2f, get(qc, suffix, label), 1e-8);
   }
 
-  void check(int mem, int cpu, Resource res) {
-    Assert.assertEquals(mem, res.getMemorySize());
-    Assert.assertEquals(cpu, res.getVirtualCores());
-  }
-
   @Test
   public void testModifyAndRead() throws Exception {
     LOG.info("Test - " + suffix);
     internalTestModifyAndRead(null);
     internalTestModifyAndRead("label");
   }
+
+  @Test
+  public void testDefaultValues() {
+    QueueCapacities qc = new QueueCapacities(false);
+    Assert.assertEquals(-1, qc.getWeight(""), 1e-6);
+    Assert.assertEquals(-1, qc.getWeight("x"), 1e-6);
+    Assert.assertEquals(0, qc.getCapacity(""), 1e-6);
+    Assert.assertEquals(0, qc.getCapacity("x"), 1e-6);
+  }
 }
\ No newline at end of file
