@@ -18,8 +18,8 @@
 
 package org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity;
 
-import com.google.common.collect.ImmutableList;
-import com.google.common.collect.ImmutableSet;
+import org.apache.hadoop.thirdparty.com.google.common.collect.ImmutableList;
+import org.apache.hadoop.thirdparty.com.google.common.collect.ImmutableSet;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.yarn.api.protocolrecords.AllocateRequest;
 import org.apache.hadoop.yarn.api.records.ExecutionType;
@@ -582,6 +582,7 @@ public class TestSchedulingRequestContainerAllocation {
     } catch (Exception e) {
       Assert.fail("Failed to reject invalid scheduling request");
     }
+    rm1.stop();
   }
 
   private static void doNodeHeartbeat(MockNM... nms) throws Exception {
@@ -862,7 +863,7 @@ public class TestSchedulingRequestContainerAllocation {
     try {
       rm.start();
 
-      MockNM nm1 = rm.registerNode("192.168.0.1:1234:", 100*GB, 100);
+      MockNM nm1 = rm.registerNode("192.168.0.1:1234", 100*GB, 100);
       MockNM nm2 = rm.registerNode("192.168.0.2:1234", 100*GB, 100);
       MockNM nm3 = rm.registerNode("192.168.0.3:1234", 100*GB, 100);
       MockNM nm4 = rm.registerNode("192.168.0.4:1234", 100*GB, 100);
