@@ -16,22 +16,25 @@
 
 package org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter;
 
-import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.CapacitySchedulerConfiguration.PREFIX;
 import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.DYNAMIC_MAX_ASSIGN;
 import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.MAX_CAPACITY_PERCENTAGE;
 import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.MAX_CHILD_CAPACITY;
 import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.QUEUE_AUTO_CREATE;
 import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.RESERVATION_SYSTEM;
-import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.SPECIFIED_NOT_FIRST;
-import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.USER_MAX_APPS_DEFAULT;
-import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.USER_MAX_RUNNING_APPS;
+import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.CHILD_STATIC_DYNAMIC_CONFLICT;
+import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.PARENT_CHILD_CREATE_DIFFERS;
+import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.FAIR_AS_DRF;
+import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.MAX_RESOURCES;
+import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.MIN_RESOURCES;
+import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.PARENT_DYNAMIC_CREATE;
+
 import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.RuleAction.ABORT;
 import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigToCSConfigRuleHandler.RuleAction.WARNING;
 import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 import static org.mockito.ArgumentMatchers.any;
+import static org.mockito.ArgumentMatchers.anyBoolean;
 import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.verifyZeroInteractions;
 
@@ -43,10 +46,12 @@ import java.util.Map;
 import org.apache.commons.io.FileUtils;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.service.ServiceStateException;
+import org.apache.hadoop.yarn.api.records.QueueACL;
 import org.apache.hadoop.yarn.api.records.Resource;
 import org.apache.hadoop.yarn.conf.YarnConfiguration;
 import org.apache.hadoop.yarn.server.resourcemanager.placement.PlacementManager;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.CapacitySchedulerConfiguration;
+import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.placement.schema.MappingRulesDescription;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.FairSchedulerConfiguration;
 import org.apache.hadoop.yarn.util.resource.DominantResourceCalculator;
 import org.junit.After;
@@ -59,6 +64,8 @@ import org.mockito.Mock;
 import org.mockito.Mockito;
 import org.mockito.junit.MockitoJUnitRunner;
 
+import com.fasterxml.jackson.databind.ObjectMapper;
+
 
 /**
  * Unit tests for FSConfigToCSConfigConverter.
@@ -125,6 +132,7 @@ public class TestFSConfigToCSConfigConverter {
     config = new Configuration(false);
     config.set(FairSchedulerConfiguration.ALLOCATION_FILE, FAIR_SCHEDULER_XML);
     config.setBoolean(FairSchedulerConfiguration.MIGRATION_MODE, true);
+    config.setBoolean(FairSchedulerConfiguration.USER_AS_DEFAULT_QUEUE, true);
     createConverter();
     converterTestCommons = new FSConfigConverterTestCommons();
     converterTestCommons.setUp();
@@ -139,6 +147,7 @@ public class TestFSConfigToCSConfigConverter {
     converter = new FSConfigToCSConfigConverter(ruleHandler,
         createDefaultConversionOptions());
     converter.setClusterResource(CLUSTER_RESOURCE);
+    converter.setConvertPlacementRules(false);
   }
 
   private FSConfigToCSConfigConverterParams.Builder
@@ -161,18 +170,44 @@ public class TestFSConfigToCSConfigConverter {
   public void testDefaultMaxAMShare() throws Exception {
     converter.convert(config);
 
-    Configuration conf = converter.getCapacitySchedulerConfig();
-    String maxAmShare =
-        conf.get(CapacitySchedulerConfiguration.
-            MAXIMUM_APPLICATION_MASTERS_RESOURCE_PERCENT);
+    CapacitySchedulerConfiguration conf = converter.getCapacitySchedulerConfig();
+    Float maxAmShare =
+        conf.getMaximumApplicationMasterResourcePercent();
 
-    assertEquals("Default max AM share", "0.16", maxAmShare);
+    assertEquals("Default max AM share", 0.16f, maxAmShare, 0.0f);
 
-    assertEquals("root.admins.alice max-am-resource-percent", "0.15",
-        conf.get(PREFIX + "root.admins.alice.maximum-am-resource-percent"));
+    assertEquals("root.admins.alice max-am-resource-percent", 0.15f,
+        conf.getMaximumApplicationMasterResourcePerQueuePercent("root.admins.alice"),
+            0.0f);
 
-    assertNull("root.users.joe maximum-am-resource-percent should be null",
-        conf.get(PREFIX + "root.users.joe maximum-am-resource-percent"));
+    //root.users.joe don’t have maximum-am-resource-percent set
+    // so falling back to the global value
+    assertEquals("root.users.joe maximum-am-resource-percent", 0.16f,
+        conf.getMaximumApplicationMasterResourcePerQueuePercent("root.users.joe"),
+            0.0f);
+  }
+
+  @Test
+  public void testDefaultUserLimitFactor() throws Exception {
+    converter.convert(config);
+
+    CapacitySchedulerConfiguration conf = converter.getCapacitySchedulerConfig();
+
+    assertEquals("root.users user-limit-factor", 1.0f,
+            conf.getUserLimitFactor("root.users"), 0.0f);
+    assertEquals("root.users auto-queue-creation-v2.enabled", true,
+            conf.isAutoQueueCreationV2Enabled("root.users"));
+
+    assertEquals("root.default user-limit-factor", -1.0f,
+            conf.getUserLimitFactor("root.default"), 0.0f);
+
+    assertEquals("root.users.joe user-limit-factor", -1.0f,
+            conf.getUserLimitFactor("root.users.joe"), 0.0f);
+
+    assertEquals("root.admins.bob user-limit-factor", -1.0f,
+            conf.getUserLimitFactor("root.admins.bob"), 0.0f);
+    assertEquals("root.admin.bob auto-queue-creation-v2.enabled", false,
+            conf.isAutoQueueCreationV2Enabled("root.admin.bob"));
   }
 
   @Test
@@ -184,110 +219,109 @@ public class TestFSConfigToCSConfigConverter {
 
     converter.convert(params);
 
-    Configuration conf = converter.getCapacitySchedulerConfig();
+    CapacitySchedulerConfiguration conf = converter.getCapacitySchedulerConfig();
 
     // -1.0 means disabled ==> 1.0 in CS
-    assertEquals("Default max-am-resource-percent", "1.0",
-        conf.get(CapacitySchedulerConfiguration.
-            MAXIMUM_APPLICATION_MASTERS_RESOURCE_PERCENT));
+    assertEquals("Default max-am-resource-percent", 1.0f,
+        conf.getMaximumApplicationMasterResourcePercent(), 0.0f);
 
-    // root.admins.bob -1.0 equals to the default -1.0
-    assertNull("root.admins.bob maximum-am-resource-percent should be null",
-        conf.get(PREFIX + "root.admins.bob.maximum-am-resource-percent"));
+    // root.admins.bob is unset,so falling back to the global value
+    assertEquals("root.admins.bob maximum-am-resource-percent", 1.0f,
+        conf.getMaximumApplicationMasterResourcePerQueuePercent("root.admins.bob"), 0.0f);
 
     // root.admins.alice 0.15 != -1.0
-    assertEquals("root.admins.alice max-am-resource-percent", "0.15",
-        conf.get(PREFIX + "root.admins.alice.maximum-am-resource-percent"));
+    assertEquals("root.admins.alice max-am-resource-percent", 0.15f,
+        conf.getMaximumApplicationMasterResourcePerQueuePercent("root.admins.alice"), 0.0f);
 
-    // root.users.joe is unset, inherits -1.0
-    assertNull("root.users.joe maximum-am-resource-percent should be null",
-        conf.get(PREFIX + "root.users.joe.maximum-am-resource-percent"));
+    // root.users.joe is unset,so falling back to the global value
+    assertEquals("root.users.joe maximum-am-resource-percent", 1.0f,
+        conf.getMaximumApplicationMasterResourcePerQueuePercent("root.users.joe"), 0.0f);
   }
 
   @Test
   public void testConvertACLs() throws Exception {
     converter.convert(config);
 
-    Configuration conf = converter.getCapacitySchedulerConfig();
+    CapacitySchedulerConfiguration conf = converter.getCapacitySchedulerConfig();
 
     // root
     assertEquals("root submit ACL", "alice,bob,joe,john hadoop_users",
-        conf.get(PREFIX + "root.acl_submit_applications"));
+        conf.getAcl("root", QueueACL.SUBMIT_APPLICATIONS).getAclString());
     assertEquals("root admin ACL", "alice,bob,joe,john hadoop_users",
-        conf.get(PREFIX + "root.acl_administer_queue"));
+        conf.getAcl("root", QueueACL.ADMINISTER_QUEUE).getAclString());
 
     // root.admins.bob
     assertEquals("root.admins.bob submit ACL", "bob ",
-        conf.get(PREFIX + "root.admins.bob.acl_submit_applications"));
+        conf.getAcl("root.admins.bob", QueueACL.SUBMIT_APPLICATIONS).getAclString());
     assertEquals("root.admins.bob admin ACL", "bob ",
-        conf.get(PREFIX + "root.admins.bob.acl_administer_queue"));
+        conf.getAcl("root.admins.bob", QueueACL.ADMINISTER_QUEUE).getAclString());
 
     // root.admins.alice
     assertEquals("root.admins.alice submit ACL", "alice ",
-        conf.get(PREFIX + "root.admins.alice.acl_submit_applications"));
+        conf.getAcl("root.admins.alice", QueueACL.SUBMIT_APPLICATIONS).getAclString());
     assertEquals("root.admins.alice admin ACL", "alice ",
-        conf.get(PREFIX + "root.admins.alice.acl_administer_queue"));
+        conf.getAcl("root.admins.alice", QueueACL.ADMINISTER_QUEUE).getAclString());
 
     // root.users.john
-    assertEquals("root.users.john submit ACL", "john ",
-        conf.get(PREFIX + "root.users.john.acl_submit_applications"));
-    assertEquals("root.users.john admin ACL", "john ",
-        conf.get(PREFIX + "root.users.john.acl_administer_queue"));
+    assertEquals("root.users.john submit ACL", "*",
+        conf.getAcl("root.users.john", QueueACL.SUBMIT_APPLICATIONS).getAclString());
+    assertEquals("root.users.john admin ACL", "*",
+        conf.getAcl("root.users.john", QueueACL.ADMINISTER_QUEUE).getAclString());
 
     // root.users.joe
     assertEquals("root.users.joe submit ACL", "joe ",
-        conf.get(PREFIX + "root.users.joe.acl_submit_applications"));
+        conf.getAcl("root.users.joe", QueueACL.SUBMIT_APPLICATIONS).getAclString());
     assertEquals("root.users.joe admin ACL", "joe ",
-        conf.get(PREFIX + "root.users.joe.acl_administer_queue"));
+        conf.getAcl("root.users.joe", QueueACL.ADMINISTER_QUEUE).getAclString());
   }
 
   @Test
   public void testDefaultQueueMaxParallelApps() throws Exception {
     converter.convert(config);
 
-    Configuration conf = converter.getCapacitySchedulerConfig();
+    CapacitySchedulerConfiguration conf = converter.getCapacitySchedulerConfig();
 
     assertEquals("Default max parallel apps", 15,
-        conf.getInt(PREFIX + "max-parallel-apps", -1));
+        conf.getDefaultMaxParallelApps(), 0);
   }
 
   @Test
   public void testSpecificQueueMaxParallelApps() throws Exception {
     converter.convert(config);
 
-    Configuration conf = converter.getCapacitySchedulerConfig();
+    CapacitySchedulerConfiguration conf = converter.getCapacitySchedulerConfig();
 
     assertEquals("root.admins.alice max parallel apps", 2,
-        conf.getInt(PREFIX + "root.admins.alice.max-parallel-apps", -1));
+        conf.getMaxParallelAppsForQueue("root.admins.alice"), 0);
   }
 
   @Test
   public void testDefaultUserMaxParallelApps() throws Exception {
     converter.convert(config);
 
-    Configuration conf = converter.getCapacitySchedulerConfig();
-    int userMaxParallelApps =
-        conf.getInt(
-            PREFIX + "user.max-parallel-apps", -1);
+    CapacitySchedulerConfiguration conf = converter.getCapacitySchedulerConfig();
 
     assertEquals("Default user max parallel apps", 10,
-        userMaxParallelApps);
+        conf.getDefaultMaxParallelAppsPerUser(), 0);
   }
 
   @Test
   public void testSpecificUserMaxParallelApps() throws Exception {
     converter.convert(config);
 
-    Configuration conf = converter.getCapacitySchedulerConfig();
+    CapacitySchedulerConfiguration conf = converter.getCapacitySchedulerConfig();
 
     assertEquals("Max parallel apps for alice", 30,
-        conf.getInt(PREFIX + "user.alice.max-parallel-apps", -1));
-    assertNull("Max parallel apps should be undefined for user bob",
-        conf.get(PREFIX + "user.bob.max-parallel-apps"));
-    assertNull("Max parallel apps should be undefined for user joe",
-        conf.get(PREFIX + "user.joe.max-parallel-apps"));
-    assertNull("Max parallel apps should be undefined for user john",
-        conf.get(PREFIX + "user.john.max-parallel-apps"));
+        conf.getMaxParallelAppsForUser("alice"), 0);
+
+    //users.bob, user.joe, user.john  don’t have max-parallel-app set
+    // so falling back to the global value for .user to 10
+    assertEquals("Max parallel apps for user bob", 10,
+        conf.getMaxParallelAppsForUser("bob"), 0);
+    assertEquals("Max parallel apps for user joe", 10,
+        conf.getMaxParallelAppsForUser("joe"), 0);
+    assertEquals("Max parallel apps for user john", 10,
+        conf.getMaxParallelAppsForUser("john"), 0);
   }
 
   @Test
@@ -387,14 +421,8 @@ public class TestFSConfigToCSConfigConverter {
         ABORT, actions.get(MAX_CAPACITY_PERCENTAGE));
     assertEquals("maxChildCapacity",
         ABORT, actions.get(MAX_CHILD_CAPACITY));
-    assertEquals("userMaxRunningApps",
-        ABORT, actions.get(USER_MAX_RUNNING_APPS));
-    assertEquals("userMaxAppsDefault",
-        ABORT, actions.get(USER_MAX_APPS_DEFAULT));
     assertEquals("dynamicMaxAssign",
         ABORT, actions.get(DYNAMIC_MAX_ASSIGN));
-    assertEquals("specifiedNotFirstRule",
-        ABORT, actions.get(SPECIFIED_NOT_FIRST));
     assertEquals("reservationSystem",
         ABORT, actions.get(RESERVATION_SYSTEM));
     assertEquals("queueAutoCreate",
@@ -422,18 +450,24 @@ public class TestFSConfigToCSConfigConverter {
         WARNING, actions.get(MAX_CAPACITY_PERCENTAGE));
     assertEquals("maxChildCapacity",
         WARNING, actions.get(MAX_CHILD_CAPACITY));
-    assertEquals("userMaxRunningApps",
-        WARNING, actions.get(USER_MAX_RUNNING_APPS));
-    assertEquals("userMaxAppsDefault",
-        WARNING, actions.get(USER_MAX_APPS_DEFAULT));
     assertEquals("dynamicMaxAssign",
         WARNING, actions.get(DYNAMIC_MAX_ASSIGN));
-    assertEquals("specifiedNotFirstRule",
-        WARNING, actions.get(SPECIFIED_NOT_FIRST));
     assertEquals("reservationSystem",
         WARNING, actions.get(RESERVATION_SYSTEM));
     assertEquals("queueAutoCreate",
         WARNING, actions.get(QUEUE_AUTO_CREATE));
+    assertEquals("childStaticDynamicConflict",
+        WARNING, actions.get(CHILD_STATIC_DYNAMIC_CONFLICT));
+    assertEquals("parentChildCreateDiffers",
+        WARNING, actions.get(PARENT_CHILD_CREATE_DIFFERS));
+    assertEquals("fairAsDrf",
+        WARNING, actions.get(FAIR_AS_DRF));
+    assertEquals("maxResources",
+        WARNING, actions.get(MAX_RESOURCES));
+    assertEquals("minResources",
+        WARNING, actions.get(MIN_RESOURCES));
+    assertEquals("parentDynamicCreate",
+        WARNING, actions.get(PARENT_DYNAMIC_CREATE));
   }
 
   @Test
@@ -456,6 +490,8 @@ public class TestFSConfigToCSConfigConverter {
   public void testConvertCheckOutputDir() throws Exception {
     FSConfigToCSConfigConverterParams params = createDefaultParamsBuilder()
         .withClusterResource(CLUSTER_RESOURCE_STRING)
+        .withConvertPlacementRules(true)
+        .withPlacementRulesToFile(true)
         .build();
 
     converter.convert(params);
@@ -473,6 +509,11 @@ public class TestFSConfigToCSConfigConverter {
         "yarn-site.xml");
     assertTrue("Yarn site exists", yarnSiteFile.exists());
     assertTrue("Yarn site length > 0", yarnSiteFile.length() > 0);
+
+    File mappingRulesFile = new File(FSConfigConverterTestCommons.OUTPUT_DIR,
+        "mapping-rules.json");
+    assertTrue("Mapping rules file exists", mappingRulesFile.exists());
+    assertTrue("Mapping rules file length > 0", mappingRulesFile.length() > 0);
   }
 
   @Test
@@ -572,123 +613,52 @@ public class TestFSConfigToCSConfigConverter {
             CapacitySchedulerConfiguration.RESOURCE_CALCULATOR_CLASS, null));
   }
 
-  @SuppressWarnings("checkstyle:linelength")
-  public void testUserAsDefaultQueueWithPlacementRules() throws Exception {
-    config = new Configuration(false);
-    config.setBoolean(FairSchedulerConfiguration.MIGRATION_MODE, true);
-    config.set(FairSchedulerConfiguration.ALLOCATION_FILE,
-        FAIR_SCHEDULER_XML);
-
-    converter.convert(config);
-
-    Configuration convertedConf = converter.getCapacitySchedulerConfig();
-
-    String expectedMappingRules =
-        "u:%user:root.admins.devs.%user,u:%user:root.users.%user,u:%user:root.default";
-    String mappingRules =
-        convertedConf.get(CapacitySchedulerConfiguration.QUEUE_MAPPING);
-    assertEquals("Mapping rules", expectedMappingRules, mappingRules);
+  @Test
+  public void testUserAsDefaultQueueWithPlacementRules()
+      throws Exception {
+    testUserAsDefaultQueueAndPlacementRules(true);
   }
 
   @Test
-  public void testUserAsDefaultQueueTrueWithoutPlacementRules()
+  public void testUserAsDefaultQueueWithoutPlacementRules()
       throws Exception {
-    testUserAsDefaultQueueWithoutPlacementRules(true);
+    testUserAsDefaultQueueAndPlacementRules(false);
   }
 
-  @Test
-  public void testUserAsDefaultQueueFalseWithoutPlacementRules()
-      throws Exception {
-    testUserAsDefaultQueueWithoutPlacementRules(false);
-  }
-
-  private void testUserAsDefaultQueueWithoutPlacementRules(boolean
-      userAsDefaultQueue) throws Exception {
+  private void testUserAsDefaultQueueAndPlacementRules(
+      boolean hasPlacementRules) throws Exception {
     config = new Configuration(false);
     config.setBoolean(FairSchedulerConfiguration.MIGRATION_MODE, true);
-    config.set(FairSchedulerConfiguration.ALLOCATION_FILE,
-        FS_NO_PLACEMENT_RULES_XML);
-    config.setBoolean(FairSchedulerConfiguration.USER_AS_DEFAULT_QUEUE,
-        userAsDefaultQueue);
 
-    converter.setConvertPlacementRules(true);
-    converter.convert(config);
-
-    Configuration convertedConf = converter.getCapacitySchedulerConfig();
-    String mappingRules =
-        convertedConf.get(CapacitySchedulerConfiguration.QUEUE_MAPPING);
-
-    if (userAsDefaultQueue) {
-      assertEquals("Mapping rules", "u:%user:%user", mappingRules);
+    if (hasPlacementRules) {
+      config.set(FairSchedulerConfiguration.ALLOCATION_FILE,
+          FAIR_SCHEDULER_XML);
     } else {
-      assertEquals("Mapping rules", "u:%user:root.default", mappingRules);
+      config.set(FairSchedulerConfiguration.ALLOCATION_FILE,
+          FS_NO_PLACEMENT_RULES_XML);
     }
-  }
 
-  @Test
-  public void testAutoCreateChildQueuesWithPlacementRules() throws Exception {
-    config = new Configuration(false);
-    config.setBoolean(FairSchedulerConfiguration.MIGRATION_MODE, true);
-    config.set(FairSchedulerConfiguration.ALLOCATION_FILE,
-        FAIR_SCHEDULER_XML);
-    config.setBoolean(FairSchedulerConfiguration.ALLOW_UNDECLARED_POOLS,
+    config.setBoolean(FairSchedulerConfiguration.USER_AS_DEFAULT_QUEUE,
         true);
 
+    converter.setConvertPlacementRules(true);
+    converter.setConsoleMode(true);
     converter.convert(config);
+    String json = converter.getCapacitySchedulerConfig()
+        .get(CapacitySchedulerConfiguration.MAPPING_RULE_JSON);
 
-    Configuration convertedConf = converter.getCapacitySchedulerConfig();
-    String property =
-        "yarn.scheduler.capacity.root.auto-create-child-queue.enabled";
-    assertNull("Auto-create queue shouldn't be set",
-        convertedConf.get(property));
-  }
+    MappingRulesDescription description =
+        new ObjectMapper()
+          .reader()
+          .forType(MappingRulesDescription.class)
+          .readValue(json);
 
-  @Test
-  public void testAutoCreateChildQueuesTrueWithoutPlacementRules()
-      throws Exception {
-    testAutoCreateChildQueuesWithoutPlacementRules(true);
-  }
-
-  @Test
-  public void testAutoCreateChildQueuesFalseWithoutPlacementRules()
-      throws Exception {
-    testAutoCreateChildQueuesWithoutPlacementRules(false);
-  }
-
-  @SuppressWarnings("checkstyle:linelength")
-  private void testAutoCreateChildQueuesWithoutPlacementRules(
-      boolean allowUndeclaredPools) throws Exception {
-    config = new Configuration(false);
-    config.setBoolean(FairSchedulerConfiguration.MIGRATION_MODE, true);
-    config.set(FairSchedulerConfiguration.ALLOCATION_FILE,
-        FS_NO_PLACEMENT_RULES_XML);
-    config.setBoolean(FairSchedulerConfiguration.ALLOW_UNDECLARED_POOLS,
-        allowUndeclaredPools);
-
-    converter.convert(config);
-
-    Configuration convertedConf = converter.getCapacitySchedulerConfig();
-    String rootUserAutoCreate =
-        "yarn.scheduler.capacity.root.users.auto-create-child-queue.enabled";
-    String rootAutoCreate =
-        "yarn.scheduler.capacity.root.auto-create-child-queue.enabled";
-    String leafQueueAutoCreate =
-        "yarn.scheduler.capacity.root.users.joe.auto-create-child-queue.enabled";
-
-    if (allowUndeclaredPools) {
-      assertEquals("Auto-create queue wasn't enabled for root.users", true,
-          convertedConf.getBoolean(rootUserAutoCreate, false));
-      assertNull("Auto-create queue shouldn't be set for root",
-          convertedConf.get(rootAutoCreate));
-      assertNull("Auto-create queue shouldn't be set for leaf",
-          convertedConf.get(leafQueueAutoCreate));
+    if (hasPlacementRules) {
+      // fs.xml defines 5 rules
+      assertEquals("Number of rules", 5, description.getRules().size());
     } else {
-      assertNull("Auto-create queue shouldn't be set for root.users",
-          convertedConf.get(rootUserAutoCreate));
-      assertNull("Auto-create queue shouldn't be set for root",
-          convertedConf.get(rootAutoCreate));
-      assertNull("Auto-create queue shouldn't be set for leaf",
-          convertedConf.get(leafQueueAutoCreate));
+      // by default, FS internally creates 2 rules
+      assertEquals("Number of rules", 2, description.getRules().size());
     }
   }
 
@@ -720,7 +690,10 @@ public class TestFSConfigToCSConfigConverter {
     verify(placementConverter).convertPlacementPolicy(
         any(PlacementManager.class),
         any(FSConfigToCSConfigRuleHandler.class),
-        any(Boolean.class));
+        any(CapacitySchedulerConfiguration.class),
+        anyBoolean());
+    assertTrue(converter.getCapacitySchedulerConfig().getBoolean(
+        CapacitySchedulerConfiguration.ENABLE_QUEUE_MAPPING_OVERRIDE, false));
   }
 
   @Test
@@ -738,6 +711,20 @@ public class TestFSConfigToCSConfigConverter {
             schedulingEnabledValue);
   }
 
+  @Test
+  public void testSiteDisabledPreemptionWithObserveOnlyConversion()
+      throws Exception{
+    FSConfigToCSConfigConverterParams params = createDefaultParamsBuilder()
+        .withDisablePreemption(FSConfigToCSConfigConverterParams.
+            PreemptionMode.OBSERVE_ONLY)
+        .build();
+
+    converter.convert(params);
+    assertTrue("The observe only should be true",
+        converter.getCapacitySchedulerConfig().
+            getPreemptionObserveOnly());
+  }
+
   private boolean testConversionWithAsyncSchedulingOption(boolean enabled) throws Exception {
     FSConfigToCSConfigConverterParams params = createDefaultParamsBuilder()
             .withClusterResource(CLUSTER_RESOURCE_STRING)
