@@ -21,12 +21,13 @@ import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.conve
 import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigConverterTestCommons.OUTPUT_DIR;
 import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigConverterTestCommons.YARN_SITE_XML;
 import static org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.converter.FSConfigConverterTestCommons.setupFSConfigConversionFiles;
-import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
 
 import java.io.File;
 import java.io.IOException;
-import java.security.Permission;
+import java.util.Arrays;
+import java.util.function.Consumer;
 
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.QueueMetrics;
 import org.junit.After;
@@ -40,43 +41,67 @@ import org.junit.Test;
  */
 public class TestFSConfigToCSConfigConverterMain {
   private FSConfigConverterTestCommons converterTestCommons;
-  private SecurityManager originalSecurityManager;
-  private ExitHandlerSecurityManager exitHandlerSecurityManager;
+  private ExitFunc exitFunc;
 
   @Before
   public void setUp() throws Exception {
-    originalSecurityManager = System.getSecurityManager();
-    exitHandlerSecurityManager = new ExitHandlerSecurityManager();
-    System.setSecurityManager(exitHandlerSecurityManager);
+    exitFunc = new ExitFunc();
     converterTestCommons = new FSConfigConverterTestCommons();
     converterTestCommons.setUp();
+    FSConfigToCSConfigConverterMain.setExit(exitFunc);
   }
 
   @After
   public void tearDown() throws Exception {
     QueueMetrics.clearQueueMetrics();
-    System.setSecurityManager(originalSecurityManager);
+    FSConfigToCSConfigConverterMain.setExit(System::exit);
     converterTestCommons.tearDown();
   }
 
   /*
    * Example command:
-   *   opt/hadoop/bin/yarn fs2cs
+   *   /opt/hadoop/bin/yarn fs2cs
    *   -o /tmp/output
    *   -y /opt/hadoop/etc/hadoop/yarn-site.xml
    *   -f /opt/hadoop/etc/hadoop/fair-scheduler.xml
    *   -r /home/systest/sample-rules-config.properties
    */
   @Test
-  public void testConvertFSConfigurationDefaults()
+  public void testConvertFSConfigurationDefaultsWeightMode()
       throws Exception {
+    testConvertFSConfigurationDefaults(false);
+  }
+
+  /*
+   * Example command:
+   *   /opt/hadoop/bin/yarn fs2cs
+   *   -pc
+   *   -o /tmp/output
+   *   -y /opt/hadoop/etc/hadoop/yarn-site.xml
+   *   -f /opt/hadoop/etc/hadoop/fair-scheduler.xml
+   *   -r /home/systest/sample-rules-config.properties
+   */
+  @Test
+  public void testConvertFSConfigurationDefaultsPercentageMode()
+      throws IOException {
+    testConvertFSConfigurationDefaults(true);
+  }
+
+  private void testConvertFSConfigurationDefaults(boolean percentage)
+      throws IOException {
     setupFSConfigConversionFiles();
 
-    FSConfigToCSConfigConverterMain.main(new String[] {
+    String[] args = new String[] {
         "-o", OUTPUT_DIR,
         "-y", YARN_SITE_XML,
         "-f", FS_ALLOC_FILE,
-        "-r", CONVERSION_RULES_FILE});
+        "-r", CONVERSION_RULES_FILE};
+    if (percentage) {
+      args = Arrays.copyOf(args, args.length + 1);
+      args[args.length - 1] = "-pc";
+    }
+
+    FSConfigToCSConfigConverterMain.main(args);
 
     boolean csConfigExists =
         new File(OUTPUT_DIR, "capacity-scheduler.xml").exists();
@@ -85,7 +110,7 @@ public class TestFSConfigToCSConfigConverterMain {
 
     assertTrue("capacity-scheduler.xml was not generated", csConfigExists);
     assertTrue("yarn-site.xml was not generated", yarnSiteConfigExists);
-    assertEquals("Exit code", 0, exitHandlerSecurityManager.exitCode);
+    assertEquals("Exit code", 0, exitFunc.exitCode);
   }
 
   @Test
@@ -95,6 +120,7 @@ public class TestFSConfigToCSConfigConverterMain {
 
     FSConfigToCSConfigConverterMain.main(new String[] {
         "-p",
+        "-e",
         "-y", YARN_SITE_XML,
         "-f", FS_ALLOC_FILE,
         "-r", CONVERSION_RULES_FILE});
@@ -104,6 +130,9 @@ public class TestFSConfigToCSConfigConverterMain {
         stdout.contains("======= yarn-site.xml ======="));
     assertTrue("Stdout doesn't contain capacity-scheduler.xml",
         stdout.contains("======= capacity-scheduler.xml ======="));
+    assertTrue("Stdout doesn't contain mapping-rules.json",
+        stdout.contains("======= mapping-rules.json ======="));
+    assertEquals("Exit code", 0, exitFunc.exitCode);
   }
 
   @Test
@@ -111,6 +140,7 @@ public class TestFSConfigToCSConfigConverterMain {
     FSConfigToCSConfigConverterMain.main(new String[] {"-h"});
 
     verifyHelpText();
+    assertEquals("Exit code", 0, exitFunc.exitCode);
   }
 
   @Test
@@ -118,6 +148,15 @@ public class TestFSConfigToCSConfigConverterMain {
     FSConfigToCSConfigConverterMain.main(new String[] {"--help"});
 
     verifyHelpText();
+    assertEquals("Exit code", 0, exitFunc.exitCode);
+  }
+
+  @Test
+  public void testHelpDisplayedWithoutArgs() {
+    FSConfigToCSConfigConverterMain.main(new String[] {});
+
+    verifyHelpText();
+    assertEquals("Exit code", 0, exitFunc.exitCode);
   }
 
   @Test
@@ -127,6 +166,8 @@ public class TestFSConfigToCSConfigConverterMain {
 
     FSConfigToCSConfigConverterMain.main(new String[] {
         "--print",
+        "--rules-to-file",
+        "--percentage",
         "--yarnsiteconfig", YARN_SITE_XML,
         "--fsconfig", FS_ALLOC_FILE,
         "--rulesconfig", CONVERSION_RULES_FILE});
@@ -136,6 +177,18 @@ public class TestFSConfigToCSConfigConverterMain {
         stdout.contains("======= yarn-site.xml ======="));
     assertTrue("Stdout doesn't contain capacity-scheduler.xml",
         stdout.contains("======= capacity-scheduler.xml ======="));
+    assertTrue("Stdout doesn't contain mapping-rules.json",
+        stdout.contains("======= mapping-rules.json ======="));
+    assertEquals("Exit code", 0, exitFunc.exitCode);
+  }
+
+  @Test
+  public void testNegativeReturnValueOnError() {
+    FSConfigToCSConfigConverterMain.main(new String[] {
+        "--print",
+        "--yarnsiteconfig"});
+
+    assertEquals("Exit code", -1, exitFunc.exitCode);
   }
 
   private void verifyHelpText() {
@@ -144,21 +197,13 @@ public class TestFSConfigToCSConfigConverterMain {
         stdout.contains("General options are:"));
   }
 
-  class ExitHandlerSecurityManager extends SecurityManager {
-    int exitCode = Integer.MIN_VALUE;
+  @SuppressWarnings("checkstyle:visibilitymodifier")
+  class ExitFunc implements Consumer<Integer> {
+    int exitCode;
 
     @Override
-    public void checkExit(int status) {
-      if (status != 0) {
-        throw new IllegalStateException(
-            "Exit code is not 0, it was " + status);
-      }
-      exitCode = status;
-    }
-
-    @Override
-    public void checkPermission(Permission perm) {
-      // allow all permissions
+    public void accept(Integer t) {
+      this.exitCode = t.intValue();
     }
   }
 }
