@@ -18,6 +18,8 @@
 
 package org.apache.hadoop.yarn.server.router.clientrm;
 
+import static org.mockito.Mockito.mock;
+
 import java.io.IOException;
 import java.security.PrivilegedExceptionAction;
 import java.util.Collections;
@@ -26,6 +28,7 @@ import java.util.concurrent.Executors;
 
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.security.UserGroupInformation;
+import org.apache.hadoop.yarn.MockApps;
 import org.apache.hadoop.yarn.api.protocolrecords.CancelDelegationTokenRequest;
 import org.apache.hadoop.yarn.api.protocolrecords.CancelDelegationTokenResponse;
 import org.apache.hadoop.yarn.api.protocolrecords.GetApplicationAttemptReportRequest;
@@ -78,6 +81,8 @@ import org.apache.hadoop.yarn.api.records.ApplicationAttemptId;
 import org.apache.hadoop.yarn.api.records.ApplicationId;
 import org.apache.hadoop.yarn.api.records.ApplicationSubmissionContext;
 import org.apache.hadoop.yarn.api.records.ContainerId;
+import org.apache.hadoop.yarn.api.records.ContainerLaunchContext;
+import org.apache.hadoop.yarn.api.records.Priority;
 import org.apache.hadoop.yarn.api.records.ReservationDefinition;
 import org.apache.hadoop.yarn.api.records.ReservationId;
 import org.apache.hadoop.yarn.api.records.ReservationRequest;
@@ -88,8 +93,13 @@ import org.apache.hadoop.yarn.api.records.Token;
 import org.apache.hadoop.yarn.conf.YarnConfiguration;
 import org.apache.hadoop.yarn.event.AsyncDispatcher;
 import org.apache.hadoop.yarn.exceptions.YarnException;
+import org.apache.hadoop.yarn.server.resourcemanager.reservation.ReservationSystemTestUtil;
+import org.apache.hadoop.yarn.server.resourcemanager.scheduler.ResourceScheduler;
+import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.CapacityScheduler;
+import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.CapacitySchedulerConfiguration;
 import org.apache.hadoop.yarn.util.Clock;
 import org.apache.hadoop.yarn.util.UTCClock;
+import org.apache.hadoop.yarn.util.resource.Resources;
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
@@ -119,12 +129,12 @@ public abstract class BaseRouterClientRMTest {
     return this.clientrmService;
   }
 
-  protected YarnConfiguration createConfiguration() {
+  protected Configuration createConfiguration() {
     YarnConfiguration config = new YarnConfiguration();
     String mockPassThroughInterceptorClass =
         PassThroughClientRequestInterceptor.class.getName();
 
-    // Create a request intercepter pipeline for testing. The last one in the
+    // Create a request interceptor pipeline for testing. The last one in the
     // chain will call the mock resource manager. The others in the chain will
     // simply forward it to the next one in the chain
     config.set(YarnConfiguration.ROUTER_CLIENTRM_INTERCEPTOR_CLASS_PIPELINE,
@@ -134,11 +144,18 @@ public abstract class BaseRouterClientRMTest {
 
     config.setInt(YarnConfiguration.ROUTER_PIPELINE_CACHE_MAX_SIZE,
         TEST_MAX_CACHE_SIZE);
-    return config;
+    CapacitySchedulerConfiguration schedulerConf =
+        new CapacitySchedulerConfiguration(config);
+    ReservationSystemTestUtil.setupQueueConfiguration(schedulerConf);
+    schedulerConf.setClass(YarnConfiguration.RM_SCHEDULER,
+        CapacityScheduler.class, ResourceScheduler.class);
+    schedulerConf.setBoolean(YarnConfiguration.RM_RESERVATION_SYSTEM_ENABLE,
+        true);
+    return schedulerConf;
   }
 
   @Before
-  public void setUp() {
+  public void setUp() throws IOException {
     this.conf = createConfiguration();
     this.dispatcher = new AsyncDispatcher();
     this.dispatcher.init(conf);
@@ -205,13 +222,19 @@ public abstract class BaseRouterClientRMTest {
         .doAs(new PrivilegedExceptionAction<SubmitApplicationResponse>() {
           @Override
           public SubmitApplicationResponse run() throws Exception {
-            ApplicationSubmissionContext context =
-                ApplicationSubmissionContext.newInstance(appId, "", "", null,
-                    null, false, false, -1, null, null);
-            SubmitApplicationRequest req =
-                SubmitApplicationRequest.newInstance(context);
-            SubmitApplicationResponse response =
-                getRouterClientRMService().submitApplication(req);
+            ContainerLaunchContext amContainerSpec = mock(
+                ContainerLaunchContext.class);
+            ApplicationSubmissionContext context = ApplicationSubmissionContext
+                .newInstance(appId, MockApps.newAppName(), "q1",
+                    Priority.newInstance(0), amContainerSpec, false, false, -1,
+                    Resources.createResource(
+                        YarnConfiguration.
+                        DEFAULT_RM_SCHEDULER_MINIMUM_ALLOCATION_MB),
+                    "MockApp");
+            SubmitApplicationRequest req = SubmitApplicationRequest
+                .newInstance(context);
+            SubmitApplicationResponse response = getRouterClientRMService()
+                .submitApplication(req);
             return response;
           }
         });
@@ -334,9 +357,9 @@ public abstract class BaseRouterClientRMTest {
             long arrival = clock.getTime();
             long duration = 60000;
             long deadline = (long) (arrival + 1.05 * duration);
-
-            ReservationSubmissionRequest req = createSimpleReservationRequest(1,
-                arrival, deadline, duration, reservationId);
+            ReservationSubmissionRequest req = ReservationSystemTestUtil
+                .createSimpleReservationRequest(reservationId, 1, arrival,
+                    deadline, duration);
             ReservationSubmissionResponse response =
                 getRouterClientRMService().submitReservation(req);
             return response;
