@@ -32,6 +32,7 @@ import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.security.authorize.AuthorizationException;
 import org.apache.hadoop.yarn.conf.YarnConfiguration;
 import org.apache.hadoop.yarn.exceptions.YarnException;
+import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.CapacitySchedulerConfiguration;
 import org.apache.hadoop.yarn.server.resourcemanager.webapp.dao.ActivitiesInfo;
 import org.apache.hadoop.yarn.server.resourcemanager.webapp.dao.AppActivitiesInfo;
 import org.apache.hadoop.yarn.server.resourcemanager.webapp.dao.AppAttemptsInfo;
@@ -50,6 +51,7 @@ import org.apache.hadoop.yarn.server.resourcemanager.webapp.dao.NodeInfo;
 import org.apache.hadoop.yarn.server.resourcemanager.webapp.dao.NodeLabelsInfo;
 import org.apache.hadoop.yarn.server.resourcemanager.webapp.dao.NodeToLabelsInfo;
 import org.apache.hadoop.yarn.server.resourcemanager.webapp.dao.NodesInfo;
+import org.apache.hadoop.yarn.server.resourcemanager.webapp.dao.BulkActivitiesInfo;
 import org.apache.hadoop.yarn.server.resourcemanager.webapp.dao.SchedulerTypeInfo;
 import org.apache.hadoop.yarn.server.router.Router;
 import org.apache.hadoop.yarn.server.router.webapp.RouterWebServices.RequestInterceptorChainWrapper;
@@ -73,10 +75,18 @@ public abstract class BaseRouterWebServicesTest {
   private Router router;
   public final static int TEST_MAX_CACHE_SIZE = 10;
 
+  public static final String QUEUE_DEFAULT = "default";
+  public static final String QUEUE_DEFAULT_FULL = CapacitySchedulerConfiguration.ROOT +
+      CapacitySchedulerConfiguration.DOT + QUEUE_DEFAULT;
+  public static final String QUEUE_DEDICATED = "dedicated";
+  public static final String QUEUE_DEDICATED_FULL = CapacitySchedulerConfiguration.ROOT +
+      CapacitySchedulerConfiguration.DOT + QUEUE_DEDICATED;
+
   private RouterWebServices routerWebService;
 
   @Before
-  public void setUp() {
+  public void setUp() throws YarnException, IOException {
+
     this.conf = createConfiguration();
 
     router = spy(new Router());
@@ -93,7 +103,7 @@ public abstract class BaseRouterWebServicesTest {
     String mockPassThroughInterceptorClass =
         PassThroughRESTRequestInterceptor.class.getName();
 
-    // Create a request intercepter pipeline for testing. The last one in the
+    // Create a request interceptor pipeline for testing. The last one in the
     // chain will call the mock resource manager. The others in the chain will
     // simply forward it to the next one in the chain
     config.set(YarnConfiguration.ROUTER_WEBAPP_INTERCEPTOR_CLASS_PIPELINE,
@@ -179,6 +189,12 @@ public abstract class BaseRouterWebServicesTest {
         createHttpServletRequest(user), null, null);
   }
 
+  protected BulkActivitiesInfo getBulkActivities(String user)
+      throws InterruptedException {
+    return routerWebService.getBulkActivities(
+        createHttpServletRequest(user), null, 0);
+  }
+
   protected AppActivitiesInfo getAppActivities(String user)
       throws IOException, InterruptedException {
     return routerWebService.getAppActivities(createHttpServletRequest(user),
@@ -238,8 +254,8 @@ public abstract class BaseRouterWebServicesTest {
         null, createHttpServletRequest(user));
   }
 
-  protected Response removeFromCluserNodeLabels(String user) throws Exception {
-    return routerWebService.removeFromCluserNodeLabels(
+  protected Response removeFromClusterNodeLabels(String user) throws Exception {
+    return routerWebService.removeFromClusterNodeLabels(
         null, createHttpServletRequest(user));
   }
 
@@ -388,4 +404,16 @@ public abstract class BaseRouterWebServicesTest {
     when(request.getRemoteUser()).thenReturn(user);
     return request;
   }
+
+  protected Response updateSchedulerConfiguration(String user)
+      throws IOException, InterruptedException {
+    return routerWebService.updateSchedulerConfiguration(null,
+        createHttpServletRequest(user));
+  }
+
+  protected Response getSchedulerConfiguration(String user)
+      throws IOException, InterruptedException {
+    return routerWebService.
+        getSchedulerConfiguration(createHttpServletRequest(user));
+  }
 }
