@@ -41,8 +41,10 @@ import org.apache.hadoop.security.token.Token;
 import org.apache.hadoop.net.ServerSocketUtil;
 import org.apache.hadoop.service.AbstractService;
 import org.apache.hadoop.service.CompositeService;
+import org.apache.hadoop.test.GenericTestUtils;
 import org.apache.hadoop.util.Shell;
 import org.apache.hadoop.util.Shell.ShellCommandExecutor;
+import org.apache.hadoop.util.Time;
 import org.apache.hadoop.yarn.api.protocolrecords.GetClusterMetricsRequest;
 import org.apache.hadoop.yarn.api.records.ApplicationAttemptId;
 import org.apache.hadoop.yarn.conf.HAUtil;
@@ -97,7 +99,7 @@ import org.apache.hadoop.yarn.util.resource.ResourceUtils;
 import org.apache.hadoop.yarn.util.timeline.TimelineUtils;
 import org.apache.hadoop.yarn.webapp.util.WebAppUtils;
 
-import com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.classification.VisibleForTesting;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -143,6 +145,7 @@ public class MiniYARNCluster extends CompositeService {
   private boolean useFixedPorts;
   private boolean useRpc = false;
   private int failoverTimeout;
+  private boolean isNMWebEnabled = true;
 
   private ConcurrentMap<ApplicationAttemptId, Long> appMasters =
       new ConcurrentHashMap<ApplicationAttemptId, Long>(16, 0.75f, 2);
@@ -171,8 +174,11 @@ public class MiniYARNCluster extends CompositeService {
     this.numLocalDirs = numLocalDirs;
     this.numLogDirs = numLogDirs;
     this.enableAHS = enableAHS;
-    String testSubDir = testName.replace("$", "");
-    File targetWorkDir = new File("target", testSubDir);
+    String yarnFolderName = String.format("yarn-%d", Time.monotonicNow());
+    File targetWorkDirRoot = GenericTestUtils.getTestDir(getName());
+    // make sure that the folder exists
+    targetWorkDirRoot.mkdirs();
+    File targetWorkDir = new File(targetWorkDirRoot, yarnFolderName);
     try {
       FileContext.getLocalFSFileContext().delete(
           new Path(targetWorkDir.getAbsolutePath()), true);
@@ -227,6 +233,7 @@ public class MiniYARNCluster extends CompositeService {
    * @param numLocalDirs the number of nm-local-dirs per nodemanager
    * @param numLogDirs the number of nm-log-dirs per nodemanager
    */
+  @SuppressWarnings("deprecation")
   public MiniYARNCluster(
       String testName, int numResourceManagers, int numNodeManagers,
       int numLocalDirs, int numLogDirs) {
@@ -245,6 +252,21 @@ public class MiniYARNCluster extends CompositeService {
     this(testName, 1, numNodeManagers, numLocalDirs, numLogDirs);
   }
 
+  /**
+   * Constructor of MiniYARNCluster.
+   *
+   * @param testName name of the test
+   * @param numNodeManagers the number of node managers in the cluster
+   * @param numLocalDirs the number of nm-local-dirs per nodemanager
+   * @param numLogDirs the number of nm-log-dirs per nodemanager
+   * @param nMWebEnabled Whether to enable the WebNM page
+   */
+  public MiniYARNCluster(String testName, int numNodeManagers,
+      int numLocalDirs, int numLogDirs, boolean nMWebEnabled) {
+    this(testName, 1, numNodeManagers, numLocalDirs, numLogDirs);
+    isNMWebEnabled = nMWebEnabled;
+  }
+
   @Override
   public void serviceInit(Configuration conf) throws Exception {
     useFixedPorts = conf.getBoolean(
@@ -613,6 +635,9 @@ public class MiniYARNCluster extends CompositeService {
     }
 
     protected synchronized void serviceStart() throws Exception {
+      if (!isNMWebEnabled) {
+        nodeManagers[index].disableWebServer();
+      }
       nodeManagers[index].start();
       if (nodeManagers[index].getServiceState() != STATE.STARTED) {
         // NM could have failed.
@@ -896,8 +921,8 @@ public class MiniYARNCluster extends CompositeService {
         LOG.info("CustomAMRMProxyService is enabled. "
             + "All the AM->RM requests will be intercepted by the proxy");
         AMRMProxyService amrmProxyService =
-            useRpc ? new AMRMProxyService(getContext(), dispatcher)
-                : new ShortCircuitedAMRMProxy(getContext(), dispatcher);
+            useRpc ? new AMRMProxyService(getContext(), getDispatcher())
+                : new ShortCircuitedAMRMProxy(getContext(), getDispatcher());
         this.setAMRMProxyService(amrmProxyService);
         addService(this.getAMRMProxyService());
       } else {
@@ -928,8 +953,8 @@ public class MiniYARNCluster extends CompositeService {
         LOG.info("CustomAMRMProxyService is enabled. "
             + "All the AM->RM requests will be intercepted by the proxy");
         AMRMProxyService amrmProxyService =
-            useRpc ? new AMRMProxyService(getContext(), dispatcher)
-                : new ShortCircuitedAMRMProxy(getContext(), dispatcher);
+            useRpc ? new AMRMProxyService(getContext(), getDispatcher())
+                : new ShortCircuitedAMRMProxy(getContext(), getDispatcher());
         this.setAMRMProxyService(amrmProxyService);
         addService(this.getAMRMProxyService());
       } else {
@@ -940,7 +965,7 @@ public class MiniYARNCluster extends CompositeService {
     @Override
     protected ContainersMonitor createContainersMonitor(ContainerExecutor
         exec) {
-      return new ContainersMonitorImpl(exec, dispatcher, this.context) {
+      return new ContainersMonitorImpl(exec, getDispatcher(), this.context) {
         @Override
         public float getVmemRatio() {
           return 2.0f;
