@@ -175,7 +175,7 @@ public class TimelineCollectorWebService {
               NodeTimelineCollectorManager.COLLECTOR_MANAGER_ATTR_KEY);
       TimelineCollector collector = collectorManager.get(appID);
       if (collector == null) {
-        LOG.error("Application: "+ appId + " is not found");
+        LOG.error("Application: {} is not found", appId);
         throw new NotFoundException("Application: "+ appId + " is not found");
       }
 
@@ -196,6 +196,10 @@ public class TimelineCollectorWebService {
       LOG.error("Error putting entities", e);
       throw new WebApplicationException(e,
           Response.Status.INTERNAL_SERVER_ERROR);
+    } catch (Exception e) {
+      LOG.error("Unexpected error while putting entities", e);
+      throw new WebApplicationException(e,
+          Response.Status.INTERNAL_SERVER_ERROR);
     } finally {
       long latency = Time.monotonicNow() - startTime;
       if (isAsync) {
@@ -240,7 +244,7 @@ public class TimelineCollectorWebService {
               NodeTimelineCollectorManager.COLLECTOR_MANAGER_ATTR_KEY);
       TimelineCollector collector = collectorManager.get(appID);
       if (collector == null) {
-        LOG.error("Application: " + appId + " is not found");
+        LOG.error("Application: {} is not found", appId);
         throw new NotFoundException("Application: " + appId + " is not found");
       }
 
@@ -266,7 +270,7 @@ public class TimelineCollectorWebService {
         return null;
       }
     } catch (IllegalFormatException e) {
-      LOG.error("Invalid application ID: " + appId);
+      LOG.error("Invalid application ID: {}", appId);
       return null;
     }
   }
