@@ -50,7 +50,7 @@ import org.apache.hadoop.yarn.webapp.GenericExceptionHandler;
 import org.apache.hadoop.yarn.webapp.YarnJacksonJaxbJsonProvider;
 import org.apache.hadoop.yarn.webapp.util.WebAppUtils;
 
-import com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.classification.VisibleForTesting;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -102,7 +102,7 @@ public class TimelineReaderServer extends CompositeService {
     String timelineReaderClassName = conf.get(
         YarnConfiguration.TIMELINE_SERVICE_READER_CLASS,
         YarnConfiguration.DEFAULT_TIMELINE_SERVICE_READER_CLASS);
-    LOG.info("Using store: " + timelineReaderClassName);
+    LOG.info("Using store: {}", timelineReaderClassName);
     try {
       Class<?> timelineReaderClazz = Class.forName(timelineReaderClassName);
       if (TimelineReader.class.isAssignableFrom(timelineReaderClazz)) {
@@ -192,7 +192,7 @@ public class TimelineReaderServer extends CompositeService {
     String bindAddress = WebAppUtils
         .getWebAppBindURL(conf, hostProperty, webAppURLWithoutScheme);
 
-    LOG.info("Instantiating TimelineReaderWebApp at " + bindAddress);
+    LOG.info("Instantiating TimelineReaderWebApp at {}", bindAddress);
     try {
 
       String httpScheme = WebAppUtils.getHttpSchemePrefix(conf);
