@@ -18,11 +18,6 @@
 
 package org.apache.hadoop.yarn.server.timelineservice.reader;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertTrue;
-
 import java.io.File;
 import java.io.IOException;
 import java.lang.reflect.UndeclaredThrowableException;
@@ -30,26 +25,8 @@ import java.net.HttpURLConnection;
 import java.net.URI;
 import java.net.URL;
 import java.util.Set;
-
 import javax.ws.rs.core.MediaType;
 
-import org.apache.commons.io.FileUtils;
-import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.yarn.api.records.timeline.TimelineAbout;
-import org.apache.hadoop.yarn.api.records.timelineservice.TimelineEntity;
-import org.apache.hadoop.yarn.api.records.timelineservice.TimelineEntityType;
-import org.apache.hadoop.yarn.conf.YarnConfiguration;
-import org.apache.hadoop.yarn.server.timelineservice.storage.FileSystemTimelineReaderImpl;
-import org.apache.hadoop.yarn.server.timelineservice.storage.TestFileSystemTimelineReaderImpl;
-import org.apache.hadoop.yarn.server.timelineservice.storage.TimelineReader;
-import org.apache.hadoop.yarn.webapp.YarnJacksonJaxbJsonProvider;
-import org.junit.After;
-import org.junit.AfterClass;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.BeforeClass;
-import org.junit.Test;
-
 import com.sun.jersey.api.client.Client;
 import com.sun.jersey.api.client.ClientResponse;
 import com.sun.jersey.api.client.ClientResponse.Status;
@@ -58,10 +35,31 @@ import com.sun.jersey.api.client.config.ClientConfig;
 import com.sun.jersey.api.client.config.DefaultClientConfig;
 import com.sun.jersey.client.urlconnection.HttpURLConnectionFactory;
 import com.sun.jersey.client.urlconnection.URLConnectionClientHandler;
+import org.junit.jupiter.api.AfterAll;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+
+import org.apache.commons.io.FileUtils;
+import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.http.JettyUtils;
+import org.apache.hadoop.yarn.api.records.timeline.TimelineAbout;
+import org.apache.hadoop.yarn.api.records.timeline.TimelineHealth;
+import org.apache.hadoop.yarn.api.records.timelineservice.TimelineEntity;
+import org.apache.hadoop.yarn.api.records.timelineservice.TimelineEntityType;
+import org.apache.hadoop.yarn.conf.YarnConfiguration;
+import org.apache.hadoop.yarn.server.timelineservice.storage.FileSystemTimelineReaderImpl;
+import org.apache.hadoop.yarn.server.timelineservice.storage.TestFileSystemTimelineReaderImpl;
+import org.apache.hadoop.yarn.server.timelineservice.storage.TimelineReader;
+import org.apache.hadoop.yarn.webapp.YarnJacksonJaxbJsonProvider;
+
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
 
-/**
- * Test TimelineReder Web Service REST API's using filesystem storage.
- */
 public class TestTimelineReaderWebServices {
 
   private static final String ROOT_DIR = new File("target",
@@ -70,17 +68,17 @@ public class TestTimelineReaderWebServices {
   private int serverPort;
   private TimelineReaderServer server;
 
-  @BeforeClass
+  @BeforeAll
   public static void setup() throws Exception {
     TestFileSystemTimelineReaderImpl.initializeDataDirectory(ROOT_DIR);
   }
 
-  @AfterClass
+  @AfterAll
   public static void tearDown() throws Exception {
     FileUtils.deleteDirectory(new File(ROOT_DIR));
   }
 
-  @Before
+  @BeforeEach
   public void init() throws Exception {
     try {
       Configuration config = new YarnConfiguration();
@@ -98,11 +96,11 @@ public class TestTimelineReaderWebServices {
       server.start();
       serverPort = server.getWebServerPort();
     } catch (Exception e) {
-      Assert.fail("Web server failed to start");
+      fail("Web server failed to start");
     }
   }
 
-  @After
+  @AfterEach
   public void stop() throws Exception {
     if (server != null) {
       server.stop();
@@ -122,7 +120,7 @@ public class TestTimelineReaderWebServices {
         client.resource(uri).accept(MediaType.APPLICATION_JSON)
         .type(MediaType.APPLICATION_JSON).get(ClientResponse.class);
     assertNotNull(resp);
-    assertEquals(resp.getClientResponseStatus().getStatusCode(),
+    assertEquals(resp.getStatusInfo().getStatusCode(),
         expectedStatus.getStatusCode());
   }
 
@@ -139,10 +137,11 @@ public class TestTimelineReaderWebServices {
         client.resource(uri).accept(MediaType.APPLICATION_JSON)
         .type(MediaType.APPLICATION_JSON).get(ClientResponse.class);
     if (resp == null ||
-        resp.getClientResponseStatus() != ClientResponse.Status.OK) {
+        resp.getStatusInfo().getStatusCode() !=
+            ClientResponse.Status.OK.getStatusCode()) {
       String msg = new String();
       if (resp != null) {
-        msg = String.valueOf(resp.getClientResponseStatus());
+        msg = String.valueOf(resp.getStatusInfo().getStatusCode());
       }
       throw new IOException("Incorrect response from timeline reader. " +
           "Status=" + msg);
@@ -165,32 +164,33 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testAbout() throws Exception {
+  void testAbout() throws Exception {
     URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/timeline/");
     Client client = createClient();
     try {
       ClientResponse resp = getResponse(client, uri);
       TimelineAbout about = resp.getEntity(TimelineAbout.class);
-      Assert.assertNotNull(about);
-      Assert.assertEquals("Timeline Reader API", about.getAbout());
+      assertNotNull(about);
+      assertEquals("Timeline Reader API", about.getAbout());
     } finally {
       client.destroy();
     }
   }
 
   @Test
-  public void testGetEntityDefaultView() throws Exception {
+  void testGetEntityDefaultView() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
           "timeline/clusters/cluster1/apps/app1/entities/app/id_1");
       ClientResponse resp = getResponse(client, uri);
       TimelineEntity entity = resp.getEntity(TimelineEntity.class);
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entity);
       assertEquals("id_1", entity.getId());
       assertEquals("app", entity.getType());
-      assertEquals((Long)1425016502000L, entity.getCreatedTime());
+      assertEquals((Long) 1425016502000L, entity.getCreatedTime());
       // Default view i.e. when no fields are specified, entity contains only
       // entity id, entity type and created time.
       assertEquals(0, entity.getConfigs().size());
@@ -201,7 +201,7 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testGetEntityWithUserAndFlowInfo() throws Exception {
+  void testGetEntityWithUserAndFlowInfo() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
@@ -209,18 +209,19 @@ public class TestTimelineReaderWebServices {
           "userid=user1&flowname=flow1&flowrunid=1");
       ClientResponse resp = getResponse(client, uri);
       TimelineEntity entity = resp.getEntity(TimelineEntity.class);
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entity);
       assertEquals("id_1", entity.getId());
       assertEquals("app", entity.getType());
-      assertEquals((Long)1425016502000L, entity.getCreatedTime());
+      assertEquals((Long) 1425016502000L, entity.getCreatedTime());
     } finally {
       client.destroy();
     }
   }
 
   @Test
-  public void testGetEntityCustomFields() throws Exception {
+  void testGetEntityCustomFields() throws Exception {
     Client client = createClient();
     try {
       // Fields are case insensitive.
@@ -229,14 +230,15 @@ public class TestTimelineReaderWebServices {
           "fields=CONFIGS,Metrics,info");
       ClientResponse resp = getResponse(client, uri);
       TimelineEntity entity = resp.getEntity(TimelineEntity.class);
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entity);
       assertEquals("id_1", entity.getId());
       assertEquals("app", entity.getType());
       assertEquals(3, entity.getConfigs().size());
       assertEquals(3, entity.getMetrics().size());
-      assertTrue("UID should be present",
-          entity.getInfo().containsKey(TimelineReaderUtils.UID_KEY));
+      assertTrue(entity.getInfo().containsKey(TimelineReaderUtils.UID_KEY),
+          "UID should be present");
       // Includes UID.
       assertEquals(3, entity.getInfo().size());
       // No events will be returned as events are not part of fields.
@@ -247,7 +249,7 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testGetEntityAllFields() throws Exception {
+  void testGetEntityAllFields() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
@@ -255,14 +257,15 @@ public class TestTimelineReaderWebServices {
           "fields=ALL");
       ClientResponse resp = getResponse(client, uri);
       TimelineEntity entity = resp.getEntity(TimelineEntity.class);
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entity);
       assertEquals("id_1", entity.getId());
       assertEquals("app", entity.getType());
       assertEquals(3, entity.getConfigs().size());
       assertEquals(3, entity.getMetrics().size());
-      assertTrue("UID should be present",
-          entity.getInfo().containsKey(TimelineReaderUtils.UID_KEY));
+      assertTrue(entity.getInfo().containsKey(TimelineReaderUtils.UID_KEY),
+          "UID should be present");
       // Includes UID.
       assertEquals(3, entity.getInfo().size());
       assertEquals(2, entity.getEvents().size());
@@ -272,7 +275,7 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testGetEntityNotPresent() throws Exception {
+  void testGetEntityNotPresent() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
@@ -284,14 +287,15 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testQueryWithoutCluster() throws Exception {
+  void testQueryWithoutCluster() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
           "timeline/apps/app1/entities/app/id_1");
       ClientResponse resp = getResponse(client, uri);
       TimelineEntity entity = resp.getEntity(TimelineEntity.class);
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entity);
       assertEquals("id_1", entity.getId());
       assertEquals("app", entity.getType());
@@ -300,8 +304,10 @@ public class TestTimelineReaderWebServices {
           "timeline/apps/app1/entities/app");
       resp = getResponse(client, uri);
       Set<TimelineEntity> entities =
-          resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+          resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+          });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(4, entities.size());
     } finally {
@@ -310,51 +316,57 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testGetEntities() throws Exception {
+  void testGetEntities() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
           "timeline/clusters/cluster1/apps/app1/entities/app");
       ClientResponse resp = getResponse(client, uri);
       Set<TimelineEntity> entities =
-          resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+          resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+          });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(4, entities.size());
-      assertTrue("Entities id_1, id_2, id_3 and id_4 should have been" +
-          " present in response",
-          entities.contains(newEntity("app", "id_1")) &&
+      assertTrue(entities.contains(newEntity("app", "id_1")) &&
           entities.contains(newEntity("app", "id_2")) &&
           entities.contains(newEntity("app", "id_3")) &&
-          entities.contains(newEntity("app", "id_4")));
+          entities.contains(newEntity("app", "id_4")),
+          "Entities id_1, id_2, id_3 and id_4 should have been" +
+              " present in response");
     } finally {
       client.destroy();
     }
   }
 
   @Test
-  public void testGetEntitiesWithLimit() throws Exception {
+  void testGetEntitiesWithLimit() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
           "timeline/clusters/cluster1/apps/app1/entities/app?limit=2");
       ClientResponse resp = getResponse(client, uri);
       Set<TimelineEntity> entities =
-          resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+          resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+          });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(2, entities.size());
       // Entities returned are based on most recent created time.
-      assertTrue("Entities with id_1 and id_4 should have been present " +
-          "in response based on entity created time.",
-          entities.contains(newEntity("app", "id_1")) &&
-          entities.contains(newEntity("app", "id_4")));
+      assertTrue(entities.contains(newEntity("app", "id_1")) &&
+          entities.contains(newEntity("app", "id_4")),
+          "Entities with id_1 and id_4 should have been present " +
+              "in response based on entity created time.");
 
       uri = URI.create("http://localhost:" + serverPort + "/ws/v2/timeline/" +
           "clusters/cluster1/apps/app1/entities/app?limit=3");
       resp = getResponse(client, uri);
-      entities = resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      entities = resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+      });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       // Even though 2 entities out of 4 have same created time, one entity
       // is left out due to limit
@@ -365,7 +377,7 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testGetEntitiesBasedOnCreatedTime() throws Exception {
+  void testGetEntitiesBasedOnCreatedTime() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
@@ -373,41 +385,47 @@ public class TestTimelineReaderWebServices {
           "createdtimestart=1425016502030&createdtimeend=1425016502060");
       ClientResponse resp = getResponse(client, uri);
       Set<TimelineEntity> entities =
-          resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+          resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+          });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(1, entities.size());
-      assertTrue("Entity with id_4 should have been present in response.",
-          entities.contains(newEntity("app", "id_4")));
+      assertTrue(entities.contains(newEntity("app", "id_4")),
+          "Entity with id_4 should have been present in response.");
 
       uri = URI.create("http://localhost:" + serverPort + "/ws/v2/timeline/" +
           "clusters/cluster1/apps/app1/entities/app?createdtimeend" +
           "=1425016502010");
       resp = getResponse(client, uri);
-      entities = resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      entities = resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+      });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(3, entities.size());
-      assertFalse("Entity with id_4 should not have been present in response.",
-          entities.contains(newEntity("app", "id_4")));
+      assertFalse(entities.contains(newEntity("app", "id_4")),
+          "Entity with id_4 should not have been present in response.");
 
       uri = URI.create("http://localhost:" + serverPort + "/ws/v2/timeline/" +
           "clusters/cluster1/apps/app1/entities/app?createdtimestart=" +
           "1425016502010");
       resp = getResponse(client, uri);
-      entities = resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      entities = resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+      });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(1, entities.size());
-      assertTrue("Entity with id_4 should have been present in response.",
-          entities.contains(newEntity("app", "id_4")));
+      assertTrue(entities.contains(newEntity("app", "id_4")),
+          "Entity with id_4 should have been present in response.");
     } finally {
       client.destroy();
     }
   }
 
   @Test
-  public void testGetEntitiesByRelations() throws Exception {
+  void testGetEntitiesByRelations() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
@@ -415,41 +433,47 @@ public class TestTimelineReaderWebServices {
           "flow:flow1");
       ClientResponse resp = getResponse(client, uri);
       Set<TimelineEntity> entities =
-          resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+          resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+          });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(1, entities.size());
-      assertTrue("Entity with id_1 should have been present in response.",
-          entities.contains(newEntity("app", "id_1")));
+      assertTrue(entities.contains(newEntity("app", "id_1")),
+          "Entity with id_1 should have been present in response.");
 
       uri = URI.create("http://localhost:" + serverPort + "/ws/v2/timeline/" +
           "clusters/cluster1/apps/app1/entities/app?isrelatedto=" +
           "type1:tid1_2,type2:tid2_1%60");
       resp = getResponse(client, uri);
-      entities = resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      entities = resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+      });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(1, entities.size());
-      assertTrue("Entity with id_1 should have been present in response.",
-          entities.contains(newEntity("app", "id_1")));
+      assertTrue(entities.contains(newEntity("app", "id_1")),
+          "Entity with id_1 should have been present in response.");
 
       uri = URI.create("http://localhost:" + serverPort + "/ws/v2/timeline/" +
           "clusters/cluster1/apps/app1/entities/app?isrelatedto=" +
           "type1:tid1_1:tid1_2,type2:tid2_1%60");
       resp = getResponse(client, uri);
-      entities = resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      entities = resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+      });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(1, entities.size());
-      assertTrue("Entity with id_1 should have been present in response.",
-          entities.contains(newEntity("app", "id_1")));
+      assertTrue(entities.contains(newEntity("app", "id_1")),
+          "Entity with id_1 should have been present in response.");
     } finally {
       client.destroy();
     }
   }
 
   @Test
-  public void testGetEntitiesByConfigFilters() throws Exception {
+  void testGetEntitiesByConfigFilters() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
@@ -457,19 +481,21 @@ public class TestTimelineReaderWebServices {
           "conffilters=config_1%20eq%20123%20AND%20config_3%20eq%20abc");
       ClientResponse resp = getResponse(client, uri);
       Set<TimelineEntity> entities =
-          resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+          resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+          });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(1, entities.size());
-      assertTrue("Entity with id_3 should have been present in response.",
-          entities.contains(newEntity("app", "id_3")));
+      assertTrue(entities.contains(newEntity("app", "id_3")),
+          "Entity with id_3 should have been present in response.");
     } finally {
       client.destroy();
     }
   }
 
   @Test
-  public void testGetEntitiesByInfoFilters() throws Exception {
+  void testGetEntitiesByInfoFilters() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
@@ -477,19 +503,21 @@ public class TestTimelineReaderWebServices {
           "infofilters=info2%20eq%203.5");
       ClientResponse resp = getResponse(client, uri);
       Set<TimelineEntity> entities =
-          resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+          resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+          });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(1, entities.size());
-      assertTrue("Entity with id_3 should have been present in response.",
-          entities.contains(newEntity("app", "id_3")));
+      assertTrue(entities.contains(newEntity("app", "id_3")),
+          "Entity with id_3 should have been present in response.");
     } finally {
       client.destroy();
     }
   }
 
   @Test
-  public void testGetEntitiesByMetricFilters() throws Exception {
+  void testGetEntitiesByMetricFilters() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
@@ -497,21 +525,23 @@ public class TestTimelineReaderWebServices {
           "metricfilters=metric3%20ge%200");
       ClientResponse resp = getResponse(client, uri);
       Set<TimelineEntity> entities =
-          resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+          resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+          });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(2, entities.size());
-      assertTrue("Entities with id_1 and id_2 should have been present" +
-          " in response.",
-          entities.contains(newEntity("app", "id_1")) &&
-          entities.contains(newEntity("app", "id_2")));
+      assertTrue(entities.contains(newEntity("app", "id_1")) &&
+          entities.contains(newEntity("app", "id_2")),
+          "Entities with id_1 and id_2 should have been present" +
+              " in response.");
     } finally {
       client.destroy();
     }
   }
 
   @Test
-  public void testGetEntitiesByEventFilters() throws Exception {
+  void testGetEntitiesByEventFilters() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
@@ -519,31 +549,35 @@ public class TestTimelineReaderWebServices {
           "eventfilters=event_2,event_4");
       ClientResponse resp = getResponse(client, uri);
       Set<TimelineEntity> entities =
-          resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+          resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+          });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(1, entities.size());
-      assertTrue("Entity with id_3 should have been present in response.",
-          entities.contains(newEntity("app", "id_3")));
+      assertTrue(entities.contains(newEntity("app", "id_3")),
+          "Entity with id_3 should have been present in response.");
     } finally {
       client.destroy();
     }
   }
 
   @Test
-  public void testGetEntitiesNoMatch() throws Exception {
+  void testGetEntitiesNoMatch() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
           "timeline/clusters/cluster1/apps/app1/entities/app?" +
-          "metricfilters=metric7%20ge%200&isrelatedto=type1:tid1_1:tid1_2,"+
+          "metricfilters=metric7%20ge%200&isrelatedto=type1:tid1_1:tid1_2," +
           "type2:tid2_1%60&relatesto=flow:flow1&eventfilters=event_2,event_4" +
           "&infofilters=info2%20eq%203.5&createdtimestart=1425016502030&" +
           "createdtimeend=1425016502060");
       ClientResponse resp = getResponse(client, uri);
       Set<TimelineEntity> entities =
-          resp.getEntity(new GenericType<Set<TimelineEntity>>(){});
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+          resp.getEntity(new GenericType<Set<TimelineEntity>>(){
+          });
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       assertEquals(0, entities.size());
     } finally {
@@ -552,7 +586,7 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testInvalidValuesHandling() throws Exception {
+  void testInvalidValuesHandling() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/" +
@@ -572,7 +606,7 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testGetAppAttempts() throws Exception {
+  void testGetAppAttempts() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/"
@@ -582,20 +616,21 @@ public class TestTimelineReaderWebServices {
       Set<TimelineEntity> entities =
           resp.getEntity(new GenericType<Set<TimelineEntity>>() {
           });
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       int totalEntities = entities.size();
       assertEquals(2, totalEntities);
       assertTrue(
-          "Entity with app-attempt-2 should have been present in response.",
           entities.contains(
               newEntity(TimelineEntityType.YARN_APPLICATION_ATTEMPT.toString(),
-                  "app-attempt-1")));
+                  "app-attempt-1")),
+          "Entity with app-attempt-2 should have been present in response.");
       assertTrue(
-          "Entity with app-attempt-2 should have been present in response.",
           entities.contains(
               newEntity(TimelineEntityType.YARN_APPLICATION_ATTEMPT.toString(),
-                  "app-attempt-2")));
+                  "app-attempt-2")),
+          "Entity with app-attempt-2 should have been present in response.");
 
       uri = URI.create("http://localhost:" + serverPort + "/ws/v2/"
           + "timeline/clusters/cluster1/apps/app1/appattempts");
@@ -607,15 +642,15 @@ public class TestTimelineReaderWebServices {
       int retrievedEntity = entities.size();
       assertEquals(2, retrievedEntity);
       assertTrue(
-          "Entity with app-attempt-2 should have been present in response.",
           entities.contains(
               newEntity(TimelineEntityType.YARN_APPLICATION_ATTEMPT.toString(),
-                  "app-attempt-1")));
+                  "app-attempt-1")),
+          "Entity with app-attempt-2 should have been present in response.");
       assertTrue(
-          "Entity with app-attempt-2 should have been present in response.",
           entities.contains(
               newEntity(TimelineEntityType.YARN_APPLICATION_ATTEMPT.toString(),
-                  "app-attempt-2")));
+                  "app-attempt-2")),
+          "Entity with app-attempt-2 should have been present in response.");
 
       assertEquals(totalEntities, retrievedEntity);
 
@@ -625,7 +660,7 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testGetAppAttempt() throws Exception {
+  void testGetAppAttempt() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/"
@@ -635,7 +670,8 @@ public class TestTimelineReaderWebServices {
       TimelineEntity entities1 =
           resp.getEntity(new GenericType<TimelineEntity>() {
           });
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities1);
 
       uri = URI.create("http://localhost:" + serverPort + "/ws/v2/"
@@ -655,7 +691,7 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testGetContainers() throws Exception {
+  void testGetContainers() throws Exception {
     Client client = createClient();
     try {
       // total 3 containers in a application.
@@ -665,22 +701,23 @@ public class TestTimelineReaderWebServices {
       Set<TimelineEntity> entities =
           resp.getEntity(new GenericType<Set<TimelineEntity>>() {
           });
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities);
       int totalEntities = entities.size();
       assertEquals(3, totalEntities);
       assertTrue(
-          "Entity with container_1_1 should have been present in response.",
           entities.contains(newEntity(
-              TimelineEntityType.YARN_CONTAINER.toString(), "container_1_1")));
+              TimelineEntityType.YARN_CONTAINER.toString(), "container_1_1")),
+          "Entity with container_1_1 should have been present in response.");
       assertTrue(
-          "Entity with container_2_1 should have been present in response.",
           entities.contains(newEntity(
-              TimelineEntityType.YARN_CONTAINER.toString(), "container_2_1")));
+              TimelineEntityType.YARN_CONTAINER.toString(), "container_2_1")),
+          "Entity with container_2_1 should have been present in response.");
       assertTrue(
-          "Entity with container_2_2 should have been present in response.",
           entities.contains(newEntity(
-              TimelineEntityType.YARN_CONTAINER.toString(), "container_2_2")));
+              TimelineEntityType.YARN_CONTAINER.toString(), "container_2_2")),
+          "Entity with container_2_2 should have been present in response.");
 
       // for app-attempt1 1 container has run
       uri = URI.create("http://localhost:" + serverPort + "/ws/v2/"
@@ -694,9 +731,9 @@ public class TestTimelineReaderWebServices {
       int retrievedEntity = entities.size();
       assertEquals(1, retrievedEntity);
       assertTrue(
-          "Entity with container_1_1 should have been present in response.",
           entities.contains(newEntity(
-              TimelineEntityType.YARN_CONTAINER.toString(), "container_1_1")));
+              TimelineEntityType.YARN_CONTAINER.toString(), "container_1_1")),
+          "Entity with container_1_1 should have been present in response.");
 
       // for app-attempt2 2 containers has run
       uri = URI.create("http://localhost:" + serverPort + "/ws/v2/"
@@ -710,13 +747,13 @@ public class TestTimelineReaderWebServices {
       retrievedEntity += entities.size();
       assertEquals(2, entities.size());
       assertTrue(
-          "Entity with container_2_1 should have been present in response.",
           entities.contains(newEntity(
-              TimelineEntityType.YARN_CONTAINER.toString(), "container_2_1")));
+              TimelineEntityType.YARN_CONTAINER.toString(), "container_2_1")),
+          "Entity with container_2_1 should have been present in response.");
       assertTrue(
-          "Entity with container_2_2 should have been present in response.",
           entities.contains(newEntity(
-              TimelineEntityType.YARN_CONTAINER.toString(), "container_2_2")));
+              TimelineEntityType.YARN_CONTAINER.toString(), "container_2_2")),
+          "Entity with container_2_2 should have been present in response.");
 
       assertEquals(totalEntities, retrievedEntity);
 
@@ -726,7 +763,7 @@ public class TestTimelineReaderWebServices {
   }
 
   @Test
-  public void testGetContainer() throws Exception {
+  void testGetContainer() throws Exception {
     Client client = createClient();
     try {
       URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/"
@@ -736,7 +773,8 @@ public class TestTimelineReaderWebServices {
       TimelineEntity entities1 =
           resp.getEntity(new GenericType<TimelineEntity>() {
           });
-      assertEquals(MediaType.APPLICATION_JSON_TYPE, resp.getType());
+      assertEquals(MediaType.APPLICATION_JSON + "; " + JettyUtils.UTF_8,
+          resp.getType().toString());
       assertNotNull(entities1);
 
       uri = URI.create("http://localhost:" + serverPort + "/ws/v2/"
@@ -754,4 +792,22 @@ public class TestTimelineReaderWebServices {
       client.destroy();
     }
   }
-}
+
+  @Test
+  void testHealthCheck() throws Exception {
+    Client client = createClient();
+    try {
+      URI uri = URI.create("http://localhost:" + serverPort + "/ws/v2/"
+          + "timeline/health");
+      ClientResponse resp = getResponse(client, uri);
+      TimelineHealth timelineHealth =
+          resp.getEntity(new GenericType<TimelineHealth>() {
+          });
+      assertEquals(200, resp.getStatus());
+      assertEquals(TimelineHealth.TimelineHealthStatus.RUNNING,
+          timelineHealth.getHealthStatus());
+    } finally {
+      client.destroy();
+    }
+  }
+}
\ No newline at end of file
