@@ -19,6 +19,9 @@
 import Ember from 'ember';
 
 export default Ember.Route.extend({
+  model() {
+    return {};
+  },
   afterModel(model/*, transition*/) {
     model.error_id = "error";
     model.isValidErrorCode = false;
