@@ -22,14 +22,23 @@ import AppAttemptMixin from 'yarn-ui/mixins/app-attempt';
 
 export default AbstractRoute.extend(AppAttemptMixin, {
   model(param, transition) {
-    transition.send('updateBreadcrumbs', param.app_id, param.service, [{text: 'Attempts'}]);
+    const {app_id} = this.paramsFor('yarn-app');
+    const {service} = param;
+
+    transition.send('updateBreadcrumbs', app_id, service, [{text: 'Attempts'}]);
     return Ember.RSVP.hash({
-      appId: param.app_id,
-      serviceName: param.service,
-      attempts: this.fetchAttemptListFromRMorATS(param.app_id, this.store)
+      appId: app_id,
+      serviceName: service,
+      attempts: this.fetchAttemptListFromRMorATS(app_id, this.store).catch(function() {
+        return Ember.A();
+      })
     });
   },
 
+  refresh() {
+    window.location.reload();
+  },
+
   unloadAll() {
     this.store.unloadAll('yarn-app-attempt');
     this.store.unloadAll('yarn-timeline-appattempt');
