@@ -48,6 +48,9 @@ export default DS.JSONAPISerializer.extend({
           finalStatus: payload.finalStatus,
           progress: payload.progress,
           applicationType: payload.applicationType,
+          applicationTags:
+              (payload.applicationTags && payload.applicationTags !== 'null')?
+                  payload.applicationTags : 'N/A',
           diagnostics: (payload.diagnostics && payload.diagnostics !== 'null')? payload.diagnostics : '',
           amHostHttpAddress: payload.amHostHttpAddress,
           masterNodeId: payload.masterNodeId,
