@@ -26,7 +26,7 @@ export default DS.JSONAPISerializer.extend({
       var children = [];
       if (payload.queues && payload.queues.queue) {
         payload.queues.queue.forEach(function(queue) {
-          children.push(queue.queueName);
+          children.push(queue.queuePath);
         });
       }
 
@@ -40,10 +40,10 @@ export default DS.JSONAPISerializer.extend({
           var maxAMResource = defaultPartitionResource.amLimit;
           includedData.push({
             type: "YarnUser",
-            id: u.username + "_" + payload.queueName,
+            id: u.username + "_" + payload.queuePath,
             attributes: {
               name: u.username,
-              queueName: payload.queueName,
+              queueName: payload.queuePath,
               usedMemoryMB: u.resourcesUsed.memory || 0,
               usedVCore: u.resourcesUsed.vCores || 0,
               maxMemoryMB: u.userResourceLimit.memory || 0,
@@ -60,7 +60,7 @@ export default DS.JSONAPISerializer.extend({
 
           relationshipUserData.push({
             type: "YarnUser",
-            id: u.username + "_" + payload.queueName,
+            id: u.username + "_" + payload.queuePath,
           });
         });
       }
@@ -92,6 +92,7 @@ export default DS.JSONAPISerializer.extend({
         type: primaryModelClass.modelName, // yarn-queue
         attributes: {
           name: payload.queueName,
+          queuePath: payload.queuePath,
           parent: payload.myParent,
           children: children,
           capacity: payload.capacity,
@@ -100,7 +101,11 @@ export default DS.JSONAPISerializer.extend({
           absCapacity: payload.absoluteCapacity,
           absMaxCapacity: payload.absoluteMaxCapacity,
           absUsedCapacity: payload.absoluteUsedCapacity,
+          weight: payload.weight,
+          normalizedWeight: payload.normalizedWeight,
+          creationMethod: payload.creationMethod,
           state: payload.state,
+          orderingPolicy: payload.orderingPolicyInfo,
           userLimit: payload.userLimit,
           userLimitFactor: payload.userLimitFactor,
           preemptionDisabled: payload.preemptionDisabled,
@@ -136,9 +141,9 @@ export default DS.JSONAPISerializer.extend({
       if (payload.queues && payload.queues.queue) {
         for (var i = 0; i < payload.queues.queue.length; i++) {
           var queue = payload.queues.queue[i];
-          queue.myParent = payload.queueName;
+          queue.myParent = payload.queuePath;
           var childResult = this.handleQueue(store, primaryModelClass, queue,
-            queue.queueName,
+            queue.queuePath,
             requestType);
 
           data = data.concat(childResult.data);
