@@ -33,6 +33,10 @@ module.exports = function(environment) {
       }
     },
 
+    moment: {
+      includeTimezone: 'all'
+    },
+
     APP: DEFAULT_CONFIG,
     contentSecurityPolicy: {
       'connect-src': "* 'self'",
