@@ -16,7 +16,7 @@
  * limitations under the License.
  */
 
-import Resolver from 'ember/resolver';
+import Resolver from 'ember-resolver';
 import config from '../../config/environment';
 
 var resolver = Resolver.create();
