@@ -20,13 +20,12 @@ import Ember from 'ember';
 import { initialize } from '../../../initializers/hosts';
 import { module, test } from 'qunit';
 
-var registry, application;
+var application;
 
 module('Unit | Initializer | hosts', {
   beforeEach: function() {
     Ember.run(function() {
       application = Ember.Application.create();
-      registry = application.registry;
       application.deferReadiness();
     });
   }
@@ -34,7 +33,7 @@ module('Unit | Initializer | hosts', {
 
 // Replace this with your real tests.
 test('it works', function(assert) {
-  initialize(registry, application);
+  initialize(application);
 
   // you would normally confirm the results of the initializer here
   assert.ok(true);
